<?php
/**
*
* @package phpBB Arcade
* @version $Id: class_admin.php 163 2012-12-30 22:29:13Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Class for handling phpBB Arcade
*/
class arcade_admin extends arcade
{
	/**
	* Set values for a game...
	*/
	function set_game_data($game_id, $config_name, $config_value, $cat_id = false, $old_cat_id = false, $game_scoretype = false, $old_game_scoretype = false)
	{
		global $db;

		if ($config_name == 'game_name')
		{
			$sql_ary = array(
				$config_name		=> $config_value,
				'game_name_clean'	=> utf8_clean_string(utf8_normalize_nfc($config_value)),
			);

			$sql = 'UPDATE ' . ARCADE_GAMES_TABLE. '
					SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
					WHERE game_id = ' . (int) $game_id;
			$db->sql_query($sql);

		}
		else
		{
			$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
					SET ' . $db->sql_escape($config_name) . " = '" . $db->sql_escape($config_value) . "'
					WHERE game_id = " . (int) $game_id;
			$db->sql_query($sql);
		}

		if ($cat_id && $old_cat_id && ($cat_id != $old_cat_id))
		{
			$this->set_last_play($cat_id);
			$this->sync('category', $cat_id);

			$this->set_last_play($old_cat_id);
			$this->sync('category', $old_cat_id);
		}

		if ($config_name == 'game_scoretype' && $game_scoretype != $old_game_scoretype)
		{
			$game_data = array(
				'game_id'			=> $game_id,
				'game_scoretype'	=> $config_value,
			);

			$this->update_highscore($game_data);
		}
	}

	/**
	* Adds game to database. Requires you to provide the game_scorevar and cat_id.
	* The rest of the info is gathered from the install file.
	*/
	function add_game($game_filename, $cat_id)
	{
		global $db, $phpbb_root_path, $phpEx, $file_functions, $arcade_config;

		@include($phpbb_root_path . $arcade_config['game_path'] .$game_filename.'/'.$game_filename.'.' . $phpEx);
		$game_name = utf8_normalize_nfc($game_data['game_name']);

		$games_name = array();
		$sql = 'SELECT game_name, game_swf
				FROM ' . ARCADE_GAMES_TABLE . '
				ORDER BY game_name';
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$games_name[$row['game_name']] = $row['game_swf'];
		}
		$db->sql_freeresult($result);

		// We have same duplicate gamename
		if (isset($games_name[$game_name]))
		{
			if ($game_data['game_swf'] == $games_name[$game_name])
			{
				return false;
			}

			$duplicate_games = 0;
			$temp_name = $game_name;
			while (isset($games_name[$game_name]))
			{
				$duplicate_games++;
				$game_name = $temp_name . ' (' . $duplicate_games . ')';
			}
		}

		$sql = 'SELECT MAX(game_order) AS max_order
				FROM ' . ARCADE_GAMES_TABLE;

		$result = $db->sql_query($sql);
		$max_order = (int) $db->sql_fetchfield('max_order');
		$db->sql_freeresult($result);

		$next_order = $max_order + 10;

		$game_insert = array(
			'game_order'		=> (int) $next_order,
			'game_image'		=> (string) utf8_normalize_nfc($game_data['game_image']),
			'game_desc'			=> (string) utf8_normalize_nfc($game_data['game_desc']),
			'game_control'		=> (!empty($game_data['game_control'])) ? (int) $game_data['game_control'] : 0,
			'game_control_desc'	=> (!empty($game_data['game_control_desc'])) ? (string) utf8_normalize_nfc($game_data['game_control_desc']) : '',
			'game_highscore'	=> 0,
			'game_highdate'		=> 0,
			'game_highuser'		=> 0,
			'game_name'			=> (string) $game_name,
			'game_name_clean'	=> (string) utf8_clean_string($game_name),
			'game_swf'			=> (string) utf8_normalize_nfc($game_data['game_swf']),
			'game_width'		=> (int) $game_data['game_width'],
			'game_height'		=> (int) $game_data['game_height'],
			'game_installdate'	=> time(),
			'game_filesize'		=> (int) $file_functions->filesize(array_filter(array($this->set_path((string) utf8_normalize_nfc($game_data['game_swf'])), $this->get_gamedata((string) utf8_normalize_nfc($game_data['game_swf']))))),
			'game_scorevar'		=> (string) utf8_normalize_nfc($game_data['game_scorevar']),
			'game_scoretype'	=> (int) $game_data['game_scoretype'],
			'game_type'			=> (int) $game_data['game_type'],
			'cat_id'			=> (int) $cat_id,
			'game_jackpot'		=> (float) $arcade_config['jackpot_Minimum'],
		);

		$db->sql_query('INSERT INTO ' . ARCADE_GAMES_TABLE . ' ' . $db->sql_build_array('INSERT', $game_insert));

		$game_insert['game_id'] = $db->sql_nextid();

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
				SET cat_games = cat_games + 1,
				cat_last_game_installdate = ' . time() . '
				WHERE cat_id = ' . (int) $cat_id;
		$db->sql_query($sql);

		// This will update any old game install files (1.0.x) to new game install file format (2.0.x)
		if (!$game_insert['game_type'] || !empty($game_data['game_files']) || !isset($game_data['game_control']))
		{
			Arcade_update_game_install_file($game_insert['game_id']);
		}

		return $game_insert;
	}

	function float_num(&$num, $negative = false)
	{
		$num = (float) $num;

		if ($negative && $num < 0)
		{
			$num = 0;
		}
	}

	/**
	* Delete a games files from the server,
	* this includes any extra files and/or folders
	*/
	function delete_files($game_ids, $game_info = false)
	{
		global $user, $phpbb_root_path, $file_functions, $arcade_config;

		$errors = array();

		// Check permissions dirs writable
		$check_dirs = array('games/', 'arcade/gamedata/', $arcade_config['game_path']);

		foreach ($check_dirs as $dir)
		{
			$path = $phpbb_root_path . $dir;

			if (!phpbb_is_writable($path))
			{
				$errors[] = sprintf($user->lang['ARCADE_ERROR_DIR_WRITABLE'], $dir);
			}
		}

		if (!$game_info)
		{
			if (!is_array($game_ids))
			{
				$game_ids = array((int) $game_ids);
			}

			$game_info = $this->get_game_data($game_ids);
		}

		foreach ($game_info as $key => $value)
		{
			$file_list = array();
			// Paths for the main and extra game folders
			$game_folder = $this->set_path($game_info[$key]['game_swf'], 'path');
			$gamedata = $this->get_gamedata($game_info[$key]['game_swf']);

			// First get file listing for main game folder
			if (file_exists($game_folder))
			{
				$file_list = array_merge($file_list, $file_functions->filelist($game_folder, '', false, true));
			}

			// Then get file listing for extra folder
			if ($gamedata)
			{
				$file_list = array_merge($file_list, $file_functions->filelist($gamedata, '', false, true));
			}

			// Now we also check the directory/file permission of each item in the list.
			$this->check_file_writable($file_list, $errors);

			// If there are no errors we finally remove the files.
			if (!sizeof($errors))
			{
				$file_functions->delete_list($file_list);
			}
			unset($file_list);
		}

		return $errors;
	}

	/**
	* Delete a plugin files from the server
	*/
	function delete_plugin_files($filename, $extra_files, &$errors)
	{
		global $phpbb_root_path, $file_functions;

		$file_list = array_merge($this->plugin_cool_files($filename, $extra_files), $file_functions->filelist("{$phpbb_root_path}arcade/xml/plugins/{$filename}/", '', false, true));

		$this->check_file_writable($file_list, $errors);

		if (!sizeof($errors))
		{
			$file_functions->delete_list($file_list);
		}

		unset($file_list);
	}

	function plugin_cool_files($filename, $extra_files, $defl = true, $reverse = false, $default_use = array())
	{
		global $arcade_plugin, $arcade_install, $phpEx;

		$file_list = $fl	= array();
		$installed_styles	= $arcade_install->installed_styles();
		$installed_langs	= $arcade_install->installed_languages();

		if ($defl && ((!$reverse) || ($reverse && sizeof($default_use))))
		{
			$plugin_files = array(
				$arcade_plugin->file_path($filename),
				$arcade_plugin->file_path($filename, 'install'),
				$arcade_plugin->file_path($filename, 'info')
			);

			if (!$reverse || ($reverse && $default_use['acp']))
			{
				$plugin_files[] = $arcade_plugin->file_path($filename, 'acp');
			}

			if (!$reverse || ($reverse && $default_use['language']))
			{
				$plugin_files[] = $arcade_plugin->file_path($filename, 'lang');
			}

			if (!$reverse || ($reverse && $default_use['template']))
			{
				$plugin_files[] = $arcade_plugin->file_path($filename . '.html', 'temp');
			}

			$this->prepare_files('default', $plugin_files, $installed_styles, $installed_langs, $file_list);
		}

		if (sizeof($extra_files))
		{
			$this->prepare_files('extra', $extra_files, $installed_styles, $installed_langs, $file_list);
		}

		$file_list = array_unique($file_list);

		foreach ($file_list as $file)
		{
			if (strpos($file, '{TEMP_PATH}') !== false || strpos($file, '{THEME_PATH}') !== false || strpos($file, '{LANG_PATH}') !== false)
			{
				continue;
			}

			$file .= (strrchr(basename($file), '.') !== false) ? '' : '.' . $phpEx;

			if ($reverse)
			{
				if (!file_exists($file))
				{
					$fl[] = $file;
				}
			}
			else
			{
				if (file_exists($file))
				{
					$fl[] = $file;
				}
			}
		}

		return $fl;
	}

	function prepare_files($mode, $files, $styles, $langs, &$file_list)
	{
		global $phpbb_root_path;

		foreach ($files as $file)
		{
			if ($mode == 'extra')
			{
				if (substr(basename($file), 0, 3) != 'ap_')
				{
					continue;
				}

				$file = $phpbb_root_path . $file;
			}

			foreach ($styles as $style)
			{
				$template_path = ($style['template_inherits_id'] && $style['template_inherit_path']) ? $style['template_inherit_path'] : $style['template_path'];

				$file_list[] = str_replace('{TEMP_PATH}', $template_path, $file);
				$file_list[] = str_replace('{THEME_PATH}', $style['theme_path'], $file);
			}

			foreach ($langs as $dir)
			{
				$file_list[] = str_replace('{LANG_PATH}', $dir, $file);
			}
		}
	}

	/**
	* Check game_extra files and/or folders
	* to make sure they are writable
	*/
	function check_file_writable($path, &$errors)
	{
		if (!sizeof($path))
		{
			return;
		}

		global $user;

		foreach ($path as $file)
		{
			if (!phpbb_is_writable($file))
			{
				$errors[] = sprintf($user->lang['ARCADE_DELETE_FILES_ERROR_' . ((is_dir($file)) ? 'DIR' : 'FILE')], $this->root_key($file));
			}
		}
	}

	function old_delete_games($method, $games_swf)
	{
		global $db, $file_functions;

		if (!is_array($games_swf))
		{
			$games_swf = array($games_swf);
		}

		switch ($method)
		{
			case 'verify':
				$sql = 'SELECT game_swf, uninstalldate FROM ' . ARCADE_DELETE_GAMES_TABLE . '
						WHERE ' . $db->sql_in_set('game_swf', $games_swf);
				$result = $db->sql_query($sql);
				$swf = array();
				while ($row = $db->sql_fetchrow($result))
				{
					$swf[$row['game_swf']] = $row['uninstalldate'];
				}
				$db->sql_freeresult($result);

				return $swf;
			break;

			case 'create':
				foreach ($games_swf as $game_swf)
				{
					$sql_ary = array(
						'game_swf'		=> $file_functions->remove_extension($game_swf),
						'uninstalldate'	=> time()
					);

					$db->sql_return_on_error(true);
					$db->sql_query($sql = 'INSERT INTO ' . ARCADE_DELETE_GAMES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
					$db->sql_return_on_error(false);
				}
			break;
		}
	}

	/**
	* Delete a game from the db and remove
	* its data from all relevant tables
	*/
	function delete_game($game_id, $cat_id, $delete_files = false, $game_swf, $resync = true)
	{
		global $db, $user;

		$errors = array();
		if (empty($game_id))
		{
			$errors[] = $user->lang['NO_GAME_ID'];
		}

		if (empty($cat_id))
		{
			$errors[] = $user->lang['NO_CAT_ID'];
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		if (!is_array($game_id))
		{
			$game_id = array((int) $game_id);
		}

		if ($delete_files)
		{
			$errors = $this->delete_files($game_id);
			// Since there are permission errors we don't delete files or db entries.
			// We will display message to user. They can either fix the permission errors
			// or not check off the option to remove the files.
			if (sizeof($errors))
			{
				array_unshift($errors, $user->lang['ARCADE_DELETE_FILES_ERROR_GAME']);
				return $errors;
			}
		}

		$tables = array(ARCADE_DOWNLOAD_TABLE, ARCADE_LOGS_TABLE, ARCADE_PLAYS_TABLE, ARCADE_SCORES_TABLE, ARCADE_GAMES_TABLE, ARCADE_FAVS_TABLE, ARCADE_RATING_TABLE, ARCADE_REPORTS_TABLE, ARCADE_SESSIONS_TABLE, ARCADE_SUPER_CHAMP_TABLE, ARCADE_CHALLENGE_TABLE, ARCADE_CHALLENGE_CHAMP_TABLE);

		foreach ($tables as $table)
		{
			$sql = "DELETE FROM $table
					WHERE " . $db->sql_in_set('game_id', $game_id);
			$db->sql_query($sql);
		}

		unset($tables);

		if ($resync)
		{
			$this->sync('total_data', 'all');
			$this->set_last_play($cat_id);
			$this->sync('category', $cat_id);
		}

		$this->old_delete_games('create', $game_swf);

		return $errors;
	}

	/**
	* Delete a user from the arcade db and remove
	* its data from all relevant tables
	*/
	function delete_user($user_id)
	{
		if (!$user_id)
		{
			return;
		}

		global $db;

		$tables = array(ACL_ARCADE_USERS_TABLE, ARCADE_USERS_BANNED_TABLE, ARCADE_USERS_TABLE, ARCADE_SESSIONS_TABLE, ARCADE_REPORTS_TABLE, ARCADE_LOGS_TABLE, ARCADE_ACCESS_TABLE, ARCADE_ANNOUNCE_TABLE);

		foreach ($tables as $table_name)
		{
			$sql = 'DELETE FROM ' . $table_name . '
					WHERE user_id = ' . (int) $user_id;
			$db->sql_query($sql);
		}

		$this->reset('user', $user_id);
		$this->reset('challenge', $user_id);
		$this->cache_purge('user');
		$this->sync('total_data', 'all');
	}
	/**
	* Toggle report opened or closed
	*/
	function report_toggle($action, $report_id)
	{
		global $db, $arcade_config;

		$sql = 'UPDATE ' . ARCADE_REPORTS_TABLE . '
				SET report_closed = ' . (($action == 'close') ? ARCADE_REPORT_CLOSE : ARCADE_REPORT_OPEN) . '
				WHERE report_id = ' . (int) $report_id;
		$db->sql_query($sql);

		if ($db->sql_affectedrows())
		{
			$this->set_config('reports_open', ($arcade_config['reports_open'] + (($action == 'close') ? -1 : 1)));
		}
	}

	/**
	* All-encompasing reset function
	*
	* Modes:
	* - user			Reset user data, must have passed $user_id
	* - user_scores		Reset user score data, must have passed $user_id
	* - jackpot_min_max	Reset games to jackpot minimum that are less than minimum and jackpot maximum that are higher than maximum.
	*/
	function reset($mode, $user_id = false)
	{
		if (!$user_id && $mode != 'jackpot_min_max')
		{
			return false;
		}

		global $db, $arcade_config;

		switch ($mode)
		{
			case 'user':
				$sql = 'SELECT game_id FROM ' . ARCADE_RATING_TABLE . '
						WHERE user_id = ' . (int) $user_id;
				$result = $db->sql_query($sql);
				$resync_ratings = array();
				while ($row = $db->sql_fetchrow($result))
				{
					$resync_ratings[] = $row['game_id'];
				}
				$db->sql_freeresult($result);

				$tables = array(ARCADE_FAVS_TABLE, ARCADE_RATING_TABLE, ARCADE_DOWNLOAD_TABLE, ARCADE_SUPER_CHAMP_TABLE);

				foreach ($tables as $table_name)
				{
					$sql = 'DELETE FROM ' . $table_name . '
							WHERE user_id = ' . (int) $user_id;
					$db->sql_query($sql);
				}

				if (sizeof($resync_ratings))
				{
					$this->sync('rating', $resync_ratings);
				}
			case 'user_scores':
				$resync_cats = $resync_games = array();

				$sql_array = array(
					'SELECT'	=> 'g.game_id, g.cat_id, g.game_scoretype, g.game_highuser',
					'FROM'		=> array(
						ARCADE_GAMES_TABLE	=> 'g',
					),
					'LEFT_JOIN'	=> array(
						array(
							'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
							'ON'	=> 'g.game_id = s.game_id'
						)
					),
					'WHERE' 	=> 's.user_id = ' . (int) $user_id,
				);

				$sql = $db->sql_build_query('SELECT', $sql_array);
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$resync_cats[] = $row['cat_id'];
					$resync_games[$row['game_id']] = array(
						'game_id'			=> $row['game_id'],
						'game_scoretype'	=> $row['game_scoretype'],
						'game_highuser'		=> $row['game_highuser'],
					);
				}
				$db->sql_freeresult($result);

				$tables = array(ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE);

				foreach ($tables as $table_name)
				{
					$sql = 'DELETE FROM ' . $table_name . '
							WHERE user_id = ' . (int) $user_id;
					$db->sql_query($sql);
				}

				if (sizeof($resync_cats))
				{
					$this->set_last_play($resync_cats);
					$this->sync('category', $resync_cats);
				}

				if (sizeof($resync_games))
				{
					$this->sync('game', array_keys($resync_games));
					foreach ($resync_games as $key => $value)
					{
						if ($user_id == $value['game_highuser'])
						{
							$this->update_highscore($value);
						}
					}
				}
			break;

			case 'jackpot_min_max':
				$game_ids_max = $game_ids_min = array();
				$sql_array = array(
					'SELECT'	=> 'g.game_id, g.game_jackpot',
					'FROM'		=> array(
						ARCADE_GAMES_TABLE	=> 'g',
					),
					'WHERE'		=> 'g.game_jackpot < ' . (float) $arcade_config['jackpot_Minimum'] . ' OR g.game_jackpot > ' . (float) $arcade_config['jackpot_Maximum'],
					'ORDER_BY'		=> 'g.game_id ASC',
				);

				$sql = $db->sql_build_query('SELECT', $sql_array);
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					if ($row['game_jackpot'] > $arcade_config['jackpot_Maximum'])
					{
						$game_ids_max[] = $row['game_id'];
					}
					else
					{
						$game_ids_min[] = $row['game_id'];
					}
				}
				$db->sql_freeresult($result);

				if (sizeof($game_ids_min))
				{
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
							SET game_jackpot = ' . (float) $arcade_config['jackpot_Minimum'] . '
							WHERE ' . $db->sql_in_set('game_id', $game_ids_min);
					$db->sql_query($sql);
				}

				if (sizeof($game_ids_max))
				{
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
							SET game_jackpot = ' . (float) $arcade_config['jackpot_Maximum'] . '
							WHERE ' . $db->sql_in_set('game_id', $game_ids_max);
					$db->sql_query($sql);
				}
			break;

			case 'challenge':
				$sql = 'DELETE FROM ' . ARCADE_CHALLENGE_TABLE . '
						WHERE challenger_id = ' . (int) $user_id . ' OR opponent_id = ' . (int) $user_id;
				$db->sql_query($sql);

				$sql = 'DELETE FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
						WHERE champ_challenger_id = ' . (int) $user_id . ' OR champ_opponent_id = ' . (int) $user_id;
				$db->sql_query($sql);
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}
	}

	/**
	* Move game to new category
	*/
	function move_game($game_id, $old_cat_id, $new_cat_id)
	{
		global $db, $user;

		$errors = array();
		if (empty($game_id))
		{
			$errors[] = $user->lang['NO_GAME_IDS'];
		}

		if (!$old_cat_id || !$new_cat_id)
		{
			$errors[] = $user->lang['NO_CAT_ID'];
		}

		if ($old_cat_id == $new_cat_id)
		{
			$errors[] = $user->lang['MOVE_SAME_CAT_ID'];
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		if (!is_array($game_id))
		{
			$game_id = array((int) $game_id);
		}

		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
				SET cat_id = ' . (int) $new_cat_id . '
				WHERE ' . $db->sql_in_set('game_id', $game_id);
		$db->sql_query($sql);

		$this->set_last_play($new_cat_id);
		$this->sync('category', $new_cat_id);

		$this->set_last_play($old_cat_id);
		$this->sync('category', $old_cat_id);

		return $errors;
	}

	/**
	* Reset game data
	*/
	function reset_game($action, $game_id, $cat_id)
	{
		global $db, $user, $arcade_config;

		$errors = array();
		if (!$game_id)
		{
			$errors[] = $user->lang['NO_GAME_ID'];
		}

		if (!$cat_id)
		{
			$errors[] = $user->lang['NO_CAT_ID'];
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		if (!is_array($game_id))
		{
			$game_id = array((int) $game_id);
		}

		$retain_plays = request_var('retain_plays', false);

		if (!$retain_plays)
		{
			$sql = 'DELETE FROM ' . ARCADE_PLAYS_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $game_id);
			$db->sql_query($sql);
		}

		$sql = 'DELETE FROM ' . ARCADE_SCORES_TABLE . '
				WHERE ' . $db->sql_in_set('game_id', $game_id);
		$db->sql_query($sql);

		$sql_ary = array(
			'game_highscore'=> 0,
			'game_highuser'	=> 0,
			'game_highdate'	=> 0
		);

		if (!$retain_plays)
		{
			$sql_ary['game_plays'] = 0;
		}

		if ($action == 'reset_install_date_marked' || $action == 'reset_game_install_date')
		{
			$action = 'reset_marked';

			$sql_ary['game_installdate'] = time();

			$sql = 'DELETE FROM ' . ARCADE_SUPER_CHAMP_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $game_id);
			$db->sql_query($sql);
		}

		if ($action == 'reset_marked')
		{
			$sql = 'DELETE FROM ' . ARCADE_RATING_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $game_id);
			$db->sql_query($sql);

			$sql = 'DELETE FROM ' . ARCADE_DOWNLOAD_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $game_id);
			$db->sql_query($sql);

			$sql_ary += array(
				'game_votetotal'		=> 0,
				'game_votesum'			=> 0,
				'game_download_total'	=> 0,
				'game_reward'			=> 0,
				'game_jackpot'			=> (float) $arcade_config['jackpot_Minimum'],
			);
		}

		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE. '
				SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE ' . $db->sql_in_set('game_id', $game_id);
		$db->sql_query($sql);

		if ($action != 'reset_game_install_date')
		{
			$this->set_last_play($cat_id);
			$this->sync('category', $cat_id);
		}

		return $errors;
	}

	/**
	* Copies permissions from one category to others
	*
	* @param int	$src_cat_id			The source category we want to copy permissions from
	* @param array	$dest_cat_ids		The destination category/categories we want to copy to
	* @param bool	$clear_dest_perms	True if destination permissions should be deleted
	* @param bool	$add_log			True if log entry should be added
	*
	* @return bool						False on error
	*
	* @author bantu
	*/
	function copy_category_permissions($src_cat_id, $dest_cat_ids, $clear_dest_perms = true, $add_log = true)
	{
		global $db;

		// Only one cat id specified
		if (!is_array($dest_cat_ids))
		{
			$dest_cat_ids = array($dest_cat_ids);
		}

		// Make sure cat ids are integers
		$src_cat_id = (int) $src_cat_id;
		$dest_cat_ids = array_map('intval', $dest_cat_ids);

		// No source category or no destination categories specified
		if (empty($src_cat_id) || empty($dest_cat_ids))
		{
			return false;
		}

		// Check if source cat exists
		$sql = 'SELECT cat_name
				FROM ' . ARCADE_CATS_TABLE . '
				WHERE cat_id = ' . $src_cat_id;
		$result = $db->sql_query($sql);
		$src_cat_name = $db->sql_fetchfield('cat_name');
		$db->sql_freeresult($result);

		// Source cat doesn't exist
		if (empty($src_cat_name))
		{
			return false;
		}

		// Check if destination cats exists
		$sql = 'SELECT cat_id, cat_name
				FROM ' . ARCADE_CATS_TABLE . '
				WHERE ' . $db->sql_in_set('cat_id', $dest_cat_ids);
		$result = $db->sql_query($sql);

		$dest_cat_ids = $dest_cat_names = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$dest_cat_ids[]		= (int) $row['cat_id'];
			$dest_cat_names[]	= $row['cat_name'];
		}
		$db->sql_freeresult($result);

		// No destination cat exists
		if (empty($dest_cat_ids))
		{
			return false;
		}

		// From the mysql documentation:
		// Prior to MySQL 4.0.14, the target table of the INSERT statement cannot appear
		// in the FROM clause of the SELECT part of the query. This limitation is lifted in 4.0.14.
		// Due to this we stay on the safe side if we do the insertion "the manual way"

		// Rowsets we're going to insert
		$users_sql_ary = $groups_sql_ary = array();

		// Query acl users table for source cat data
		$sql = 'SELECT user_id, auth_option_id, auth_role_id, auth_setting
				FROM ' . ACL_ARCADE_USERS_TABLE . '
				WHERE cat_id = ' . $src_cat_id;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$row = array(
				'user_id'			=> (int) $row['user_id'],
				'auth_option_id'	=> (int) $row['auth_option_id'],
				'auth_role_id'		=> (int) $row['auth_role_id'],
				'auth_setting'		=> (int) $row['auth_setting'],
			);

			foreach ($dest_cat_ids as $dest_cat_id)
			{
				$users_sql_ary[] = $row + array('cat_id' => $dest_cat_id);
			}
		}
		$db->sql_freeresult($result);

		// Query acl groups table for source cat data
		$sql = 'SELECT group_id, auth_option_id, auth_role_id, auth_setting
				FROM ' . ACL_ARCADE_GROUPS_TABLE . '
				WHERE cat_id = ' . $src_cat_id;
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$row = array(
				'group_id'			=> (int) $row['group_id'],
				'auth_option_id'	=> (int) $row['auth_option_id'],
				'auth_role_id'		=> (int) $row['auth_role_id'],
				'auth_setting'		=> (int) $row['auth_setting'],
			);

			foreach ($dest_cat_ids as $dest_cat_id)
			{
				$groups_sql_ary[] = $row + array('cat_id' => $dest_cat_id);
			}
		}
		$db->sql_freeresult($result);

		$db->sql_transaction('begin');

		// Clear current permissions of destination cats
		if ($clear_dest_perms)
		{
			$sql = 'DELETE FROM ' . ACL_ARCADE_USERS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $dest_cat_ids);
			$db->sql_query($sql);

			$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $dest_cat_ids);
			$db->sql_query($sql);
		}

		$db->sql_multi_insert(ACL_ARCADE_USERS_TABLE, $users_sql_ary);
		$db->sql_multi_insert(ACL_ARCADE_GROUPS_TABLE, $groups_sql_ary);

		if ($add_log)
		{
			$this->add_log('admin', 'LOG_ARCADE_CATEGORY_COPIED_PERMISSIONS', $src_cat_name, implode(', ', $dest_cat_names));
		}

		$db->sql_transaction('commit');

		return true;
	}

	/**
	* Pulls the arcade download data from a remote site
	* This expects the data to be gzcompressed, and serialized
	* it is then cached locally
	*/
	function get_remote_data($url, $timeout = 10)
	{
		global $cache, $arcade_config;

		$md5_url = md5($url);
		if (($list = $cache->get('_arcade_dl_' . $md5_url)) === false)
		{
			if (function_exists('curl_init'))
			{
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, $url);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
				$list = curl_exec($ch);
				curl_close($ch);
			}
			else if (@ini_get('allow_url_fopen'))
			{
				$list = get_file_contents($url);
			}

			$list = @gzuncompress($list);
			$list = (is_string($list) && (preg_match("/^[adobis]:[0-9]+:.*[;}]/si", $list))) ? unserialize($list) : false;

			$cache->put('_arcade_dl_' . $md5_url, $list, $arcade_config['cache_time'] * 3600);
		}

		return $list;
	}

	/**
	* Purges the cached arcade download list
	*/
	function purge_download_cache()
	{
		global $cache;

		// Purge all arcade acp download list cache files
		if ($files = scandir($cache->cache_dir))
		{
			foreach ($files as $file)
			{
				if (strpos($file, 'data_arcade_dl_') !== false && strpos($file, 'data_arcade_recent_sites') !== false)
				{
					continue;
				}

				$cache->remove_file($cache->cache_dir . $file);
			}
		}
	}

	/**
	* Display correct report type based on constant passed
	*/
	function display_report_type($report_type)
	{
		global $user;

		$type = '';
		switch ($report_type)
		{
			case ARCADE_REPORT_SCORING:
				$type = $user->lang['ARCADE_REPORT_SCORING'];
			break;

			case ARCADE_REPORT_PLAYING:
				$type = $user->lang['ARCADE_REPORT_PLAYING'];
			break;

			case ARCADE_REPORT_DOWNLOADING:
				$type = $user->lang['ARCADE_REPORT_DOWNLOADING'];
			break;

			default:
				$type = $user->lang['ARCADE_REPORT_OTHER'];
			break;
		}
		return $type;
	}

	/**
	* Move games up and down by fixed order
	*/
	function move_game_by($game_id, $cat_id, $action)
	{
		global $db;

		switch($action)
		{
			case 'g_move_up':
				$move = 15;
			break;

			case 'g_move_down':
				$move = -15;
			break;
		}

		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
				SET game_order = game_order + ' . $move . '
				WHERE game_id = ' . (int) $game_id;
		$db->sql_query($sql);

		$sql = 'SELECT game_id, game_order FROM ' . ARCADE_GAMES_TABLE . '
				WHERE cat_id = ' . (int) $cat_id . '
				ORDER BY game_order ASC';
		$result = $db->sql_query($sql);

		$i = 10;
		while ($row = $db->sql_fetchrow($result))
		{
			$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
					SET game_order = ' . $i . '
					WHERE game_id = ' . (int) $row['game_id'];
			$db->sql_query($sql);
			$i += 10;
		}
		$db->sql_freeresult($result);
	}

	/**
	* Correctly updated the current highscore holder of a game
	*/
	function update_highscore($game_data)
	{
		global $db;

		$score_order = ($game_data['game_scoretype'] == SCORETYPE_HIGH) ? 'DESC' : 'ASC';

		$sql = 'SELECT score, user_id, score_date
				FROM ' . ARCADE_SCORES_TABLE . '
				WHERE game_id = ' . (int) $game_data['game_id'] . '
				ORDER BY score ' . $score_order . ', score_date ASC';
		$result = $db->sql_query_limit($sql, 1);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!empty($row))
		{
			$sql_ary = array(
				'game_highscore'	=> $row['score'],
				'game_highuser'		=> $row['user_id'],
				'game_highdate'		=> $row['score_date'],
			);
		}
		else
		{
			$sql_ary = array(
				'game_highscore'	=> 0,
				'game_highuser'		=> 0,
				'game_highdate'		=> 0,
			);
		}

		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE game_id = ' . (int) $game_data['game_id'];
		$db->sql_query($sql);
	}

	/**
	* Update the edited user score data
	*/
	function update_score($score_data, $user_id, $game_id)
	{
		global $db;

		if ($score_data['comment_text'] != '')
		{
			generate_text_for_storage($score_data['comment_text'], $score_data['comment_uid'], $score_data['comment_bitfield'], $score_data['comment_options'], true, true, true);
		}

		$sql = 'UPDATE ' . ARCADE_SCORES_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $score_data) . '
				WHERE user_id = ' . (int) $user_id . '
				AND game_id = ' . (int) $game_id;
		$db->sql_query($sql);

		// After we update the score and comment data
		// We update the highscore holder for the game
		$row = $this->get_game_data($game_id);
		$this->set_last_play($row['cat_id']);
		$this->update_highscore($row);
	}

	/**
	* Delete a user score
	*/
	function delete_score($game_data, $user_id)
	{
		global $db;

		$retain_plays = request_var('retain_plays', false);

		$sql = 'DELETE FROM ' . ARCADE_SCORES_TABLE . '
				WHERE ' . $db->sql_in_set('game_id', $game_data['game_id']) . '
				AND   ' . $db->sql_in_set('user_id', $user_id);
		$db->sql_query($sql);

		if (!$retain_plays)
		{
			$sql = 'DELETE FROM ' . ARCADE_PLAYS_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $game_data['game_id']) . '
					AND   ' . $db->sql_in_set('user_id', $user_id);
			$db->sql_query($sql);
		}

		$this->set_last_play($game_data['cat_id']);
		$this->sync('game', $game_data['game_id']);
		$this->sync('category', $game_data['cat_id']);
		$this->sync('total_data', 'plays');

		// After we update the score we update the highscore holder for the game
		if ($user_id == $game_data['game_highuser'])
		{
			$this->update_highscore($game_data);
		}
	}

	/**
	* Delete a user super score
	*/
	function delete_super_score($game_ids, $user_ids)
	{
		global $db;

		if (!is_array($game_ids))
		{
			$game_ids = array($game_ids);
		}

		if (!is_array($user_ids))
		{
			$user_ids = array($user_ids);
		}

		$sql = 'DELETE FROM ' . ARCADE_SUPER_CHAMP_TABLE . '
				WHERE ' . $db->sql_in_set('game_id', array_map('intval', $game_ids)) . '
				AND   ' . $db->sql_in_set('user_id', array_map('intval', $user_ids));
		$db->sql_query($sql);
	}

	/**
	* Update the edited user super score data
	*/
	function update_super_score($score_data, $user_id, $game_id)
	{
		global $db;

		$sql = 'UPDATE ' . ARCADE_SUPER_CHAMP_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $score_data) . '
				WHERE user_id = ' . (int) $user_id . '
				AND game_id = ' . (int) $game_id;
		$db->sql_query($sql);
	}

	/*
	* Set the last play data for the category
	* If the correct last play data cannot be
	* found it clears
	*/
	function set_last_play($cat_id = false)
	{
		global $db;

		// If we don't have a cat id we get all categories
		if (!$cat_id)
		{
			$cat_id = array();
			$sql = 'SELECT cat_id
					FROM ' . ARCADE_CATS_TABLE . '
					ORDER BY cat_id';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$cat_id[] = $row['cat_id'];
			}
			$db->sql_freeresult($result);
		}

		if (is_array($cat_id))
		{
			$cat_id = array_map('intval', $cat_id);
		}
		else
		{
			$cat_id = array((int) $cat_id);
		}

		foreach ($cat_id as $id)
		{
			$sql_array = array(
				'SELECT'	=> 'g.game_id, g.game_name, u.user_id, u.username, u.user_colour, s.score, s.score_date',
				'FROM'		=> array(
					ARCADE_GAMES_TABLE	=> 'g',
				),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
						'ON'	=> 'g.game_id = s.game_id'
					),
					array(
						'FROM'	=> array(USERS_TABLE => 'u'),
						'ON'	=> 's.user_id = u.user_id'
					),
				),
				'WHERE'		=> 'g.game_id = s.game_id
					AND g.cat_id = ' . $id,
				'ORDER_BY'		=> 's.score_date DESC',
			);

			$sql = $db->sql_build_query('SELECT', $sql_array);
			$result = $db->sql_query_limit($sql, 1);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if ($row)
			{
				$sql_ary = array(
					'cat_last_play_game_id'		=> $row['game_id'],
					'cat_last_play_game_name'	=> $row['game_name'],
					'cat_last_play_user_id'		=> $row['user_id'],
					'cat_last_play_score'		=> $row['score'],
					'cat_last_play_time'		=> $row['score_date'],
					'cat_last_play_username'	=> $row['username'],
					'cat_last_play_user_colour'	=> $row['user_colour'],
				);
			}
			else
			{
				$sql_ary = array(
					'cat_last_play_game_id'		=> 0,
					'cat_last_play_game_name'	=> '',
					'cat_last_play_user_id'		=> 0,
					'cat_last_play_score'		=> 0,
					'cat_last_play_time'		=> 0,
					'cat_last_play_username'	=> '',
					'cat_last_play_user_colour'	=> '',
				);
			}

			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
					WHERE ' . $db->sql_in_set('cat_id', $id);
			$db->sql_query($sql);
		}
	}

	function get_cat_game_ids($type, $id, $gametypev3 = false, $gametypev32 = false)
	{
		global $db, $phpbb_root_path;

		$sql_in_set = false;
		$gsv = $game_datas = array();

		switch ($type)
		{
			case 'gametype':
				if ($id == IBPROV3_GAME && (($gametypev3 && !$gametypev32) || (!$gametypev3 && $gametypev32)))
				{
					if ($gamedata = scandir($phpbb_root_path . 'arcade/gamedata/'))
					{
						foreach ($gamedata as $game_folder)
						{
							$dir = $phpbb_root_path . 'arcade/gamedata/' . $game_folder;
							if (is_file($dir) || in_array($game_folder, array('.', '..')))
							{
								continue;
							}
							else if (file_exists($dir . '/') && $game = scandir($dir . '/'))
							{
								foreach ($game as $file)
								{
									if (!is_file($dir . '/' . $file) || in_array($file, array('.', '..')))
									{
										continue;
									}
									else if (($gametypev3 && $file == 'v3game.txt') || ($gametypev32 && $file == 'v32game.txt'))
									{
										$gsv[] = $game_folder;
									}
								}
							}
						}

						$gsv = array_unique($gsv);
					}

					if (sizeof($gsv))
					{
						$sql_in_set = $db->sql_in_set('g.game_scorevar', $gsv);
					}
				}
				else
				{
					$sql_in_set = $db->sql_in_set('g.game_type', $id);
				}
			break;

			case 'cat':
				$sql_in_set = $db->sql_in_set('g.cat_id', $id);
			break;
		}

		if ($sql_in_set == false)
		{
			return false;
		}

		$sql_array = array(
			'SELECT'		=> 'g.game_id, g.game_name, g.game_name_clean, g.cat_id, g.game_swf',
			'FROM'			=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),
			'WHERE'			=> $sql_in_set,
			'ORDER_BY'		=> 'g.game_name_clean',
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$game_datas['game_ids'][] = $row['game_id'];
			$game_datas['game_names'][] = $row['game_name'];
			$game_datas['cat_ids'][] = $row['cat_id'];
			$game_datas['games_swf'][] = $row['game_swf'];
		}
		$db->sql_freeresult($result);

		return (sizeof($game_datas)) ? $game_datas : false;
	}

	/**
	* Send install file to the browser
	*/
	function download_install_file($file, $filename, $use_method, $methods)
	{
		global $phpbb_root_path, $phpEx;

		if (!in_array($use_method, $methods))
		{
			$use_method = '.tar';
		}

		include($phpbb_root_path . 'includes/functions_compress.' . $phpEx);

		if ($use_method == '.zip')
		{
			$compress = new compress_zip('w', $phpbb_root_path . 'store/' . $filename . $use_method);
		}
		else
		{
			$compress = new compress_tar('w', $phpbb_root_path . 'store/' . $filename . $use_method, $use_method);
		}

		$compress->add_data('', 'index.htm');
		$compress->add_data($file, $filename . '.' . $phpEx);

		$compress->close();
		$compress->download($filename);
		@unlink($phpbb_root_path . 'store/' . $filename . $use_method);

		garbage_collection();
		exit_handler();
	}

	function game_control_select($value = 0)
	{
		$option_ary = (in_array($value, array(GAME_CONTROL_KEYBOARD_MOUSE, GAME_CONTROL_KEYBOARD, GAME_CONTROL_MOUSE))) ? array() : array(0 => 'ARCADE_UNKNOWN');

		$option_ary += array(
			GAME_CONTROL_KEYBOARD_MOUSE	=> 'ARCADE_KEYBOARD_MOUSE',
			GAME_CONTROL_KEYBOARD		=> 'ARCADE_KEYBOARD',
			GAME_CONTROL_MOUSE			=> 'ARCADE_MOUSE'
		);

		return build_select(array_filter($option_ary), $value);
	}
	/**
	* Create game type dropdown
	*/
	function game_type_select($value = 0)
	{
		$option_ary = ($value == UNKNOWN_GAME) ? array(UNKNOWN_GAME => 'ARCADE_UNKNOWN') : array();

		$option_ary += array(
			AMOD_GAME			=> 'AMOD_GAME',
			AR_GAME				=> 'AR_GAME',
			IBPRO_GAME			=> 'IBPRO_GAME',
			ARCADELIB_GAME		=> 'ARCADELIB_GAME',
			V3ARCADE_GAME		=> 'V3ARCADE_GAME',
			IBPROV3_GAME		=> 'IBPROV3_GAME',
			PHPBBARCADE_GAME	=> 'PHPBBARCADE_GAME',
			NOSCORE_GAME		=> 'NOSCORE_GAME'
		);

		return build_select(array_filter($option_ary), $value);
	}

	/**
	* Create game score type dropdown
	*/
	function game_scoretype_select($value = 0)
	{
		$option_ary = array(
			SCORETYPE_HIGH	=> 'ARCADE_GAME_SCORETYPE_HIGH',
			SCORETYPE_LOW	=> 'ARCADE_GAME_SCORETYPE_LOW'
		);

		return build_select($option_ary, $value);
	}

	/**
	* Simple version of jumpbox, just lists authed categories
	*/
	function make_cat_select($select_id = false, $ignore_id = false, $ignore_acl = false, $ignore_nongame = false, $ignore_emptycat = true, $only_acl_play = false, $return_array = false)
	{
		global $db, $auth, $auth_arcade;

		$acl = ($ignore_acl) ? '' : (($only_acl_play) ? 'c_play' : array('c_list'));

		// This query is identical to the jumpbox one
		$sql = 'SELECT cat_id, cat_name, parent_id, cat_type, left_id, right_id
				FROM ' . ARCADE_CATS_TABLE . '
				ORDER BY left_id ASC';
		$result = $db->sql_query($sql, 600);

		$right = 0;
		$padding_store = array('0' => '');
		$padding = '';
		$cat_list = ($return_array) ? array() : '';

		// Sometimes it could happen that category will be displayed here not be displayed within the index page
		// This is the result of categories not displayed at index, having list permissions and a parent of a category with no permissions.
		// If this happens, the padding could be "broken"

		while ($row = $db->sql_fetchrow($result))
		{
			if ($row['left_id'] < $right)
			{
				$padding .= '&nbsp; &nbsp;';
				$padding_store[$row['parent_id']] = $padding;
			}
			else if ($row['left_id'] > $right + 1)
			{
				$padding = (isset($padding_store[$row['parent_id']])) ? $padding_store[$row['parent_id']] : '';
			}

			$right = $row['right_id'];
			$disabled = false;

			if ($acl && !$auth_arcade->acl_gets($acl, $row['cat_id']) && !$auth->acl_get('a_arcade_cat'))
			{
				// List permission?
				if ($auth_arcade->acl_get('c_list', $row['cat_id']))
				{
					$disabled = true;
				}
				else
				{
					continue;
				}
			}

			if (
				((is_array($ignore_id) && in_array($row['cat_id'], $ignore_id)) || $row['cat_id'] == $ignore_id)
				||
				// Non-postable forum with no subforums, don't display
				($row['cat_type'] == ARCADE_CAT && ($row['left_id'] + 1 == $row['right_id']) && $ignore_emptycat)
				||
				($row['cat_type'] != ARCADE_CAT_GAMES && $ignore_nongame)
				)
			{
				// continue;
				$disabled = true;
			}

			if ($return_array)
			{
				// Include some more information...
				$selected = (is_array($select_id)) ? ((in_array($row['cat_id'], $select_id)) ? true : false) : (($row['cat_id'] == $select_id) ? true : false);
				$cat_list[$row['cat_id']] = array_merge(array('padding' => $padding, 'selected' => ($selected && !$disabled), 'disabled' => $disabled), $row);
			}
			else
			{
				$selected = (is_array($select_id)) ? ((in_array($row['cat_id'], $select_id)) ? ' selected="selected"' : '') : (($row['cat_id'] == $select_id) ? ' selected="selected"' : '');
				$cat_list .= '<option value="' . $row['cat_id'] . '"' . (($disabled) ? ' disabled="disabled" class="disabled-option"' : $selected) . '>' . $padding . $row['cat_name'] . '</option>';
			}
		}
		$db->sql_freeresult($result);
		unset($padding_store);

		return $cat_list;
	}

	/**
	* Get cat details
	*/
	function get_cat_info($cat_id)
	{
		global $db;

		if ($cat_id == '')
		{
			return false;
		}

		$sql = 'SELECT *
				FROM ' . ARCADE_CATS_TABLE . '
				WHERE cat_id = ' . (int) $cat_id;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$row)
		{
			return false;
		}

		return $row;
	}

	/**
	* Get cat branch
	*/
	function get_cat_branch($cat_id, $type = 'all', $order = 'descending', $include_cat = true)
	{
		global $db;

		switch ($type)
		{
			case 'parents':
				$condition = 'f1.left_id BETWEEN f2.left_id AND f2.right_id';
			break;

			case 'children':
				$condition = 'f2.left_id BETWEEN f1.left_id AND f1.right_id';
			break;

			default:
				$condition = 'f2.left_id BETWEEN f1.left_id AND f1.right_id OR f1.left_id BETWEEN f2.left_id AND f2.right_id';
			break;
		}

		$rows = array();

		$sql = 'SELECT f2.*
				FROM ' . ARCADE_CATS_TABLE . ' f1
				LEFT JOIN ' . ARCADE_CATS_TABLE . " f2 ON ($condition)
				WHERE f1.cat_id = " . (int) $cat_id . '
				ORDER BY f2.left_id ' . (($order == 'descending') ? 'ASC' : 'DESC');
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			if (!$include_cat && $row['cat_id'] == $cat_id)
			{
				continue;
			}

			$rows[] = $row;
		}
		$db->sql_freeresult($result);

		return $rows;
	}

	/**
	* Creates cat image dropdown
	*/
	function generate_cat_images($cat_image)
	{
		global $user, $phpbb_root_path, $arcade_config;

		$dir = $phpbb_root_path . $arcade_config['cat_image_path'];

		if (!file_exists($dir))
		{
			trigger_error('ARCADE_CAT_IMAGE_PATH_ERROR', E_USER_ERROR);
		}

		$category_images = array();
		if ($files = scandir($dir))
		{
			foreach ($files as $file)
			{
				$ext = strrchr($file, '.');

				if (!in_array(strtolower($ext), array('.gif', '.jpg', '.png', '.jpeg')))
				{
					continue;
				}

				list($width, $height) = getimagesize($dir . $file);

				if ($width && $height)
				{
					$category_images[] = $file;
				}
			}
		}

		$filename_list = '';
		$selected = false;
		for ($i = 0, $category_images_count = sizeof($category_images); $i < $category_images_count; $i++)
		{
			if ($category_images[$i] == $cat_image)
			{
				$category_selected = 'selected="selected"';
				$selected = true;
			}
			else
			{
				$category_selected = '';
			}
			$filename_list .= '<option value="' . $category_images[$i] . '"' . $category_selected . '>' . $category_images[$i] . '</option>';
		}

		return (($selected) ? '<option value=""></option>' : '<option value="" selected="selected"></option>') . $filename_list;
	}

	/**
	* Post game announcement
	*/
	function create_announcement($game_data, $forum_id, $author_name)
	{
		global $config, $phpbb_root_path, $phpEx;
		global $arcade_config, $lang;

		if (!function_exists('submit_post'))
		{
			include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);
		}

		$lang = $this->message_language($config['default_lang']);
		$subject = utf8_normalize_nfc($this->prepare_announce($arcade_config['announce_subject'], $game_data));
		$text = utf8_normalize_nfc($this->prepare_announce($arcade_config['announce_message'], $game_data));

		// Do not try to post message if subject or text is empty
		if (empty($subject) || empty($text))
		{
			return;
		}

		// variables to hold the parameters for submit_post
		$poll = $uid = $bitfield = $options = '';

		generate_text_for_storage($text, $uid, $bitfield, $options, true, true, true);

		$data = array(
			'forum_id'			=> $forum_id,
			'icon_id'			=> false,

			'enable_bbcode'		=> true,
			'enable_smilies'	=> true,
			'enable_urls'		=> true,
			'enable_sig'		=> true,

			'message'			=> $text,
			'message_md5'		=> md5($text),

			'bbcode_bitfield'	=> $bitfield,
			'bbcode_uid'		=> $uid,

			'post_edit_locked'	=> 0,
			'topic_title'		=> $subject,
			'topic_status'		=> false,
			'notify_set'		=> false,
			'notify'			=> false,
			'post_time'			=> 0,
			'forum_name'		=> '',
			'enable_indexing'	=> true,
		);

		submit_post('post', $subject, $author_name, POST_NORMAL, $poll, $data);
	}

	/**
	* Replace place holders for announcement
	*/
	function prepare_announce($text, $data)
	{
		global $lang, $phpbb_root_path, $phpEx;

		$arcade_url			= generate_board_url() . '/arcade.' . $phpEx;
		$game_link			= '[url=' . $arcade_url . '?mode=play&amp;g=' . $data['game_id'] . $this->gametop . ']{TEXT_VALUE}[/url]';
		$game_image			= '[img]' . $arcade_url . '?img=' . $data['game_image'] . '[/img]';
		$game_image			= str_replace('{TEXT_VALUE}', $game_image, $game_link);
		$game_popup_link	= '[popup-game=' . $arcade_url . '?mode=popup&amp;g=' . $data['game_id'] . ', ' . $data['game_width'] . ', ' . $data['game_height'] . ']' . $lang['ARCADE_PLAYING_NEW_WINDOW'] . '[/popup-game]';
		$download_link		= '[url=' . $arcade_url . '?mode=download&amp;g=' . $data['game_id'] . ']' . $lang['ARCADE_DOWNLOAD_GAME'] . '[/url]';
		$stats_link			= '[url=' . $arcade_url . '?mode=stats&amp;g=' . $data['game_id'] . ']' . $lang['ARCADE_STATS'] . '[/url]';
		$game_link			= str_replace('{TEXT_VALUE}', $lang['ARCADE_PLAY_LINK'], $game_link);
		$game_control		= str_replace($phpbb_root_path, '/', $this->game_control($data['game_control'], 'src'));
		$game_control		= ($game_control) ? '[img]' . generate_board_url() . $game_control . '[/img]' : '';
		$game_control	   .= ($data['game_control_desc']) ? '<br />' . $data['game_control_desc'] : '';

		$text = str_replace('[game_id]'			, $data['game_id']	, $text);
		$text = str_replace('[game_desc]'		, $data['game_desc'], $text);
		$text = str_replace('[game_control]'	, $game_control		, $text);
		$text = str_replace('[game_image]'		, $game_image		, $text);
		$text = str_replace('[game_name]'		, $data['game_name'], $text);
		$text = str_replace('[game_popup_link]'	, $game_popup_link	, $text);
		$text = str_replace('[download_link]'	, $download_link	, $text);
		$text = str_replace('[stats_link]'		, $stats_link		, $text);
		$text = str_replace('[game_link]'		, $game_link		, $text);

		return $text;
	}

	function delete_table_data($tables)
	{
		global $db;

		if (!is_array($tables))
		{
			$tables = array($tables);
		}

		foreach ($tables as $table)
		{
			switch ($db->sql_layer)
			{
				case 'sqlite':
				case 'firebird':
					$db->sql_query('DELETE FROM ' . $table);
				break;

				default:
					$db->sql_query('TRUNCATE TABLE ' . $table);
				break;
			}
		}
	}

	function proces_champion_users($action, $total_champ_data, $u_action, $user_id = false, $game_ids = false, $parent_id = false)
	{
		if (!$total_champ_data)
		{
			return false;
		}

		if ($game_ids !== false)
		{
			if (!is_array($game_ids))
			{
				$game_ids = array($game_ids);
			}
		}

		global $db, $user, $template, $arcade_config;

		$and = '';
		$limit = (int) $arcade_config['champion_proces_limit'];
		$proces_count = (int) request_var('proces_count', 0);

		if ($game_ids !== false)
		{
			$and .= ' AND ' . $db->sql_in_set('g.game_id', array_map('intval', $game_ids));
		}

		if ($user_id)
		{
			$and .= ' AND g.game_highuser = ' . (int) $user_id;
		}

		$sql = 'SELECT g.game_id, g.game_scoretype, g.game_highuser, g.game_highscore, g.game_highdate
				FROM ' . ARCADE_GAMES_TABLE . ' g
				LEFT JOIN ' . ARCADE_SUPER_CHAMP_TABLE . ' s ON g.game_id = s.game_id
				WHERE g.game_highuser > 0
				' . $and . '
				AND g.game_highdate > 0
				AND ((NOT ISNULL(s.score) AND ((g.game_highscore > s.score AND g.game_scoretype = ' . SCORETYPE_HIGH . ') OR
					(g.game_highscore < s.score AND g.game_scoretype = ' . SCORETYPE_LOW . '))) OR ISNULL(s.score))
				ORDER BY g.game_id ASC';
		$result = $db->sql_query_limit($sql, $limit);
		while ($row = $db->sql_fetchrow($result))
		{
			$sql_ary = array(
				'user_id'		=> (int) $row['game_highuser'],
				'score'			=> $row['game_highscore'],
				'score_date'	=> $row['game_highdate']
			);

			$sql = 'UPDATE ' . ARCADE_SUPER_CHAMP_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
					WHERE game_id = ' . (int) $row['game_id'];
			$db->sql_query($sql);

			if (!$db->sql_affectedrows())
			{
				$sql_ary = array(
					'game_id'		=> (int) $row['game_id'],
					'user_id'		=> (int) $row['game_highuser'],
					'score'			=> $row['game_highscore'],
					'score_date'	=> $row['game_highdate']
				);

				$db->sql_query('INSERT INTO ' . ARCADE_SUPER_CHAMP_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
			}

			$proces_count++;
		}
		$db->sql_freeresult($result);

		if ($limit && $proces_count < $total_champ_data && $limit < $total_champ_data)
		{
			$template->assign_vars(array(
				'S_ARCADE_SUPER_CHAMPION_PROCESSING'	=> true,
				'ARCADE_SUPER_CHAMPION_PROCESSING'		=> $user->lang['ARCADE_SUPER_CHAMPION_PROCESSING'] . '<br /><br />' . sprintf($user->lang['ARCADE_SUPER_CHAMPION_PROCESSING_BEING'], $this->get_image('full', 'img', 'loading2.gif', 'ARCADE_SUPER_CHAMPION_PROCESSING', true), $proces_count, $total_champ_data)
			));

			$redirect = $u_action . "&amp;action=$action&amp;process=1&amp;total_champ_data=$total_champ_data&amp;proces_count=$proces_count";

			if ($user_id)
			{
				$redirect .= "&amp;u=$user_id";
			}

			if ($game_ids !== false)
			{
				$redirect .= "&amp;parent_id=$parent_id&amp;update=1";

				foreach ($game_ids as $game_id)
				{
					$redirect .= "&amp;game_ids[]=$game_id";
				}
			}

			$redirect = meta_refresh(3, $redirect);

			return true;
		}

		return false;
	}

	function rev_optionset(&$user_row, $key, $value)
	{
		$var = $user_row['user_arcade_options'];

		if ($value && !($var & 1 << $this->keyoptions[$key]))
		{
			$var += 1 << $this->keyoptions[$key];
		}
		else if (!$value && ($var & 1 << $this->keyoptions[$key]))
		{
			$var -= 1 << $this->keyoptions[$key];
		}
		else
		{
			return false;
		}

		$user_row['user_arcade_options'] = $var;
		return true;
	}

	/**
	* View log
	* If $log_count is set to false, we will skip counting all entries in the database.
	*/
	function view_log($mode, &$log, &$log_count, $limit = 0, $offset = 0, $cat_id = 0, $game_id = 0, $limit_days = 0, $sort_by = 'l.log_time DESC', $keywords = '')
	{
		global $db, $user, $auth, $phpEx, $phpbb_root_path, $phpbb_admin_path, $auth_arcade;

		$sql_cat = '';
		$game_id_list = $is_auth = array();

		$profile_url = (defined('IN_ADMIN')) ? append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_manage&amp;mode=users&amp;action=prefs') : append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=viewprofile');

		switch ($mode)
		{
			case 'admin':
				$log_type = ARCADE_LOG_ADMIN;
			break;
/*
			case 'mod':
				$log_type = ARCADE_LOG_MOD;

				if ($game_id)
				{
					$sql_cat = 'AND l.game_id = ' . (int) $game_id;
				}
				else if (is_array($cat_id))
				{
					$sql_cat = 'AND ' . $db->sql_in_set('l.cat_id', array_map('intval', $cat_id));
				}
				else if ($cat_id)
				{
					$sql_cat = 'AND l.cat_id = ' . (int) $cat_id;
				}
			break;
*/
			case 'user':
			case 'users':
				$log_type = ARCADE_LOG_USERS;
			break;

			case 'critical':
				$log_type = ARCADE_LOG_CRITICAL;
			break;

			default:
				return;
		}

		// Use no preg_quote for $keywords because this would lead to sole backslashes being added
		// We also use an OR connection here for spaces and the | string. Currently, regex is not supported for searching (but may come later).
		$keywords = preg_split('#[\s|]+#u', utf8_strtolower($keywords), 0, PREG_SPLIT_NO_EMPTY);
		$sql_keywords = '';

		if (!empty($keywords))
		{
			$keywords_pattern = array();

			// Build pattern and keywords...
			for ($i = 0, $num_keywords = sizeof($keywords); $i < $num_keywords; $i++)
			{
				$keywords_pattern[] = preg_quote($keywords[$i], '#');
				$keywords[$i] = $db->sql_like_expression($db->any_char . $keywords[$i] . $db->any_char);
			}

			$keywords_pattern = '#' . implode('|', $keywords_pattern) . '#ui';

			$operations = array();
			foreach ($user->lang as $key => $value)
			{
				if (substr($key, 0, 4) == 'LOG_' && preg_match($keywords_pattern, $value))
				{
					$operations[] = $key;
				}
			}

			$sql_keywords = 'AND (';
			if (!empty($operations))
			{
				$sql_keywords .= $db->sql_in_set('l.log_operation', $operations) . ' OR ';
			}
			$sql_keywords .= 'LOWER(l.log_data) ' . implode(' OR LOWER(l.log_data) ', $keywords) . ')';
		}

		if ($log_count !== false)
		{
			$sql = 'SELECT COUNT(l.log_id) AS total_entries
					FROM ' . ARCADE_LOGS_TABLE . ' l, ' . USERS_TABLE . " u
					WHERE l.log_type = $log_type
					AND l.user_id = u.user_id
					AND l.log_time >= $limit_days
					$sql_keywords
					$sql_cat";
			$result = $db->sql_query($sql);
			$log_count = (int) $db->sql_fetchfield('total_entries');
			$db->sql_freeresult($result);
		}

		// $log_count may be false here if false was passed in for it,
		// because in this case we did not run the COUNT() query above.
		// If we ran the COUNT() query and it returned zero rows, return;
		// otherwise query for logs below.
		if ($log_count === 0)
		{
			// Save the queries, because there are no logs to display
			return 0;
		}

		if ($offset >= $log_count)
		{
			$offset = ($offset - $limit < 0) ? 0 : $offset - $limit;
		}

		$sql = 'SELECT l.*, u.username, u.username_clean, u.user_colour
				FROM ' . ARCADE_LOGS_TABLE . ' l, ' . USERS_TABLE . " u
				WHERE l.log_type = $log_type
				AND u.user_id = l.user_id
				" . (($limit_days) ? "AND l.log_time >= $limit_days" : '') . "
				$sql_keywords
				$sql_cat
				ORDER BY $sort_by";
		$result = $db->sql_query_limit($sql, $limit, $offset);

		$i = 0;
		$log = array();
		while ($row = $db->sql_fetchrow($result))
		{
			if ($row['game_id'])
			{
				$game_id_list[] = $row['game_id'];
			}

			$log[$i] = array(
				'id'				=> $row['log_id'],

				'user_id'			=> $row['user_id'],
				'username'			=> $row['username'],
				'username_full'		=> get_username_string('full', $row['user_id'], $row['username'], $row['user_colour'], false, $profile_url),

				'ip'				=> $row['log_ip'],
				'time'				=> $row['log_time'],
				'cat_id'			=> $row['cat_id'],
				'game_id'			=> $row['game_id'],

				'action'			=> (isset($user->lang[$row['log_operation']])) ? $user->lang[$row['log_operation']] : '{' . ucfirst(str_replace('_', ' ', $row['log_operation'])) . '}'
			);

			if (!empty($row['log_data']))
			{
				$log_data_ary = @unserialize($row['log_data']);
				$log_data_ary = ($log_data_ary === false) ? array() : $log_data_ary;

				if (isset($user->lang[$row['log_operation']]))
				{
					// Check if there are more occurrences of % than arguments, if there are we fill out the arguments array
					// It doesn't matter if we add more arguments than placeholders
					if ((substr_count($log[$i]['action'], '%') - sizeof($log_data_ary)) > 0)
					{
						$log_data_ary = array_merge($log_data_ary, array_fill(0, substr_count($log[$i]['action'], '%') - sizeof($log_data_ary), ''));
					}

					$log_ary = array();
					foreach ($log_data_ary as $value)
					{
						$ld_ary = explode(', ', $value);

						$log_msg = '';
						foreach ($ld_ary as $ld)
						{
							$log_msg .= (($log_msg != '') ? ', ' : '') . $this->lang_value($ld);
						}

						$log_ary[] = $log_msg;
					}

					$log[$i]['action'] = vsprintf($log[$i]['action'], $log_ary);

					// If within the admin panel we do not censor text out
					if (defined('IN_ADMIN'))
					{
						$log[$i]['action'] = bbcode_nl2br($log[$i]['action']);
					}
					else
					{
						$log[$i]['action'] = bbcode_nl2br(censor_text($log[$i]['action']));
					}
				}
				else if (!empty($log_data_ary))
				{
					$log[$i]['action'] .= '<br />' . implode('', $log_data_ary);
				}
			}

			$i++;
		}
		$db->sql_freeresult($result);

		if (sizeof($game_id_list))
		{
			$game_id_list = array_unique($game_id_list);

			// This query is not really needed if move_games() updates the cat_id field,
			// although it's also used to determine if the game still exists in the database
			$sql = 'SELECT game_id, cat_id
				FROM ' . ARCADE_GAMES_TABLE . '
				WHERE ' . $db->sql_in_set('game_id', array_map('intval', $game_id_list)) . '
				AND ' . $db->sql_in_set('cat_id', $this->get_permissions(array('c_view', 'c_play')), false, true);
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				$is_auth[$row['game_id']] = $row['cat_id'];
			}
			$db->sql_freeresult($result);

			foreach ($log as $key => $row)
			{
				if (isset($is_auth[$row['game_id']]))
				{
					$log[$key]['viewgame'] = ($auth_arcade->acl_get('c_play', $is_auth[$row['game_id']])) ? $this->url("mode=play&amp;g={$row['game_id']}") : false;
					//$log[$key]['viewlogs'] = ($auth->acl_get('m_')) ? append_sid("{$phpbb_root_path}mcp.$phpEx", 'i=arcade_logs&amp;mode=game_logs&amp;g=' . $row['game_id'], true, $user->session_id) : false;
					$log[$key]['editgame'] = ($auth->acl_get('a_arcade_game')) ? append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_manage&amp;mode=games&amp;action=edit&amp;g=' . $row['game_id']) : false;
				}
			}
		}

		return $offset;
	}

	/**
	* Display progress bar for syncinc categories
	*/
	function display_progress_bar()
	{
		global $template, $user;

		adm_page_header($user->lang['SYNC_IN_PROGRESS']);

		$template->set_filenames(array(
			'body' => 'progress_bar.html')
		);

		$template->assign_vars(array(
			'L_PROGRESS'			=> $user->lang['SYNC_IN_PROGRESS'],
			'L_PROGRESS_EXPLAIN'	=> $user->lang['SYNC_IN_PROGRESS_EXPLAIN']
		));

		adm_page_footer();
	}

	function adm_username($user_id, $username, $user_colour, $url = '')
	{
		if (!$url)
		{
			global $phpbb_admin_path, $phpEx;
			$url = append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_manage&amp;mode=users&amp;action=prefs');
		}
		else
		{
			$url .= '&amp;action=prefs';
		}

		return get_arcade_user_name('full', $user_id, $username, $user_colour, false, '', $url);
	}

	function confirm_box($method, $confirm, $s_hidden_fields = '')
	{
		global $user, $template;

		switch ($method)
		{
			case 'delete_files':
				$l_title	= 'ARCADE_DELETE_FILES';
				$l_explain	= "{$l_title}_WARNING";
			break;

			case 'retain_super_champion':
				$l_title	= 'ARCADE_RETAIN_SUPER_CHAMPION';
				$l_explain	= "{$l_title}_EXPLAIN";
			break;

			case 'retain_plays':
				$l_title	= 'ARCADE_RETAIN_PLAYS';
				$l_explain	= "{$l_title}_EXPLAIN";
			break;
		}

		$template->assign_vars(array(
			'L_ARCADE_CONFIRM'			=> $user->lang[$l_title],
			'L_ARCADE_CONFIRM_EXPLAIN'	=> $user->lang[$l_explain],
			'CONFIRM_NAME'				=> $method
		));

		$s_hidden_fields = ($s_hidden_fields != '') ? build_hidden_fields($s_hidden_fields) : '';

		confirm_box(false, $confirm, $s_hidden_fields, 'arcade/confirm_body.html');
	}
}

?>