<?php
/**
*
* @package phpBB Arcade
* @version $Id: download.php 153 2012-12-27 04:43:25Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB') || !defined('IN_PHPBB_ARCADE'))
{
	exit;
}

if (!$game_id)
{
	trigger_error($user->lang['NO_GAME_ID'] . $arcade->back_link());
}

$game_info = $arcade->get_game_cat_data($game_id);

if (!$game_info['cat_download'] || !$game_info['game_download'])
{
	trigger_error(sprintf($user->lang['ARCADE_GAME_DOWNLOAD_NOT_ALLOWED'], $game_info['game_name']) . $arcade->back_link());
}

if (!$auth->acl_get('u_arcade_download') || !$auth_arcade->acl_get('c_download', $game_info['cat_id']))
{
	trigger_error($user->lang['NO_PERMISSION_ARCADE_DOWNLOAD'] . $arcade->back_link());
}

// Check flood control
$interval			= intval(($user->data['user_id'] == ANONYMOUS) ? $arcade_config['download_anonymous_interval'] : $arcade_config['download_interval']);
$download_per_day	= intval($arcade_config['download_per_day']);

if (($interval || $download_per_day) && !$auth_arcade->acl_get('c_ignoreflood_download', $game_info['cat_id']))
{
	$time = time() - $interval;
	$download_count = $last_download_time = 0;
	$limit = ($download_per_day) ? $download_per_day : 1;
	$sql_and = ($download_per_day) ? 'AND download_time > ' . $arcade->time_stamp(gmdate('Y'), gmdate('n'), gmdate('j')) : '';

	$sql = 'SELECT download_time
			FROM ' . ARCADE_DOWNLOAD_TABLE . '
			WHERE user_id = ' . (int) $user->data['user_id'] . "
			$sql_and
			ORDER BY download_time DESC";
	$result = $db->sql_query_limit($sql, $limit);
	while ($row = $db->sql_fetchrow($result))
	{
		if (!$download_count)
		{
			$last_download_time = (int) $row['download_time'];
		}

		$download_count++;
	}
	$db->sql_freeresult($result);

	if ($interval && $last_download_time > $time)
	{
		trigger_error(sprintf($user->lang['ARCADE_NO_DOWNLOAD_TIME'], $arcade->time_format($interval, true), $arcade->time_format($last_download_time - $time, true)) . $arcade->back_link());
	}

	if ($download_per_day && $download_count >= $download_per_day)
	{
		trigger_error(sprintf($user->lang['ARCADE_NO_DOWNLOAD_DAY'], $arcade->number_format($download_per_day)) . $arcade->back_link());
	}
}

generate_arcade_nav($game_info, true);

$use_method = ($arcade_config['download_file_type'] !== '') ? $arcade_config['download_file_type'] : request_var('use_method', '');
$methods = $arcade->compress_methods();

$show_points = false;
if ($arcade->points['show'])
{
	$game_download_cost = $arcade->get_download_cost($game_info);

	if ($game_download_cost != ARCADE_FREE && $game_download_cost > 0 && !$auth_arcade->acl_get('c_downloadfree', $game_info['cat_id']))
	{
		$show_points = true;

		if ($arcade->points['total'] < $game_download_cost)
		{
			trigger_error(sprintf($user->lang['ARCADE_NO_POINTS_DOWLOAD'], $arcade->points['name']) . $arcade->back_link());
		}
	}
}

// Let the user decide in which format he wants to have the game downloaded in
if ((!request_var('download', false) && $show_points && $type != 'acp') || (!$use_method))
{
	$disabled_method = ($arcade_config['download_file_type'] !== '') ? true : false;
	$page_title = sprintf($user->lang[(($disabled_method) ? 'DOWNLOADING_GAME' : 'ARCADE_DOWNLOAD_FORMAT')], $game_info['game_name']);

	$radio_buttons = '';

	if (!$disabled_method)
	{
		foreach ($methods as $method)
		{
			$checked = (($method == $use_method) || (!$use_method && $method == '.tar')) ? ' checked="checked"' : '';
			$radio_buttons .= '<input type="radio"' . ((!$radio_buttons) ? ' id="use_method"' : '') . ' class="radio" value="' . $method . '" name="use_method"' . $checked . ' />&nbsp;' . $method . '&nbsp;';
		}
	}

	$template->assign_vars(array(
		'S_DOWNLOAD_GAME_COST'		=> $show_points,
		'S_SELECT_METHOD'			=> (!$arcade_config['download_file_type']) ? true : false,
		'U_ACTION'					=> $arcade->url("mode=download&amp;g={$game_id}"),

		'L_ARCADE_DOWNLOAD_COST'	=> ($show_points) ? sprintf($user->lang['ARCADE_DOWNLOAD_COST_POINTS'], $arcade->number_format($game_download_cost) . ' ' . $arcade->points['name']) : '',
		'GAME_NAME'					=> $game_info['game_name'],
		'ARCADE_DOWNLOAD_FORMAT'	=> $page_title,
		'RADIO_BUTTONS'				=> $radio_buttons
	));

	display_arcade_online();
	// Output page
	page_header($page_title, false);

	$template->set_filenames(array(
		'body' => 'arcade/download_body.html')
	);

	page_footer();
}

$arcade->download_game($game_info, $use_method, $methods);

?>