<?php
/**
*
* @package phpBB Arcade
* @version $Id: game_install_file.php 158 2012-12-28 12:56:32Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Creates install file for the game
*/
function Arcade_create_game_install_file(&$data)
{
	static $_cache_default_install_file;

	if (empty($_cache_default_install_file))
	{
		global $phpbb_root_path, $phpEx;
		$_cache_default_install_file = get_file_contents($phpbb_root_path . 'arcade/includes/default_install_file.' . $phpEx);
	}

	global $file_functions;

	$data['game_type'] = (int) $data['game_type'];
	$game_type = arcade_game_type($data['game_type']);

	$data['game_width'] = (int) $data['game_width'];
	$data['game_height'] = (int) $data['game_height'];

	if ($data['game_width'] <= 0 || $data['game_height'] <=0)
	{
		$data['game_width'] = 640;
		$data['game_height'] = 480;
	}

	$data['game_control'] = (!empty($data['game_control'])) ? (int) $data['game_control'] : 0;

	if (!isset($data['game_control_desc']))
	{
		$data['game_control_desc'] = '';
	}

	switch ($data['game_control'])
	{
		case GAME_CONTROL_KEYBOARD_MOUSE:
			$game_control = 'GAME_CONTROL_KEYBOARD_MOUSE';
		break;

		case GAME_CONTROL_KEYBOARD:
			$game_control = 'GAME_CONTROL_KEYBOARD';
		break;

		case GAME_CONTROL_MOUSE:
			$game_control = 'GAME_CONTROL_MOUSE';
		break;

		default:
			$game_control = 'UNKNOWN_GAME';
		break;
	}

	$data['game_scoretype'] = (int) $data['game_scoretype'];

	switch ($data['game_scoretype'])
	{
		case SCORETYPE_HIGH:
			$game_scoretype = 'SCORETYPE_HIGH';
		break;

		case SCORETYPE_LOW:
			$game_scoretype = 'SCORETYPE_LOW';
		break;

		default:
			$game_scoretype = 'SCORETYPE_HIGH';
		break;
	}

	$game_image = '$' . "game_file . '.gif'";

	if (!empty($data['game_image']) && !empty($data['game_swf']))
	{
		if ($file_functions->remove_extension($data['game_image']) == $file_functions->remove_extension($data['game_swf']))
		{
			$game_image = '$' . "game_file . '" . strrchr($data['game_image'], '.') . "'";
		}
		else
		{
			$game_image = "'" . addslashes($data['game_image']) . "'";
		}
	}

	$game_scorevar = '$game_file';

	if (isset($data['game_swf']) && isset($data['game_scorevar']))
	{
		if ($file_functions->remove_extension($data['game_swf']) != $data['game_scorevar'])
		{
			$game_scorevar = "'" . addslashes($data['game_scorevar']) . "'";
		}
	}

	$search = array('{GAME_NAME}', "'{GAME_SCOREVAR}'", '{GAME_DESC}', "'{GAME_CONTROL}'", '{GAME_CONTROL_DESC}', "'{GAME_IMAGE}'", "'{GAME_TYPE}'", "'{GAME_WIDTH}'", "'{GAME_HEIGHT}'", "'{GAME_SCORETYPE}'");
	$replace = array(addslashes($data['game_name']), $game_scorevar, addslashes($data['game_desc']), $game_control, addslashes($data['game_control_desc']), $game_image, $game_type, $data['game_width'], $data['game_height'], $game_scoretype);

	return str_replace($search, $replace, $_cache_default_install_file);
}

/**
* Updates install file from the information in the db
*/
function Arcade_update_game_install_file($game_id, $install_file = false, $game_data = false)
{
	if ($game_id)
	{
		global $arcade;

		$game_data = $arcade->get_game_data($game_id);
		$install_file = $arcade->set_path($game_data['game_swf'], 'install');
	}

	if (!$install_file || !$game_data)
	{
		return false;
	}

	return arcade_fwrite($install_file, Arcade_create_game_install_file($game_data));
}

function Arcade_convert_game_install_file($dir, $game_filename, $extract = false)
{
	global $arcade, $file_functions, $phpbb_root_path, $phpEx;

	// As far as I know all ibpro games start with "game_" if this is not the case
	// please let me know as this will have to be changed
	$is_xml = $create_install_file = false;
	$orig_game_filename = $game_filename;

	if (substr($game_filename, 0, 5) == 'game_')
	{
		$game_filename = str_replace('game_', '', $game_filename);
	}

	// Check to make sure a valid config file is present
	if ($extract)
	{
		$game_config1 = $dir . $game_filename . '.' . $phpEx;
		$game_config2 = $dir . $orig_game_filename . '.' . $phpEx;
		$game_xml1	  = $dir . 'game.xml';
		$game_xml2	  = $dir . 'game.xml';
	}
	else
	{
		$game_config1 = $dir . $game_filename . '/' . $game_filename . '.' . $phpEx;
		$game_config2 = $dir . $orig_game_filename . '/'. $orig_game_filename . '.' . $phpEx;
		$game_xml1	  = $dir . $game_filename . '/' . 'game.xml';
		$game_xml2	  = $dir . $orig_game_filename . '/' . 'game.xml';
	}

	$install_file = '';

	if (file_exists($game_config1))
	{
		$install_file = $game_config1;
	}
	else if (file_exists($game_config2))
	{
		$install_file = $game_config2;
		$game_filename = $orig_game_filename;
	}
	else if (file_exists($game_xml1))
	{
		$is_xml = true;
		$install_file = $game_xml1;
	}
	else if (file_exists($game_xml2))
	{
		$is_xml = true;
		$install_file = $game_xml2;
		$game_filename = $orig_game_filename;
	}

	if ($install_file)
	{
		$update = true;

		if ($is_xml)
		{
			$install_array = $file_functions->xml2array($install_file);
			$file_functions->delete_file($install_file);
			$install_file = $dir . (($extract) ? '' : $game_filename . '/') . $game_filename . '.' . $phpEx;

			foreach ($install_array as $key => $value)
			{
				if (is_string($value))
				{
					$install_array[$key] = trim($value);
				}
			}

			$data = array(
				'game_name'			=> $install_array['game']['name'],
				'game_desc'			=> $install_array['game']['description'] . ' ' . $install_array['game']['directions'],
				'game_width'		=> (int) $install_array['game']['width'],
				'game_height'		=> (int) $install_array['game']['height'],
				'game_scoretype'	=> (strtolower($install_array['game']['revscore']) == 'false') ? SCORETYPE_HIGH : SCORETYPE_LOW,
				'game_type'			=> (strtolower($install_array['game']['highscore']) == 'false') ? NOSCORE_GAME : AR_GAME
			);

			if (isset($install_array['game']['image']))
			{
				$data['game_image'] = $install_array['game']['image'];
			}

			if (isset($install_array['game']['swf']))
			{
				$data['game_swf'] = $install_array['game']['swf'];
			}

			$create_install_file = true;
			$game_filename = $file_functions->remove_extension($install_array['game']['swf']);
		}
		else
		{
			$file_contents = get_file_contents($install_file);

			if (strpos($file_contents, '$game_data') !== false)
			{
				$update = false;
				$create_install_file = true;
			}

			if (!$create_install_file && strpos($file_contents, '$ibp_config') === false)
			{
				if (strpos($file_contents, '$config') === false)
				{
					$update = false;
				}

				// We have to read the ibpro config file contents and replace the $config variable
				// or there will be problems since we will have overwritten the phpbb config variable
				if ($update)
				{
					$ibp_config = str_replace('$config', '$ibp_config', $file_contents);
					arcade_fwrite($install_file, $ibp_config);
				}
			}

			unset($file_contents);

			if ($update)
			{
				// Since we now have a nice config file lets include it
				@include($install_file);

				// Remove extra spaces from array strings
				foreach ($ibp_config as $key => $value)
				{
					if (is_string($value))
					{
						$ibp_config[$key] = trim($value);
					}
				}

				// $ibp_config['gname'] -> $game_filename

				$data = array();
				$gamedata_path = $phpbb_root_path . 'arcade/gamedata/' . $game_filename;

				// Find game type
				if (file_exists($gamedata_path . '/v32game.txt') || file_exists($gamedata_path . '/v3game.txt'))
				{
					// We have found a v3x game
					$data['game_type'] = IBPROV3_GAME;
				}
				else if (file_exists($gamedata_path . '/'. $game_filename . '.txt'))
				{
					// We have found an arcadelib game
					$data['game_type'] = ARCADELIB_GAME;
				}
				else
				{
					$data['game_type'] = IBPRO_GAME;
				}

				$create_install_file = true;
				$gkeys = strtolower($ibp_config['gkeys']);

				$data += array(
					'game_name'			=> $ibp_config['gtitle'],
					'game_desc'			=> $ibp_config['gwords'] . (($ibp_config['object'] && $ibp_config['object'] != '1' && $ibp_config['gwords'] != $ibp_config['object']) ? ' ' . $ibp_config['object'] : ''),
					'game_control'		=> ($gkeys == 'mouse' || $gkeys == 'mouse.') ? GAME_CONTROL_MOUSE : UNKNOWN_GAME,
					'game_control_desc'	=> ($gkeys && $gkeys != 'mouse' && $gkeys == 'mouse.') ? $ibp_config['gkeys'] : '',
					'game_width'		=> $ibp_config['gwidth'],
					'game_height'		=> $ibp_config['gheight'],
					'game_scoretype'	=> (!isset($ibp_config['highscore_type']) || $ibp_config['highscore_type'] == 'high') ? SCORETYPE_HIGH : SCORETYPE_LOW
				);
			}
		}

		// Here is where we create the new install file and place it in the correct location
		if ($update && $create_install_file)
		{
			if (!(Arcade_update_game_install_file(false, $install_file, $data)))
			{
				$create_install_file = false;
			}
		}

		return ($create_install_file) ? $game_filename : false;
	}

	return false;
}

function Arcade_game_install_data($dir, $file, $install_file)
{
	$game_data = $correct_install_file = false;

	if (file_exists($install_file))
	{
		$file_contents = get_file_contents($install_file);

		if (strpos($file_contents, '$game_data') !== false)
		{
			$correct_install_file = true;
		}

		unset($file_contents);
	}

	if (!$correct_install_file)
	{
		Arcade_convert_game_install_file($dir, $file);
	}


	if (file_exists($install_file))
	{
		@include($install_file);

		if (!empty($game_data))
		{
			if (!isset($game_data['game_control']))
			{
				Arcade_update_game_install_file(false, $install_file, $game_data);
			}
		}
		else
		{
			$game_data = array();
		}
	}

	return $game_data;
}

?>