<?php
/**
*
* @package phpBB Arcade
* @version $Id: phpBBArcade.php 161 2012-12-30 00:26:39Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class phpBBArcade
{
	var $session;
	var $gametop = '#gametop';

	function phpBBArcade($ini = false)
	{
		if ($ini)
		{
			global $phpbb_root_path, $phpEx;
			global $arcade_cache, $arcade_config, $auth_arcade, $arcade_plugin;

			include($phpbb_root_path . 'arcade/includes/cache.' . $phpEx);
			include($phpbb_root_path . 'arcade/includes/auth.' . $phpEx);
			include($phpbb_root_path . 'arcade/includes/plugin.' . $phpEx);

			$arcade_cache	= new arcade_cache();
			$arcade_config	= $arcade_cache->obtain_arcade_config();
			$auth_arcade	= new auth_arcade();

			if (!$this->version_check())
			{
				return;
			}

			$arcade_plugin	= new arcade_plugin();
		}
	}

	function send_score($index = true)
	{
		if ($index)
		{
			$arcade_act	= strtolower(request_var('act', ''));
			$scoretype	= ($arcade_act == 'arcade' && strtolower(request_var('do', '')) == 'newscore') ? IBPRO_GAME : (($arcade_act == 'arcadelib') ? ARCADELIB_GAME : ((strtolower(request_var('autocom', '')) == 'arcade') ? IBPROV3_GAME : false));

			if ($scoretype !== false)
			{
				define('IN_PHPBB_ARCADE', true);
			}
		}
		else
		{
			$phparcade_type	= (isset($_POST['sendscore'])) ? strtolower(request_var('sendscore', '')) : false;
			$v3arcade_type	= (isset($_POST['sessdo'])) ? strtolower(request_var('sessdo', '')) : false;
			$scoretype		= ($phparcade_type) ? PHPBBARCADE_GAME : (($v3arcade_type) ? V3ARCADE_GAME : false);
		}

		return $scoretype;
	}

	function user($lang_set = true, $auth_set = true)
	{
		global $user, $auth_arcade;

		if ($lang_set && !isset($user->lang['PHPBBARCADE_GAME']))
		{
			$user->add_lang('mods/arcade/arcade');
		}

		if ($auth_set && !isset($user->data['user_arcade_options']))
		{
			$this->set_userdata($user->data);
			$auth_arcade->acl($user->data);
		}
	}

	function set_userdata(&$userdata)
	{
		if (!$userdata['user_id'])
		{
			return;
		}

		global $db;

		// Add arcade user data to $user->data
		$sql = 'SELECT *
				FROM ' . ARCADE_USERS_TABLE . '
				WHERE user_id = ' . (int) $userdata['user_id'];
		$result = $db->sql_query($sql);
		$arcade_data = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$arcade_data)
		{
			$arcade_data = $this->user_add($userdata['user_id'], false);
		}

		$userdata = array_merge($userdata, $arcade_data);
	}

	function user_add($user_id, $new_user = true)
	{
		global $db;

		$sql_ary = array(
			'user_id'						=> (int) $user_id,
			'user_arcade_permissions'		=> '',
			'user_arcade_perm_from'			=> 0,
			'user_arcade_last_download'		=> 0,
			'user_arcade_last_play'			=> 0,
			'games_last_search'				=> 0,
			'games_last_search_term'		=> '',
			'user_arcade_options'			=> ($user_id == ANONYMOUS) ? 220 : 223,
			'games_per_page'				=> 0,
			'games_sort_dir'				=> 'd',
			'games_sort_order'				=> 'd'
		);

		$db->sql_query('INSERT INTO ' . ARCADE_USERS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));

		if ($new_user)
		{
			global $cache;

			$cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);
			return;
		}

		return $sql_ary;
	}

	function ban_check($return = false)
	{
		global $user;

		if ($user->data['is_registered'] && $user->data['user_type'] == USER_FOUNDER)
		{
			return false;
		}

		static $_arcade_user_ban_cache;

		if (!isset($_arcade_user_ban_cache))
		{
			global $arcade_config;

			$_arcade_user_ban_cache = false;

			if ($arcade_config['ban_user_ip'])
			{
				foreach ($this->ban_users() as $user_id => $user_ip)
				{
					if ($user->data['user_id'] == $user_id || $user_ip == $user->ip)
					{
						$_arcade_user_ban_cache = true;
						break;
					}
				}
			}
			else
			{
				$_arcade_user_ban_cache = (in_array($user->data['user_id'], array_keys($this->ban_users()))) ? true : false;
			}
		}

		if ($_arcade_user_ban_cache)
		{
			if ($return)
			{
				return true;
			}

			$this->user(true, false);
			meta_refresh(5, $this->url(false, 'index'));
			trigger_error('ARCADE_BANNED');
		}

		return false;
	}

	function ban_users()
	{
		global $arcade_cache;

		return $arcade_cache->obtain_arcade_ban_users();
	}

	function banned($type, $user_id = false, $user_ip = '', $game_id = false, $score = 0, $formtime = 0)
	{
		global $db, $user;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
			$user_ip = $user->ip;
		}

		if ($user_id == ANONYMOUS || is_array($user_id))
		{
			return;
		}

		$sql_ary = array(
			'user_id'		=> (int) $user_id,
			'banned_type'	=> (int) $type,
			'banned_ip'		=> $user_ip,
			'banned_date'	=> time(),
			'game_id'		=> (int) $game_id,
			'formtime'		=> (int) $formtime,
			'score'			=> (float) $score
		);

		$db->sql_return_on_error(true);
		$db->sql_query('INSERT INTO ' . ARCADE_USERS_BANNED_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
		$db->sql_return_on_error(false);

		$this->cache_purge('banned');
	}

	/**
	* Set config value. Creates missing config entry.
	*/
	function set_config($config_name, $config_value, $is_dynamic = false)
	{
		global $db, $cache, $arcade_config;

		$sql = 'UPDATE ' . ARCADE_CONFIG_TABLE . "
				SET config_value = '" . $db->sql_escape($config_value) . "'
				WHERE config_name = '" . $db->sql_escape($config_name) . "'";
		$db->sql_query($sql);

		if (!$db->sql_affectedrows() && !isset($arcade_config[$config_name]))
		{
			$sql = 'INSERT INTO ' . ARCADE_CONFIG_TABLE . ' ' . $db->sql_build_array('INSERT', array(
					'config_name'	=> $config_name,
					'config_value'	=> $config_value,
					'is_dynamic'	=> ($is_dynamic) ? 1 : 0));
			$db->sql_query($sql);
		}

		$arcade_config[$config_name] = $config_value;

		if (!$is_dynamic)
		{
			$cache->destroy('_arcade');
		}
	}

	function begin($ini = false, $in_arcade = true, $set_data = true, $chall = false, $sess = false)
	{
		global $arcade, $phpbb_root_path, $phpEx;

		$this->user();

		if (!class_exists('arcade'))
		{
			include($phpbb_root_path . 'arcade/includes/class.' . $phpEx);
		}

		if (!function_exists('display_arcade_cat'))
		{
			include($phpbb_root_path . 'arcade/includes/arcade.' . $phpEx);
		}

		if (defined('IN_ADMIN') || defined('USE_ARCADE_ADMIN'))
		{
			if (!class_exists('auth_arcade_admin'))
			{
				include($phpbb_root_path . 'arcade/includes/auth_admin.' . $phpEx);
			}

			if (!class_exists('arcade_admin'))
			{
				include($phpbb_root_path . 'arcade/includes/class_admin.' . $phpEx);
			}

			$arcade = new arcade_admin();
		}
		else if ($ini)
		{
			$arcade = new arcade($in_arcade, $set_data, $chall);

			if ($sess)
			{
				$arcade->session_begin();
			}
		}

		if (defined('IN_PHPBB_ARCADE'))
		{
			global $file_functions;

			if (!function_exists('Arcade_game_install_data'))
			{
				include($phpbb_root_path . 'arcade/includes/game_install_file.' . $phpEx);

				if (!defined('IN_ADMIN') && !defined('USE_ARCADE_ADMIN'))
				{
					set_error_handler(defined('PHPBB_MSG_HANDLER') ? PHPBB_MSG_HANDLER : 'arcade_msg_handler');
				}
			}

			if (!isset($file_functions))
			{
				if (!class_exists('file_functions'))
				{
					include($phpbb_root_path . 'arcade/includes/functions_files.' . $phpEx);
				}

				$file_functions = new file_functions();
			}
		}
	}

	function access($type = '', $admin = true)
	{
		global $user, $auth;
		global $arcade_config;

		$admin			= ($admin && $user->data['is_registered'] && $auth->acl_get('a_')) ? true : false;
		$arcade_access	= ($this->version_check() && $auth->acl_get('u_arcade') && !$this->ban_check(true) && (!$arcade_config['arcade_disable'] || $admin)) ? true : false;

		switch ($type)
		{
			case 'challenge':
				$arcade_access = ($arcade_access && $auth->acl_get('u_arcade_challenge') && (!$arcade_config['challenge_disable'] || $admin)) ? true : false;
			break;
		}

		return $arcade_access;
	}

	function session_begin()
	{
		if (!class_exists('arcade_session'))
		{
			global $phpbb_root_path, $phpEx;
			include($phpbb_root_path . 'arcade/includes/session.' . $phpEx);
		}

		if (empty($this->session))
		{
			$this->session = new arcade_session();
		}
	}

	function cat_style($cat_id, $game_id = false)
	{
		global $arcade_cache;

		$cat_style = false;

		if (!$cat_id && $game_id)
		{
			$games = $this->games();

			if (isset($games[$game_id]['cat_id']))
			{
				$cat_id = (int) $games[$game_id]['cat_id'];
			}

			unset($games);
		}

		if ($cat_id)
		{
			$cats = $arcade_cache->obtain_arcade_cats();

			if (isset($cats[$cat_id]['cat_style']))
			{
				$cat_style = (int) $cats[$cat_id]['cat_style'];
			}

			unset($cats);
		}

		return $cat_style;
	}

	function cat_style_update($style_id, $new_id = 0)
	{
		global $db, $cache;

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
				SET cat_style = ' . (int) $new_id . '
				WHERE cat_style = ' . (int) $style_id;
		$db->sql_query($sql);

		$cache->destroy('sql', ARCADE_CATS_TABLE);
		$cache->destroy('_arcade_cats');
	}

	function update_username($old_name, $new_name)
	{
		global $db;

		$update_ary = array(
			ARCADE_CATS_TABLE		=> 'cat_last_play_username',
			ARCADE_ANNOUNCE_TABLE	=> 'username'
		);

		foreach ($update_ary as $table => $field)
		{
			$sql = "UPDATE $table
					SET $field = '" . $db->sql_escape($new_name) . "'
					WHERE $field = '" . $db->sql_escape($old_name) . "'";
			$db->sql_query($sql);
		}

		$this->cache_purge('profile');
	}

	function update_user_colour($user_id_ary, $user_colour)
	{
		global $db;

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . " SET cat_last_play_user_colour = '" . $db->sql_escape($user_colour) . "'
				WHERE " . $db->sql_in_set('cat_last_play_user_id', $user_id_ary);
		$db->sql_query($sql);

		$sql = 'UPDATE ' . ARCADE_ANNOUNCE_TABLE . " SET user_colour = '" . $db->sql_escape($user_colour) . "'
				WHERE " . $db->sql_in_set('user_id', $user_id_ary);
		$db->sql_query($sql);

		$this->cache_purge('profile');
	}

	function ghost_permissions($user_id, $username)
	{
		global $user, $phpbb_root_path, $phpEx;

		$this->user();

		if (!class_exists('auth_arcade_admin'))
		{
			include($phpbb_root_path . 'arcade/includes/auth_admin.' . $phpEx);
		}

		$auth_arcade_admin = new auth_arcade_admin();
		$massage = 'ARCADE_SWITCH_' . (($auth_arcade_admin->ghost_permissions($user_id, $user->data['user_id'])) ? 'PERMISSION' : 'NO_PERMISSION');
		return (!empty($user->lang[$massage])) ? '<br />' . sprintf($user->lang[$massage], $username) : '';
	}

	function restore_perm()
	{
		global $user, $auth_arcade;

		$this->user(false);

		if (!$user->data['user_arcade_perm_from'])
		{
			return;
		}

		$auth_arcade->acl_cache($user->data);
	}

	function acl_group_delete($group_id)
	{
		global $db;

		$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . '
				WHERE group_id = ' . (int) $group_id;
		$db->sql_query($sql);
	}

	/**
	* Returns an array of ids for the specified opt
	* This is for the local arcade permissions
	*/
	function get_permissions($opts)
	{
		global $auth_arcade;

		$return = array();

		if (!is_array($opts))
		{
			$opts = array($opts);
		}

		foreach ($opts as $opt)
		{
			$return = array_merge($return, array_keys($auth_arcade->acl_getc($opt, true)));
		}

		return array_unique($return);
	}

	/**
	* Set the headers to notify the browser not to cache
	* If this isn't here the IBPro v3 games will not work
	*/
	function set_header_no_cache()
	{
		@header("HTTP/1.0 200 OK");
		@header("HTTP/1.1 200 OK");
		@header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		@header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
		@header('Pragma: no-cache');
	}

	/**
	* Generate the correct arcade url
	*
	* @param mixed $params String or array of additional url parameters
	* @param string $page The name of page
	* @param string $session_id Possibility to use a custom session id instead of the global one
	*/
	function url($params = false, $page = 'arcade', $sid = false)
	{
		global $phpbb_root_path, $phpEx;

		if ($params)
		{
			if ((strpos($params, 'img=') !== false) || (strpos($params, 'swf=') !== false))
			{
				return "{$phpbb_root_path}{$page}.{$phpEx}?{$params}";
			}
		}

		return append_sid($phpbb_root_path . $page . '.' . $phpEx, $params, true, $sid) . (($params && (strpos($params, 'mode=play') !== false || strpos($params, 'mode=random') !== false)) ? $this->gametop : '');
	}

	function back_link($params = false, $lang = 'arcade', $br2 = true)
	{
		global $user;

		$s_content_flow_begin = ($user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right';
		return (($br2) ? '<br /><br />' : '<br />') . '<a class="'. $s_content_flow_begin .'" href="' . $this->url($params) . '">' . $user->lang['ARCADE_BACK_TO_' . strtoupper($lang)] . '</a>';
	}

	function close_win()
	{
		global $user;

		return '<br /><br /><a href="#" onclick="window.close(); return false;">' . $user->lang['CLOSE_WINDOW'] . '</a>';
	}

	function gbl($use_simple_header)
	{
		return ($use_simple_header) ? $this->close_win() : $this->back_link();
	}

	function cache_purge($mode)
	{
		global $cache;

		$tables = array(USERS_TABLE);

		switch ($mode)
		{
			case 'user':
				$cache->destroy('_arcade_leaders_all');
				$cache->destroy('_arcade_challenge_leaders_all');

				$tables[] = ARCADE_REPORTS_TABLE;
				$tables[] = ARCADE_CHALLENGE_TABLE;
				$tables[] = ARCADE_CHALLENGE_CHAMP_TABLE;
			case 'play':
				$tables[] = ARCADE_GAMES_TABLE;
				$tables[] = ARCADE_SCORES_TABLE;
				$tables[] = ARCADE_PLAYS_TABLE;
			case 'profile':
				$tables[] = ARCADE_CATS_TABLE;

				if ($mode == 'play')
				{
					break;
				}

				$tables[] = ARCADE_ANNOUNCE_TABLE;

				$cache->destroy('_arcade_leaders');
				$cache->destroy('_arcade_challenge_leaders');

				if ($mode == 'profile')
				{
					break;
				}
			case 'banned':
				$cache->destroy('_arcade_ban_users');
			break;
		}

		$cache->destroy('sql', $tables);
	}

	function root_key($file)
	{
		global $phpbb_root_path;

		return str_replace($phpbb_root_path, '[ROOT]/', $file);
	}

	function bn()
	{
		global $user, $phpEx;

		$bn = false;

		if (!empty($user->page['page']))
		{
			$bn = basename($user->page['page'], '.' . $phpEx);
		}

		return $bn;
	}

	function rand_num($g_id = 1)
	{
		$rand_nums['r1'] = $this->rand();
		$rand_nums['r2'] = $this->rand($rand_nums['r1']);

		$rand_nums['rgid'] = (int) $g_id * $rand_nums['r1'] ^ $rand_nums['r2'];

		if (!$rand_nums['rgid'])
		{
			$this->rand_num();
		}

		return $rand_nums;
	}

	function rand($rand_num = false, $start_num = 2, $end_num = 200)
	{
		$start_num = intval(($start_num < 2) ? 2 : $start_num);
		$end_num = intval(($end_num < 2) ? 2 : $end_num);

		if ($start_num >= $end_num)
		{
			$end_num = ($start_num + 1);
		}

		$new_rand_num = rand($start_num, $end_num);

		if ($rand_num !== false && $rand_num == $new_rand_num)
		{
			$new_rand_num = $this->rand($rand_num, $start_num, $end_num);
		}

		return $new_rand_num;
	}

	function games()
	{
		return obtain_arcade_games();
	}

	function lang_value($key, $empty_value = false)
	{
		global $user;
		return (!empty($user->lang[$key])) ? $user->lang[$key] : (($empty_value) ? '' : $key);
	}

	function post_var($key, $format = 'string', $undefined = false)
	{
		$return = (!empty($_POST[$key])) ? true : false;

		if ($undefined)
		{
			return ($return && request_var($key, '') == 'undefined') ? true : false;
		}

		switch ($format)
		{
			case 'int':
				$return = ($return) ? (int) request_var($key, 0) : 0;
			break;

			case 'float':
				$return = ($return) ? (float) request_var($key, 0.000) : 0.000;
			break;

			default:
				$return = ($return) ? request_var($key, '') : '';
			break;
		}

		return $return;
	}

	function version_check()
	{
		global $arcade_config;
		return (phpbb_version_compare($arcade_config['version'], '2.0.RC5', '>=')) ? true : false;
	}
}

/*
*
* display_arcade()
* display_arcade_menu()
* display_arcade_highscores()
*
* obtain_arcade_games()
*
* arcade_delete_user()
*
*/

function display_arcade($function, $datas = array())
{
	global $arcade;

	if (!$function || !function_exists('display_arcade_' . $function) || !$arcade->access())
	{
		if ($function == 'menu')
		{
			global $template, $phpbb_root_path, $arcade_config;

			$template->assign_vars(array(
				'T_ARCADE_PATH'		=> "{$phpbb_root_path}arcade",

				'ARCADE_VERSION'	=> $arcade_config['version']
			));
		}

		return false;
	}

	if (is_array($datas) && sizeof($datas))
	{
		return call_user_func_array('display_arcade_' . $function, $datas);
	}
	else
	{
		call_user_func('display_arcade_' . $function);
	}
}

/**
* Display Arcade Menu
*/
function display_arcade_menu()
{
	global $user, $auth, $config, $template, $phpbb_root_path, $phpEx;
	global $arcade, $arcade_config, $arcade_plugin;

	$arcade->user();

	if ($arcade->bn() != 'portal')
	{
		$arcade_plugin->display();
	}

	$template->assign_vars(array(
		'S_DISPLAY_ARCADE'				=> true,
		'S_DISPLAY_ARCADE_CHALLENGE'	=> $arcade->access('challenge'),
		'S_ARCADE_FLASH_INTRO'			=> ($arcade_config['arcade_flash_intro']) ? 'yes' : 'no',

		'T_ARCADE_PATH'					=> "{$phpbb_root_path}arcade",

		'U_ARCADE'						=> $arcade->url(),
		'U_ARCADE_CHALLENGE'			=> $arcade->url('mode=challenge'),
		'U_ARCADE_SCORE'				=> $arcade->url('swf=score'),
		'U_ARCADE_AJAX'					=> $arcade->url(false, 'arcade/js/ajax'),

		'S_ARCADE_REPORTS'				=> ($user->data['is_registered'] && $auth->acl_get('a_arcade_utilities')) ? true : false,
		'S_ARCADE_REPORTS_OPEN'			=> ($arcade_config['reports_open']) ? true : false,

		'U_ARCADE_REPORTS_OPEN'			=> append_sid("{$phpbb_root_path}adm/index.$phpEx", 'i=arcade_utilities&amp;mode=reports', true, $user->session_id),

		'ARCADE_REPORTS_OPEN'			=> ($arcade_config['reports_open'] > 1) ? sprintf($user->lang['ARCADE_REPORTS_OPEN'], $arcade_config['reports_open']) : sprintf($user->lang['ARCADE_REPORT_OPEN'], $arcade_config['reports_open']),
		'ARCADE_FLASH_INTO_WIDTH'		=> ARCADE_FLASH_WIDTH,
		'ARCADE_FLASH_INTO_HEIGHT'		=> ARCADE_FLASH_HEIGHT,
		'ARCADE_VERSION'				=> $arcade_config['version']
	));
}

/**
* Display Arcade Highscores
*/
function display_arcade_highscores($page, $user_id, $highscore = false)
{
	static $_arcade_highscores_cache;

	if (!defined('IN_ADMIN'))
	{
		$highscore = false;
	}

	if (empty($_arcade_highscores_cache))
	{
		global $auth, $arcade, $arcade_cache, $arcade_config;

		if (($page == 'memberlist' && !$arcade_config['display_memberlist']) || ($page == 'viewtopic' && !$arcade_config['display_viewtopic']))
		{
			$_arcade_highscores_cache = true;
			return false;
		}

		$ban_users = ($highscore) ? array() : $arcade->ban_users();

		$_arcade_highscores_cache = array(
			's_viewstats'		=> ($auth->acl_get('u_arcade_viewstats')) ? true : false,

			'u_arcade_stat'		=> $arcade->url('mode=stats&amp;u={USER_ID}') . '#userbox',
			'u_challenge_stat'	=> $arcade->url('mode=view_challenge_stats&amp;u={USER_ID}') . '#userbox',

			'arcade_data'		=> array_diff_key($arcade_cache->obtain_arcade_leaders_all(), $ban_users),
			'challenge_data'	=> ($highscore || $arcade->access('challenge')) ? array_diff_key($arcade_cache->obtain_challenge_leaders_all(), $ban_users) : false,
			'ban_users'			=> $ban_users
		);

		unset($ban_users);
	}

	if (!$highscore && !empty($_arcade_highscores_cache['ban_users'][$user_id]))
	{
		return false;
	}

	$total_highscores = $a_leader_id = 0;
	$total_wins = $c_leader_id = 0;

	if (!empty($_arcade_highscores_cache['arcade_data'][$user_id]))
	{
		$total_highscores = $_arcade_highscores_cache['arcade_data'][$user_id];
		$a_leader_id = array_shift(array_keys($_arcade_highscores_cache['arcade_data']));
	}

	if (!empty($_arcade_highscores_cache['challenge_data'][$user_id]))
	{
		$total_wins = $_arcade_highscores_cache['challenge_data'][$user_id];
		$c_leader_id = array_shift(array_keys($_arcade_highscores_cache['challenge_data']));
	}

	if ($highscore == 'arcade')
	{
		return $total_highscores;
	}
	else if ($highscore == 'challenge')
	{
		return $total_wins;
	}
	else
	{
		return array(
			'S_HAS_ARCADE_DISPLAY'				=> ($_arcade_highscores_cache['arcade_data'] !== false) ? true : false,
			'S_HAS_ARCADE_CHALLENGE_DISPLAY'	=> ($_arcade_highscores_cache['challenge_data'] !== false) ? true : false,

			'S_IS_ARCADE_LEADER'				=> ($a_leader_id > 0 && $a_leader_id == $user_id) ? true : false,
			'S_ARCADE_STATS'					=> ($total_highscores && $_arcade_highscores_cache['s_viewstats']) ? true : false,

			'S_IS_ARCADE_CHALLENGE_LEADER'		=> ($c_leader_id > 0 && $c_leader_id == $user_id) ? true : false,
			'S_ARCADE_CHALLENGE_STATS'			=> ($total_wins && $_arcade_highscores_cache['s_viewstats']) ? true : false,

			'U_ARCADE_STATS'					=> str_replace('{USER_ID}', $user_id, $_arcade_highscores_cache['u_arcade_stat']),
			'U_ARCADE_CHALLENGE_STATS'			=> str_replace('{USER_ID}', $user_id, $_arcade_highscores_cache['u_challenge_stat']),

			'ARCADE_TOTAL_HIGHSCORES'			=> $total_highscores,
			'ARCADE_CHALLENGE_TOTAL_WINS'		=> $total_wins
		);
	}
}

function obtain_arcade_games()
{
	global $db, $arcade, $arcade_config;

	$arcade->user(false);

	$sql = 'SELECT game_id, game_name, game_name_clean, game_swf, game_scorevar, game_highuser, game_highdate, game_use_jackpot, cat_id
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE ' . $db->sql_in_set('cat_id', $arcade->get_permissions(array('c_view', 'c_play')), false, true) . '
			ORDER BY game_name_clean ASC';
	$result = $db->sql_query($sql, intval($arcade_config['cache_time'] * 3600));

	$games = array();
	while ($row = $db->sql_fetchrow($result))
	{
		$games[$row['game_id']] = array(
			'game_name'			=> $row['game_name'],
			'game_swf'			=> $row['game_swf'],
			'game_scorevar'		=> $row['game_scorevar'],
			'game_highuser'		=> $row['game_highuser'],
			'game_highdate'		=> $row['game_highdate'],
			'game_use_jackpot'	=> $row['game_use_jackpot'],
			'cat_id'			=> $row['cat_id']
		);
	}
	$db->sql_freeresult($result);

	return $games;
}

function arcade_delete_user($user_id)
{
	global $arcade;

	if (!defined('USE_ARCADE_ADMIN'))
	{
		define('USE_ARCADE_ADMIN', true);
	}

	if (!class_exists('arcade_admin'))
	{
		$arcade->begin();
	}

	$arcade->delete_user($user_id);
}

?>