<?php
/**
*
* @package phpBB Arcade acp
* @version $Id: acp_arcade_logs.php 126 2012-12-15 12:22:16Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class acp_arcade_logs
{
	var $u_action;
	var $tpl_name;
	var $page_title;
	var $log_type;

	function main($id, $mode)
	{
		global $db, $user, $cache, $auth, $config, $template, $phpbb_admin_path, $phpbb_root_path, $phpEx;
		global $arcade, $auth_arcade, $arcade_cache, $arcade_config, $arcade_plugin;

		define('IN_PHPBB_ARCADE', true);
		$arcade->begin();

		$arcade_plugin->add_lang(array('mods/arcade/logs', 'mods/info_mcp_arcade'), false, $arcade_cache->obtain_arcade_plugins());

		// Set up general vars
		$ip			= request_var('ip', 'ip');
		$action		= request_var('action', '');
		$cat_id		= request_var('c', 0);
		$start		= request_var('start', 0);
		$deletemark = (!empty($_POST['delmarked'])) ? true : false;
		$deleteall	= (!empty($_POST['delall'])) ? true : false;
		$marked		= request_var('mark', array(0));

		// Sort keys
		$sort_days	= request_var('st', 0);
		$sort_key	= request_var('sk', 't');
		$sort_dir	= request_var('sd', 'd');

		$this->tpl_name = 'arcade/acp_logs';
		$this->log_type = constant('ARCADE_LOG_' . strtoupper($mode));

		// Delete entries if requested and able
		if (($deletemark || $deleteall) && $auth->acl_get('a_arcade_clearlogs'))
		{
			if ($deletemark && !sizeof($marked))
			{
				trigger_error($user->lang['ARCADE_SELECT_EMPTY'] . adm_back_link($this->u_action), E_USER_WARNING);
			}

			if (confirm_box(true))
			{
				$where_sql = '';

				if ($deletemark && sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
					}
					$where_sql = ' AND ' . $db->sql_in_set('log_id', $sql_in);
					unset($sql_in);
				}

				if ($where_sql || $deleteall)
				{
					$sql = 'DELETE FROM ' . ARCADE_LOGS_TABLE . "
							WHERE log_type = {$this->log_type}
							$where_sql";
					$db->sql_query($sql);

					$arcade->add_log('admin', 'LOG_CLEAR_' . strtoupper($mode));
				}
			}
			else
			{
				confirm_box(false, $user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'c'			=> $cat_id,
					'start'		=> $start,
					'delmarked'	=> $deletemark,
					'delall'	=> $deleteall,
					'mark'		=> $marked,
					'st'		=> $sort_days,
					'sk'		=> $sort_key,
					'sd'		=> $sort_dir,
					'i'			=> $id,
					'mode'		=> $mode,
					'action'	=> $action))
				);
			}
		}

		// Sorting
		$limit_days = array(0 => $user->lang['ALL_ENTRIES'], 1 => $user->lang['1_DAY'], 7 => $user->lang['7_DAYS'], 14 => $user->lang['2_WEEKS'], 30 => $user->lang['1_MONTH'], 90 => $user->lang['3_MONTHS'], 180 => $user->lang['6_MONTHS'], 365 => $user->lang['1_YEAR']);
		$sort_by_text = array('u' => $user->lang['SORT_USERNAME'], 't' => $user->lang['SORT_DATE'], 'i' => $user->lang['SORT_IP'], 'o' => $user->lang['SORT_ACTION']);
		$sort_by_sql = array('u' => 'u.username_clean', 't' => 'l.log_time', 'i' => 'l.log_ip', 'o' => 'l.log_operation');

		$s_limit_days = $s_sort_key = $s_sort_dir = $u_sort_param = '';
		gen_sort_selects($limit_days, $sort_by_text, $sort_days, $sort_key, $sort_dir, $s_limit_days, $s_sort_key, $s_sort_dir, $u_sort_param);

		// Define where and sort sql for use in displaying logs
		$sql_where = ($sort_days) ? (time() - ($sort_days * 86400)) : 0;
		$sql_sort = $sort_by_sql[$sort_key] . ' ' . (($sort_dir == 'd') ? 'DESC' : 'ASC');

		$keywords = utf8_normalize_nfc(request_var('keywords', '', true));
		$keywords_param = !empty($keywords) ? '&amp;keywords=' . urlencode(htmlspecialchars_decode($keywords)) : '';

		$l_title = $user->lang['ACP_ARCADE_LOGS_' . strtoupper($mode)];
		$l_title_explain = $user->lang['ACP_ARCADE_LOGS_' . strtoupper($mode) . '_EXPLAIN'];

		$this->page_title = $l_title;

		// Define category list if we're looking @ mod logs
		if ($mode == 'mod')
		{
			$cat_box = '<option value="0">' . $user->lang['ALL_CATS'] . '</option>' . $arcade->make_cat_select($cat_id);

			$template->assign_vars(array(
				'S_SHOW_CATS'	=> true,
				'S_CAT_BOX'		=> $cat_box
			));
		}

		// Grab log data
		$log_data = array();
		$log_count = 0;
		$start = $arcade->view_log($mode, $log_data, $log_count, $arcade_config['acp_items_per_page'], $start, $cat_id, 0, $sql_where, $sql_sort, $keywords);

		$template->assign_vars(array(
			'L_TITLE'		=> $l_title,
			'L_EXPLAIN'		=> $l_title_explain,
			'U_ACTION'		=> $this->u_action . "&amp;$u_sort_param$keywords_param&amp;start=$start",

			'S_ON_PAGE'		=> on_page($log_count, $arcade_config['acp_items_per_page'], $start),
			'PAGINATION'	=> generate_pagination($this->u_action . "&amp;$u_sort_param$keywords_param", $log_count, $arcade_config['acp_items_per_page'], $start, true),

			'S_LIMIT_DAYS'	=> $s_limit_days,
			'S_SORT_KEY'	=> $s_sort_key,
			'S_SORT_DIR'	=> $s_sort_dir,
			'S_CLEARLOGS'	=> $auth->acl_get('a_arcade_clearlogs'),
			'S_KEYWORDS'	=> $keywords,
		));

		foreach ($log_data as $row)
		{
			$data = array();
			$checks = array('viewlogs', 'viewcat', 'viewgame', 'editgame');

			foreach ($checks as $check)
			{
				if (isset($row[$check]) && $row[$check])
				{
					$data[] = '<a href="' . $row[$check] . '">' . $user->lang['ARCADE_LOGVIEW_' . strtoupper($check)] . '</a>';
				}
			}

			$template->assign_block_vars('log', array(
				'U_IP'		=> $this->u_action . "&amp;ip=" . (($ip == 'ip') ? 'hostname' : 'ip'),
				'U_WHOIS'	=> append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_utilities&amp;action=whois&amp;whois_ip={$row['ip']}"),

				'USERNAME'	=> $row['username_full'],
				'IP'		=> ($ip == 'hostname') ? gethostbyaddr($row['ip']) : $row['ip'],
				'DATE'		=> $user->format_date($row['time']),
				'ACTION'	=> $row['action'],
				'DATA'		=> (sizeof($data)) ? implode(' | ', $data) : '',
				'ID'		=> $row['id'],
			));
		}
	}
}

?>