<?php
/**
*
* @package phpBB Arcade acp
* @version $Id: acp_arcade_manage.php 160 2012-12-29 19:01:08Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class acp_arcade_manage
{
	var $u_action;
	var $tpl_name;
	var $page_title;
	var $parent_id = 0;
	var $new_game_data = array();

	function main($id, $mode)
	{
		global $db, $user, $cache, $auth, $config, $template, $phpbb_admin_path, $phpbb_root_path, $phpEx;
		global $arcade, $auth_arcade, $arcade_cache, $arcade_config, $file_functions;

		define('IN_PHPBB_ARCADE', true);
		$arcade->begin();

		$this->tpl_name = 'arcade/acp_manage';
		$action = request_var('action', '');
		$update = (request_var('update', false)) ? true : false;
		$total_champ_data = (int) request_var('total_champ_data', 0);

		$errors = array();
		$page_title = 'ACP_ARCADE_MANAGE';
		$return = false;

		switch ($mode)
		{
			case 'category':
				$form_key = 'acp_arcade_category';
				add_form_key($form_key);
				$template->assign_var('S_MANAGE_CAT', true);

				$confirm_action = array('delete_marked', 'g_delete', 'reset_marked', 'reset_scores_marked', 'move_marked', 'reset_install_date_marked');
				$start = request_var('start', 0);

				$cat_data = array();
				if ($update && !check_form_key($form_key) && !in_array($action, $confirm_action))
				{
					$update = false;
					$errors[] = $user->lang['FORM_INVALID'];
				}

				$page_title = 'ACP_ARCADE_MANAGE_CATEGORIES';

				$cat_id				= request_var('c', 0);
				$game_id			= request_var('g', 0);
				$this->parent_id	= request_var('parent_id', 0);
				$game_ids			= request_var('game_ids', array(0));

				// Check additional permissions
				switch ($action)
				{
					case 'progress_bar':
						$arcade->display_progress_bar();
					break;

					case 'delete':
						if (!$auth->acl_get('a_arcade_delete_cat'))
						{
							trigger_error($user->lang['NO_PERMISSION_CAT_DELETE'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'delete_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}
					case 'g_delete':
						if (!$auth->acl_get('a_arcade_delete_game'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_DELETE'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'reset_marked':
						$auth_check = ($auth->acl_get('a_arcade_game')) ? true : false;
					case 'reset_scores_marked':
						if ($action == 'reset_scores_marked')
						{
							$auth_check = true;
						}

						$auth_check = ($auth_check && $auth->acl_get('a_arcade_user')) ? true : false;

						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}

						if (!$auth_check)
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_' . (($action == 'reset_marked') ? 'RESET' : 'SCORE_RESET')] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'sync_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							$update = false;
						}
					break;

					case 'add':
						if (!$auth->acl_get('a_arcade_cat'))
						{
							trigger_error($user->lang['NO_PERMISSION_CAT_ADD'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'reset_install_date_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}

						if (!$auth->acl_get('a_arcade_user') || !$auth->acl_get('a_arcade_game'))
						{
							trigger_error($user->lang['NO_PERMISSION_GAME_INSTALL_DATE_RESET'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;
				}

				// Major routines
				if ($update)
				{
					switch ($action)
					{
						case 'delete':
							$action_subcats		= request_var('action_subcats', '');
							$subcats_to_id		= request_var('subcats_to_id', 0);
							$action_games		= request_var('action_games', '');
							$games_to_id		= request_var('games_to_id', 0);
							$delete_files		= (request_var('delete_files', false)) ? true : false;
							$sub_delete_files	= (request_var('sub_delete_files', false)) ? true : false;

							$errors = $this->delete_cat($cat_id, $action_games, $delete_files, $action_subcats, $sub_delete_files, $games_to_id, $subcats_to_id);

							if (sizeof($errors))
							{
								break;
							}

							$auth_arcade->acl_clear_prefetch();

							$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
							$cache->destroy('_arcade_cats');
							$cache->destroy('_arcade_leaders');
							$cache->destroy('_arcade_leaders_all');

							trigger_error($user->lang['CAT_DELETED'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
						break;

						case 'g_delete':
							if (confirm_box(true))
							{
								$delete_files = (request_var('delete_files', false)) ? true : false;

								$game_data = $arcade->get_game_data($game_id);
								$errors = $arcade->delete_game($game_id, $game_data['cat_id'], $delete_files, $game_data['game_swf']);

								if (sizeof($errors))
								{
									break;
								}

								$arcade->add_log('admin', 'LOG_ARCADE_DELETE_GAME', $game_data['game_name']);

								$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE, ARCADE_CHALLENGE_TABLE, ARCADE_CHALLENGE_CHAMP_TABLE));
								$cache->destroy('_arcade_cats');
								$cache->destroy('_arcade_leaders');
								$cache->destroy('_arcade_leaders_all');
								$cache->destroy('_arcade_games_filesize');
								$cache->destroy('_arcade_challenge_leaders');
								$cache->destroy('_arcade_challenge_leaders_all');

								trigger_error(sprintf($user->lang['GAME_DELETED'], $game_data['game_name']) . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : '')));
							}
						break;

						case 'move_marked':
							if (confirm_box(true))
							{
								$game_data = $arcade->get_game_data($game_ids);
								$game_data_array = array();
								foreach ($game_data as $key => $value)
								{
									$game_data_array[] = $game_data[$key]['game_name'];
								}
								$game_name_list = implode(', ', $game_data_array);

								$to_cat_id = request_var('to_cat_id', 0);

								$errors = $arcade->move_game($game_ids, $this->parent_id, $to_cat_id);
								if (sizeof($errors))
								{
									break;
								}

								$old_cat_name = $arcade->get_cat_field($this->parent_id, 'cat_name');
								$new_cat_name = $arcade->get_cat_field($to_cat_id, 'cat_name');

								if (sizeof($game_data_array) > 1)
								{
									$message = sprintf($user->lang['GAMES_MOVED'], $game_name_list);
									$arcade->add_log('admin', 'LOG_ARCADE_MOVE_GAMES', $old_cat_name, $new_cat_name, $game_name_list);
								}
								else
								{
									$message = sprintf($user->lang['GAME_MOVED'], $game_name_list);
									$arcade->add_log('admin', 'LOG_ARCADE_MOVE_GAME', $old_cat_name, $new_cat_name, $game_name_list);
								}

								$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
								$cache->destroy('_arcade_leaders');
								$cache->destroy('_arcade_leaders_all');

								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
							}
						break;

						case 'reset_install_date_marked':
							if (confirm_box(true))
							{
								$game_data = $arcade->get_game_data($game_ids);
								$game_data_array = array();

								foreach ($game_data as $key => $value)
								{
									$game_data_array[] = $game_data[$key]['game_name'];
								}

								unset($game_data);
								$errors = $arcade->reset_game($action, $game_ids, $this->parent_id);

								if (sizeof($errors))
								{
									break;
								}

								$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
								$cache->destroy('_arcade_leaders');
								$cache->destroy('_arcade_leaders_all');

								$game_name_list = implode(', ', $game_data_array);
								$arcade->add_log('admin', 'LOG_ARCADE_RESET_INSTALL_DATE_GAME' . ((sizeof($game_data_array) > 1) ? 'S' : ''), $game_name_list);
								trigger_error(sprintf($user->lang['RESET_INSTALL_DATE_GAME' . ((sizeof($game_data_array) > 1) ? 'S' : '') . '_SUCCESS'], $game_name_list) . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
							}
						break;

						case 'sync_marked':
							$arcade->sync('game', $game_ids);
							$arcade->sync('rating', $game_ids);
							$game_data = $arcade->get_game_data($game_ids);

							$game_data_array = array();
							foreach ($game_data as $game)
							{
								$arcade->update_highscore($game);
								$game_data_array[] = $game['game_name'];
							}

							unset($game_data);
							$game_name_list = implode(', ', $game_data_array);

							if (sizeof($game_data_array) > 1)
							{
								$message = sprintf($user->lang['GAMES_RESYNCED'], $game_name_list);
								$arcade->add_log('admin', 'LOG_ARCADE_SYNC_GAMES', $game_name_list);
							}
							else
							{
								$message = sprintf($user->lang['GAME_RESYNCED'], $game_name_list);
								$arcade->add_log('admin', 'LOG_ARCADE_SYNC_GAME', $game_name_list);
							}

							$template->assign_var('L_CAT_RESYNCED', $message);
						break;

						case 'reset_marked':
						case 'reset_scores_marked':
							if (($total_champ_data && request_var('process', false)) || (confirm_box(true)))
							{
								$super_champion_process = false;
								if ($total_champ_data && $arcade->proces_champion_users($action, $total_champ_data, $this->u_action, false, $game_ids, $this->parent_id))
								{
									$super_champion_process = true;
								}

								if (!$super_champion_process)
								{
									$game_data = $arcade->get_game_data($game_ids);
									$game_data_array = array();
									foreach ($game_data as $key => $value)
									{
										$game_data_array[] = $game_data[$key]['game_name'];
									}
									$game_name_list = implode(', ', $game_data_array);

									$message = sprintf($user->lang['GAME' . ((sizeof($game_data_array) > 1) ? 'S' : '') . '_' . (($action == 'reset_marked') ? 'RESET' : 'SCORES_RESET')], $game_name_list);

									$errors = $arcade->reset_game($action, $game_ids, $this->parent_id);
									if (sizeof($errors))
									{
										$action = '';
										break;
									}

									$reset_log = 'LOG_ARCADE_RESET_GAME' . ((sizeof($game_data_array) > 1) ? 'S' : '');

									$arcade->add_log('admin', $reset_log . (($action == 'reset_marked') ? '' : '_SCORES'), $game_name_list);

									$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
									$cache->destroy('_arcade_leaders');
									$cache->destroy('_arcade_leaders_all');

									$arcade->sync('total_data', 'all');

									trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : '')));
								}
								else
								{
									$action = '';
									$return = true;
								}
							}
						break;

						case 'delete_marked':
							if (confirm_box(true))
							{
								$delete_files = (request_var('delete_files', false)) ? true : false;

								$game_data = $arcade->get_game_data($game_ids);

								$game_data_array = $games_swf = array();

								if (!empty($game_data))
								{
									foreach ($game_data as $key => $value)
									{
										$game_data_array[] = $game_data[$key]['game_name'];
										$games_swf[] = $game_data[$key]['game_swf'];
									}
								}

								$errors = $arcade->delete_game($game_ids, $this->parent_id, $delete_files, $games_swf);

								if (sizeof($errors))
								{
									break;
								}

								$game_name_list = implode(', ', $game_data_array);

								if (sizeof($game_data_array) > 1)
								{
									$message = sprintf($user->lang['GAMES_DELETED'], $game_name_list);
									$arcade->add_log('admin', 'LOG_ARCADE_DELETE_GAMES', $game_name_list);
								}
								else
								{
									$message = sprintf($user->lang['GAME_DELETED'], $game_name_list);
									$arcade->add_log('admin', 'LOG_ARCADE_DELETE_GAME', $game_name_list);
								}

								$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE, ARCADE_CHALLENGE_TABLE, ARCADE_CHALLENGE_CHAMP_TABLE));
								$cache->destroy('_arcade_cats');
								$cache->destroy('_arcade_leaders');
								$cache->destroy('_arcade_leaders_all');
								$cache->destroy('_arcade_games_filesize');
								$cache->destroy('_arcade_challenge_leaders');
								$cache->destroy('_arcade_challenge_leaders_all');

								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : '')));
							}
						break;

						case 'edit':
							$cat_data = array(
								'cat_id' => $cat_id
							);

						// No break here
						case 'add':
							$cat_data += array(
								'parent_id'				=> request_var('cat_parent_id', $this->parent_id),
								'cat_type'				=> request_var('cat_type', ARCADE_CAT_GAMES),
								'cat_test'				=> request_var('cat_test', false),
								'type_action'			=> request_var('type_action', ''),
								'cat_status'			=> request_var('cat_status', ITEM_UNLOCKED),
								'cat_parents'			=> '',
								'cat_name'				=> utf8_normalize_nfc(request_var('cat_name', '', true)),
								'cat_link'				=> request_var('cat_link', ''),
								'cat_link_track'		=> request_var('cat_link_track', false),
								'cat_desc'				=> utf8_normalize_nfc(request_var('cat_desc', '', true)),
								'cat_age'				=> request_var('cat_age', 0),
								'cat_desc_uid'			=> '',
								'cat_desc_options'		=> 7,
								'cat_desc_bitfield'		=> '',
								'cat_rules'				=> utf8_normalize_nfc(request_var('cat_rules', '', true)),
								'cat_rules_uid'			=> '',
								'cat_rules_options'		=> 7,
								'cat_rules_bitfield'	=> '',
								'cat_rules_link'		=> request_var('cat_rules_link', ''),
								'cat_image'				=> request_var('cat_image', ''),
								'cat_display'			=> request_var('cat_display', 0),
								'cat_style'				=> request_var('cat_style', 0),
								'cat_games_per_page'	=> request_var('cat_games_per_page', 0),
								'cat_download'			=> request_var('cat_download', true),
								'cat_use_jackpot'		=> request_var('cat_use_jackpot', false),
								'cat_download_cost'		=> request_var('cat_download_cost', 0.000),
								'cat_cost'				=> request_var('cat_cost', 0.000),
								'cat_reward'			=> request_var('cat_reward', 0.000),
								'cat_password'			=> request_var('cat_password', ''),
								'cat_password_confirm'	=> request_var('cat_password_confirm', ''),
								'cat_password_unset'	=> request_var('cat_password_unset', false),
								'display_subcat_list'	=> request_var('display_subcat_list', false),
								'display_on_index'		=> request_var('display_on_index', false),
							);

							// Use link_display_on_index setting if category type is link
							if ($cat_data['cat_type'] == ARCADE_LINK)
							{
								$cat_data['display_on_index'] = request_var('link_display_on_index', false);
							}

							// Linked categories and categories are not able to be locked...
							if ($cat_data['cat_type'] == ARCADE_LINK || $cat_data['cat_type'] == ARCADE_CAT)
							{
								// Linked categories are not able to be locked...
								$cat_data['cat_status'] = ITEM_UNLOCKED;
							}

							// Get data for category rules if specified...
							if ($cat_data['cat_rules'])
							{
								generate_text_for_storage($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options'], request_var('rules_parse_bbcode', false), request_var('rules_parse_urls', false), request_var('rules_parse_smilies', false));
							}

							// Get data for category description if specified
							if ($cat_data['cat_desc'])
							{
								generate_text_for_storage($cat_data['cat_desc'], $cat_data['cat_desc_uid'], $cat_data['cat_desc_bitfield'], $cat_data['cat_desc_options'], request_var('desc_parse_bbcode', false), request_var('desc_parse_urls', false), request_var('desc_parse_smilies', false));
							}

							$errors = $this->update_cat_data($cat_data);

							if (!sizeof($errors))
							{
								$cat_perm_from = request_var('cat_perm_from', 0);
								$cache->destroy('sql', ARCADE_CATS_TABLE);
								$cache->destroy('_arcade_cats');

								$copied_permissions = false;
								// Copy permissions?
								if ($cat_perm_from && $cat_perm_from != $cat_data['cat_id'] &&
									($action != 'edit' || empty($cat_id) || ($auth->acl_get('a_cauth') && $auth->acl_get('a_authusers') && $auth->acl_get('a_authgroups') && $auth->acl_get('a_mauth'))))
								{
									$arcade->copy_category_permissions($cat_perm_from, $cat_data['cat_id'], ($action == 'edit') ? true : false);
									$copied_permissions = true;
								}

								$auth_arcade->acl_clear_prefetch();

								$acl_url = '&amp;mode=setting_category_local&amp;cat_id[]=' . $cat_data['cat_id'];

								$message = ($action == 'add') ? $user->lang['CAT_CREATED'] : $user->lang['CAT_UPDATED'];

								// redirect directly to permission settings screen if authed
								if ($action == 'add' && !$copied_permissions && $auth->acl_get('a_cauth'))
								{
									$message .= '<br /><br />' . sprintf($user->lang['REDIRECT_ACL'], '<a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_permissions' . $acl_url) . '">', '</a>');

									meta_refresh(4, append_sid("{$phpbb_admin_path}index.$phpEx", 'i=arcade_permissions' . $acl_url));
								}

								trigger_error($message . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id));
							}
						break;
					}
				}

				switch ($action)
				{
					case 'move_up':
					case 'move_down':
						if (!$cat_id)
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$sql = 'SELECT *
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_id = ' . (int) $cat_id;
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$row)
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$move_cat_name = $this->move_cat_by($row, $action, 1);

						if ($move_cat_name !== false)
						{
							$arcade->add_log('admin', 'LOG_ARCADE_' . strtoupper($action), $row['cat_name'], $move_cat_name);
							$cache->destroy('sql', ARCADE_CATS_TABLE);
						}
					break;

					case 'g_move_up':
					case 'g_move_down':
						if (!$game_id)
						{
							trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$sql = 'SELECT game_name
								FROM ' . ARCADE_GAMES_TABLE . '
								WHERE game_id = ' . (int) $game_id;
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$row)
						{
							trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						//
						// Change order of games in the DB
						//
						$arcade->move_game_by($game_id, $this->parent_id, $action);
						$arcade->add_log('admin', 'LOG_ARCADE_' . strtoupper($action), $row['game_name']);
						$cache->destroy('sql', ARCADE_GAMES_TABLE);
					break;

					case 'g_delete':
						$s_hidden_fields = array(
							'start'		=> $start,
							'g'			=> $game_id,
							'update'	=> true
						);

						$arcade->confirm_box('delete_files', 'DELETE_SELECTED_GAME', $s_hidden_fields);
					break;


					case 'delete_marked':
						$s_hidden_fields = array(
							'start'		=> $start,
							'action'	=> 'delete_marked',
							'update'	=> true,
							'game_ids'	=> $game_ids
						);

						$arcade->confirm_box('delete_files', 'DELETE_SELECTED_GAMES', $s_hidden_fields);
					break;

					case 'move_marked':
						if (empty($game_ids))
						{
							$errors[] = $user->lang['NO_GAME_IDS'];
							break;
						}

						$s_hidden_fields = array(
							'action'	=> 'move_marked',
							'update'	=> true,
							'game_ids'	=> $game_ids,
						);

						$template->assign_var('S_CATEGORY_SELECT', $arcade->make_cat_select($this->parent_id, $this->parent_id, true));

						confirm_box(false, 'MOVE_SELECTED_GAMES', build_hidden_fields($s_hidden_fields), 'arcade/confirm_move_games.html');
					break;

					case 'reset_install_date_marked':
						$s_hidden_fields = array(
							'action'			=> $action,
							'update'			=> true,
							'game_ids'			=> $game_ids
						);

						$confirm = 'RESET_INSTALL_DATE_SELECTED_GAME' . ((sizeof($game_ids) > 1) ? 'S' : '');

						confirm_box(false, $confirm, build_hidden_fields($s_hidden_fields));
					break;

					case 'reset_marked':
					case 'reset_scores_marked':
						if (!$total_champ_data)
						{
							$total_champ_data = $arcade->get_total('update_super_champion', $game_ids);
						}

						$s_hidden_fields = array(
							'start'				=> $start,
							'action'			=> $action,
							'update'			=> true,
							'game_ids'			=> $game_ids,
							'total_champ_data'	=> $total_champ_data
						);

						$confirm = 'RESET_SELECTED_GAME' . ((sizeof($game_ids) > 1) ? 'S' : '');

						if ($action == 'reset_marked')
						{
							confirm_box(false, $confirm, build_hidden_fields($s_hidden_fields));
						}
						else
						{
							$arcade->confirm_box('retain_plays', "{$confirm}_SCORES", $s_hidden_fields);
						}
					break;

					case 'sync':
						if ($cat_id || $game_id)
						{
							if (!$cat_id && !$game_id)
							{
								trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
							}

							@set_time_limit(0);

							if ($cat_id)
							{
								$cat_name = $arcade->get_cat_field($cat_id, 'cat_name');

								$arcade->set_last_play($cat_id);
								$arcade->sync('category', $cat_id);
								$cache->destroy('sql', ARCADE_CATS_TABLE);

								$arcade->add_log('admin', 'LOG_ARCADE_SYNC_CAT', $cat_name);
								$template->assign_var('L_CAT_RESYNCED', sprintf($user->lang['CAT_RESYNCED'], $cat_name));
							}
							else if ($game_id)
							{
								$sql = 'SELECT game_name
										FROM ' . ARCADE_GAMES_TABLE . '
										WHERE game_id = ' . (int) $game_id;
								$result = $db->sql_query($sql);
								$row = $db->sql_fetchrow($result);
								$db->sql_freeresult($result);

								if (!$row)
								{
									trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
								}

								$arcade->sync('game', $game_id);
								$cache->destroy('sql', ARCADE_GAMES_TABLE);

								$arcade->add_log('admin', 'LOG_ARCADE_SYNC_GAME', $row['game_name']);
								$template->assign_var('L_CAT_RESYNCED', sprintf($user->lang['GAME_RESYNCED'], $row['game_name']));
							}
						}
						else
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}
					break;

					case 'add':
					case 'edit':
						if ($update)
						{
							$cat_data['cat_flags'] = 0;
							$cat_data['cat_flags'] += (request_var('cat_link_track', false)) ? ARCADE_FLAG_LINK_TRACK : 0;
						}

						// Show form to create/modify a category
						if ($action == 'edit')
						{
							$page_title = 'EDIT_CAT';
							$row = $arcade->get_cat_info($cat_id);
							$old_cat_type = $row['cat_type'];

							if (!$update)
							{
								$cat_data = $row;
							}
							else
							{
								$cat_data['left_id'] = $row['left_id'];
								$cat_data['right_id'] = $row['right_id'];
							}

							// Make sure no direct child categories are able to be selected as parents.
							$childs = $arcade->get_cat_branch($cat_id, 'children');

							$exclude_cat = array();
							foreach ($childs as $row)
							{
								$exclude_cat[] = $row['cat_id'];
							}

							$parents_list = $arcade->make_cat_select($cat_data['parent_id'], $exclude_cat, false, false, false);
							$cat_data['cat_password_confirm'] = $cat_data['cat_password'];
						}
						else
						{
							$page_title = 'CREATE_CAT';

							$cat_id = $this->parent_id;
							$parents_list = $arcade->make_cat_select($this->parent_id, false, false, false, false);

							// Fill category data with default values
							if (!$update)
							{
								$cat_data = array(
									'parent_id'				=> $this->parent_id,
									'cat_type'				=> ARCADE_CAT_GAMES,
									'cat_test'				=> false,
									'cat_status'			=> ITEM_UNLOCKED,
									'cat_name'				=> utf8_normalize_nfc(request_var('cat_name', '', true)),
									'cat_age'				=> 0,
									'cat_link'				=> '',
									'cat_link_track'		=> false,
									'cat_desc'				=> '',
									'cat_rules'				=> '',
									'cat_rules_link'		=> '',
									'cat_image'				=> '',
									'cat_display'			=> 0,
									'cat_style'				=> 0,
									'cat_games_per_page'	=> 0,
									'cat_download'			=> true,
									'cat_use_jackpot'		=> false,
									'cat_download_cost'		=> 0,
									'cat_cost'				=> 0,
									'cat_reward'			=> 0,
									'cat_flags'				=> 0,
									'cat_password'			=> '',
									'cat_password_confirm'	=> '',
									'display_subcat_list'	=> true,
									'display_on_index'		=> true,
								);
							}
						}

						$cat_rules_data = array(
							'text'			=> $cat_data['cat_rules'],
							'allow_bbcode'	=> true,
							'allow_smilies'	=> true,
							'allow_urls'	=> true
						);

						$cat_desc_data = array(
							'text'			=> $cat_data['cat_desc'],
							'allow_bbcode'	=> true,
							'allow_smilies'	=> true,
							'allow_urls'	=> true
						);

						$cat_rules_preview = '';

						// Parse rules if specified
						if ($cat_data['cat_rules'])
						{
							if (!isset($cat_data['cat_rules_uid']))
							{
								// Before we are able to display the preview and plane text, we need to parse our request_var()'d value...
								$cat_data['cat_rules_uid'] = '';
								$cat_data['cat_rules_bitfield'] = '';
								$cat_data['cat_rules_options'] = 0;

								generate_text_for_storage($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options'], request_var('rules_allow_bbcode', false), request_var('rules_allow_urls', false), request_var('rules_allow_smiliess', false));
							}

							// Generate preview content
							$cat_rules_preview = generate_text_for_display($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_bitfield'], $cat_data['cat_rules_options']);

							// decode...
							$cat_rules_data = generate_text_for_edit($cat_data['cat_rules'], $cat_data['cat_rules_uid'], $cat_data['cat_rules_options']);
						}

						// Parse desciption if specified
						if ($cat_data['cat_desc'])
						{
							if (!isset($cat_data['cat_desc_uid']))
							{
								// Before we are able to display the preview and plane text, we need to parse our request_var()'d value...
								$cat_data['cat_desc_uid'] = '';
								$cat_data['cat_desc_bitfield'] = '';
								$cat_data['cat_desc_options'] = 0;

								generate_text_for_storage($cat_data['cat_desc'], $cat_data['cat_desc_uid'], $cat_data['cat_desc_bitfield'], $cat_data['cat_desc_options'], request_var('desc_allow_bbcode', false), request_var('desc_allow_urls', false), request_var('desc_allow_smiliess', false));
							}

							// decode...
							$cat_desc_data = generate_text_for_edit($cat_data['cat_desc'], $cat_data['cat_desc_uid'], $cat_data['cat_desc_options']);
						}

						$cat_type_options = '';
						$cat_type_ary = array(ARCADE_CAT => 'CAT', ARCADE_CAT_GAMES => 'CAT_GAMES', ARCADE_LINK => 'LINK');

						foreach ($cat_type_ary as $value => $lang)
						{
							$cat_type_options .= '<option value="' . $value . '"' . (($value == $cat_data['cat_type']) ? ' selected="selected"' : '') . '>' . $user->lang['TYPE_' . $lang] . '</option>';
						}

						$styles_list = style_select($cat_data['cat_style'], true);
						$cat_display_list = $this->cat_display_select($cat_data['cat_display']);
						$statuslist = '<option value="' . ITEM_UNLOCKED . '"' . (($cat_data['cat_status'] == ITEM_UNLOCKED) ? ' selected="selected"' : '') . '>' . $user->lang['UNLOCKED'] . '</option><option value="' . ITEM_LOCKED . '"' . (($cat_data['cat_status'] == ITEM_LOCKED) ? ' selected="selected"' : '') . '>' . $user->lang['LOCKED'] . '</option>';

						$sql = 'SELECT cat_id
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_type = ' . ARCADE_CAT_GAMES . '
								AND cat_id <> ' . (int) $cat_id;
						$result = $db->sql_query_limit($sql, 1);

						$arcade_cat_games_exists = false;
						if ($db->sql_fetchrow($result))
						{
							$arcade_cat_games_exists = true;
						}
						$db->sql_freeresult($result);

						// Subcategory move options
						if ($action == 'edit' && $cat_data['cat_type'] == ARCADE_CAT)
						{
							$subcat_id = array();
							$subcat = $arcade->get_cat_branch($cat_id, 'children');

							foreach ($subcat as $row)
							{
								$subcat_id[] = $row['cat_id'];
							}

							$cat_list = $arcade->make_cat_select($cat_data['parent_id'], $subcat_id);

							if ($arcade_cat_games_exists)
							{
								$template->assign_var('S_MOVE_CAT_OPTIONS', $arcade->make_cat_select($cat_data['parent_id'], $subcat_id));
							}
							$db->sql_freeresult($result);

							$template->assign_vars(array(
								'S_HAS_SUBCATS'	=> ($cat_data['right_id'] - $cat_data['left_id'] > 1) ? true : false,
								'S_CATS_LIST'	=> $cat_list
							));
						}
						else if ($arcade_cat_games_exists)
						{
							$template->assign_var('S_MOVE_CAT_OPTIONS', $arcade->make_cat_select($cat_data['parent_id'], $cat_id, false, true, false));
						}

						$s_show_display_on_index = false;

						if ($cat_data['parent_id'] > 0)
						{
							// if this category is a subcategory put the "display on index" checkbox
							if ($parent_info = $arcade->get_cat_info($cat_data['parent_id']))
							{
								if ($parent_info['parent_id'] > 0 || $parent_info['cat_type'] == ARCADE_CAT)
								{
									$s_show_display_on_index = true;
								}
							}
						}

						$l_title = $user->lang[$page_title];
						$filename_list = $arcade->generate_cat_images($cat_data['cat_image']);

						$template->assign_vars(array(
							'S_EDIT_CAT'					=> true,
							'S_PARENT_ID'					=> $this->parent_id,
							'S_CAT_PARENT_ID'				=> $cat_data['parent_id'],
							'S_ADD_ACTION'					=> ($action == 'add') ? true : false,

							'U_BACK'						=> $this->u_action . '&amp;parent_id=' . $this->parent_id,
							'U_EDIT_ACTION'					=> $this->u_action . "&amp;parent_id={$this->parent_id}&amp;action=$action&amp;c=$cat_id",

							'L_COPY_PERMISSIONS_EXPLAIN'	=> $user->lang['COPY_PERMISSIONS_' . strtoupper($action) . '_EXPLAIN'],

							'CAT_NAME'						=> $cat_data['cat_name'],
							'CAT_DATA_LINK'					=> $cat_data['cat_link'],
							'CAT_IMAGE'						=> $cat_data['cat_image'],
							'S_FILENAME_OPTIONS'			=> $filename_list,
							'S_IMAGE_BASEDIR'				=> $phpbb_root_path . $arcade_config['cat_image_path'],
							'CAT_IMAGE_SRC'					=> ($cat_data['cat_image']) ? $phpbb_root_path . $arcade_config['cat_image_path'] . $cat_data['cat_image'] : $phpbb_root_path . 'images/spacer.gif',
							'ARCADE_LINK'					=> ARCADE_LINK,
							'ARCADE_CAT'					=> ARCADE_CAT,
							'ARCADE_CAT_GAMES'				=> ARCADE_CAT_GAMES,
							'CAT_AGE'						=> $cat_data['cat_age'],
							'CAT_GAMES_PER_PAGE'			=> $cat_data['cat_games_per_page'],
							'CAT_DOWNLOAD'					=> $cat_data['cat_download'],
							'CAT_PASSWORD'					=> $cat_data['cat_password'],
							'CAT_PASSWORD_CONFIRM'			=> $cat_data['cat_password_confirm'],
							'CAT_RULES_LINK'				=> $cat_data['cat_rules_link'],
							'CAT_RULES'						=> $cat_data['cat_rules'],
							'CAT_RULES_PREVIEW'				=> $cat_rules_preview,
							'CAT_RULES_PLAIN'				=> $cat_rules_data['text'],
							'CAT_DOWNLOAD_COST'				=> $cat_data['cat_download_cost'],
							'CAT_COST'						=> $cat_data['cat_cost'],
							'CAT_REWARD'					=> $cat_data['cat_reward'],
							'CAT_USE_JACKPOT'				=> $cat_data['cat_use_jackpot'],
							'S_BBCODE_CHECKED'				=> ($cat_rules_data['allow_bbcode']) ? true : false,
							'S_SMILIES_CHECKED'				=> ($cat_rules_data['allow_smilies']) ? true : false,
							'S_URLS_CHECKED'				=> ($cat_rules_data['allow_urls']) ? true : false,
							'S_CAT_PASSWORD_SET'			=> (empty($cat_data['cat_password'])) ? false : true,

							'CAT_DESC'						=> $cat_desc_data['text'],
							'S_DESC_BBCODE_CHECKED'			=> ($cat_desc_data['allow_bbcode']) ? true : false,
							'S_DESC_SMILIES_CHECKED'		=> ($cat_desc_data['allow_smilies']) ? true : false,
							'S_DESC_URLS_CHECKED'			=> ($cat_desc_data['allow_urls']) ? true : false,

							'S_SHOW_POINTS'					=> ($arcade->points['installed'] && $auth->acl_get('a_arcade_points_settings')) ? true : false,
							'S_CAT_TYPE_OPTIONS'			=> $cat_type_options,
							'S_STATUS_OPTIONS'				=> $statuslist,
							'S_PARENT_OPTIONS'				=> $parents_list,
							'S_STYLES_OPTIONS'				=> $styles_list,
							'S_CAT_DISPLAY_OPTIONS'			=> $cat_display_list,
							'S_CAT_OPTIONS'					=> $arcade->make_cat_select(($action == 'add') ? $cat_data['parent_id'] : false, ($action == 'edit') ? $cat_data['cat_id'] : false, false, false, false),
							'S_SHOW_DISPLAY_ON_INDEX'		=> $s_show_display_on_index,
							'S_DISPLAY_SUBCAT_LIST'			=> ($cat_data['display_subcat_list']) ? true : false,
							'S_DISPLAY_ON_INDEX'			=> ($cat_data['display_on_index']) ? true : false,
							'S_ARCADE_CAT'					=> ($cat_data['cat_type'] == ARCADE_CAT) ? true : false,
							'S_ARCADE_CAT_TEST'				=> ($cat_data['cat_test']) ? true : false,
							'S_CAT_ORIG_CAT_GAMES'			=> (isset($old_cat_type) && $old_cat_type == ARCADE_CAT_GAMES) ? true : false,
							'S_CAT_ORIG_CAT'				=> (isset($old_cat_type) && $old_cat_type == ARCADE_CAT) ? true : false,
							'S_CAT_ORIG_LINK'				=> (isset($old_cat_type) && $old_cat_type == ARCADE_LINK) ? true : false,
							'S_ARCADE_LINK'					=> ($cat_data['cat_type'] == ARCADE_LINK) ? true : false,
							'S_ARCADE_CAT_GAMES'			=> ($cat_data['cat_type'] == ARCADE_CAT_GAMES) ? true : false,
							'S_ARCADE_LINK_TRACK'			=> ($cat_data['cat_flags'] & ARCADE_FLAG_LINK_TRACK) ? true : false,
							'S_CAN_COPY_PERMISSIONS'		=> ($action != 'edit' || empty($cat_id) || ($auth->acl_get('a_cauth') && $auth->acl_get('a_authusers') && $auth->acl_get('a_authgroups'))) ? true : false,
						));

						$return = true;
					break;

					case 'delete':
						if (!$cat_id)
						{
							trigger_error($user->lang['NO_CAT_ID'] . adm_back_link($this->u_action . '&amp;parent_id=' . $this->parent_id), E_USER_WARNING);
						}

						$cat_data = $arcade->get_cat_info($cat_id);

						$subcat_id = array();
						$subcat = $arcade->get_cat_branch($cat_id, 'children');

						foreach ($subcat as $row)
						{
							$subcat_id[] = $row['cat_id'];
						}

						$cat_list = $arcade->make_cat_select($cat_data['parent_id'], $subcat_id);

						$sql = 'SELECT cat_id
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_type = ' . ARCADE_CAT_GAMES . "
								AND cat_id <> $cat_id";
						$result = $db->sql_query($sql);

						if ($db->sql_fetchrow($result))
						{
							$template->assign_var('S_MOVE_CAT_OPTIONS', $arcade->make_cat_select($cat_data['parent_id'], $subcat_id, false, true));
						}
						$db->sql_freeresult($result);

						$parent_id = ($this->parent_id == $cat_id) ? 0 : $this->parent_id;

						$template->assign_vars(array(
							'S_DELETE_CAT'			=> true,
							'U_ACTION'				=> $this->u_action . "&amp;parent_id={$parent_id}&amp;action=delete&amp;c=$cat_id",
							'U_BACK'				=> $this->u_action . '&amp;parent_id=' . $this->parent_id,

							'CAT_NAME'				=> $cat_data['cat_name'],
							'S_ARCADE_CAT_GAMES'	=> ($cat_data['cat_type'] == ARCADE_CAT_GAMES) ? true : false,
							'S_ARCADE_LINK'			=> ($cat_data['cat_type'] == ARCADE_LINK) ? true : false,
							'S_HAS_SUBCATS'			=> ($cat_data['right_id'] - $cat_data['left_id'] > 1) ? true : false,
							'S_CATS_LIST'			=> $cat_list
						));

						$return = true;
					break;
				}

				if (!$return)
				{
					// Default management page
					$parent_cat_type = '';
					if (!$this->parent_id)
					{
						$navigation = $user->lang['ARCADE_INDEX'];
					}
					else
					{
						$navigation = '<a href="' . $this->u_action . '">' . $user->lang['ARCADE_INDEX'] . '</a>';
						$cat_nav = $arcade->get_cat_branch($this->parent_id, 'parents', 'descending');
						foreach ($cat_nav as $row)
						{
							if ($row['cat_id'] == $this->parent_id)
							{
								$parent_cat_type = $row['cat_type'];
								$navigation .= ' -&gt; ' . $row['cat_name'];
							}
							else
							{
								$navigation .= ' -&gt; <a href="' . $this->u_action . '&amp;parent_id=' . $row['cat_id'] . '">' . $row['cat_name'] . '</a>';
							}
						}
					}

					// Jumpbox
					$cat_box = $arcade->make_cat_select($this->parent_id, false, false, false, false);
					if (($action == 'sync' || $action == 'sync_marked') && !sizeof($errors))
					{
						$template->assign_var('S_RESYNCED', true);
					}

					$sql = 'SELECT *
							FROM ' . ARCADE_CATS_TABLE . "
							WHERE parent_id = $this->parent_id
							ORDER BY left_id";
					$result = $db->sql_query($sql);

					if ($row = $db->sql_fetchrow($result))
					{
						do
						{
							$cat_type = $row['cat_type'];

							if ($row['cat_status'] == ITEM_LOCKED)
							{
								$folder_image = '<img src="images/icon_folder_lock.gif" alt="' . $user->lang['LOCKED'] . '" />';
							}
							else
							{
								switch ($cat_type)
								{
									case ARCADE_LINK:
										$folder_image = '<img src="images/icon_folder_link.gif" alt="' . $user->lang['TYPE_LINK'] . '" />';
									break;

									default:
										$folder_image = ($row['left_id'] + 1 != $row['right_id']) ? '<img src="images/icon_subfolder.gif" alt="' . $user->lang['ARCADE_SUBCAT'] . '" />' : '<img src="images/icon_folder.gif" alt="' . $user->lang['FOLDER'] . '" />';
									break;
								}
							}

							$url = $this->u_action . "&amp;parent_id=$this->parent_id&amp;c={$row['cat_id']}" . (($start) ? "&amp;start=$start" : '');

							$cat_title = ($cat_type != ARCADE_LINK) ? '<a href="' . $this->u_action . '&amp;parent_id=' . $row['cat_id'] . '">' : '';
							$cat_title .= $row['cat_name'];
							$cat_title .= ($cat_type != ARCADE_LINK) ? '</a>' : '';

							$template->assign_block_vars('cats', array(
								'FOLDER_IMAGE'			=> $folder_image,
								'CAT_IMAGE'				=> ($row['cat_image']) ? '<img src="' . $phpbb_root_path . $arcade_config['cat_image_path'] . $row['cat_image'] . '" alt="" />' : '',
								'CAT_IMAGE_SRC'			=> ($row['cat_image']) ? $phpbb_root_path . $row['cat_image'] : '',
								'CAT_NAME'				=> $row['cat_name'],
								'CAT_DESCRIPTION'		=> generate_text_for_display($row['cat_desc'], $row['cat_desc_uid'], $row['cat_desc_bitfield'], $row['cat_desc_options']),
								'CAT_GAMES'				=> $row['cat_games'],
								'CAT_PLAYS'				=> $row['cat_plays'],

								'S_ARCADE_LINK'			=> ($cat_type == ARCADE_LINK) ? true : false,
								'S_ARCADE_CAT_GAMES'	=> ($cat_type == ARCADE_CAT_GAMES) ? true : false,

								'U_CAT'					=> $this->u_action . '&amp;parent_id=' . $row['cat_id'],
								'U_MOVE_UP'				=> $url . '&amp;action=move_up',
								'U_MOVE_DOWN'			=> $url . '&amp;action=move_down',
								'U_EDIT'				=> $url . '&amp;action=edit',
								'U_DELETE'				=> $url . '&amp;action=delete',
								'U_SYNC'				=> $url . '&amp;action=sync'
							));
						}
						while ($row = $db->sql_fetchrow($result));
					}
					else if ($this->parent_id)
					{
						$row = $arcade->get_cat_info($this->parent_id);

						$url = $this->u_action . '&amp;parent_id=' . $this->parent_id . '&amp;c=' . $row['cat_id'] . (($start) ? "&amp;start=$start" : '');

						$template->assign_vars(array(
							'S_NO_CATS'	=> true,

							'U_EDIT'	=> $url . '&amp;action=edit',
							'U_DELETE'	=> $url . '&amp;action=delete',
							'U_SYNC'	=> $url . '&amp;action=sync'
						));
					}
					$db->sql_freeresult($result);

					$total_games = $arcade->get_total('games', $this->parent_id);

					if ($start >= $total_games)
					{
						$start = $start - $arcade_config['acp_items_per_page'];
						$start = ($start < 0) ? 0 : $start;
					}

					if ($total_games && $this->parent_id)
					{
						$sql = 'SELECT *
								FROM ' . ARCADE_GAMES_TABLE . "
								WHERE cat_id = $this->parent_id
								ORDER BY game_order DESC";
						$result = $db->sql_query_limit($sql, $arcade_config['acp_items_per_page'], $start);

						if ($row = $db->sql_fetchrow($result))
						{
							do
							{
								$url = $this->u_action . '&amp;parent_id=' . $this->parent_id . '&amp;g=' . $row['game_id'] . (($start) ? "&amp;start=$start" : '');

								$template->assign_block_vars('games', array(
									'GAME_ID'		=> $row['game_id'],
									'GAME_NAME'		=> get_arcade_game_name($row['game_name'], false, 'play', $row['cat_id'], $row['game_id'], 'x', false, false, false, true),
									'GAME_DESC'		=> nl2br($row['game_desc']),
									'GAME_PLAYS'	=> $row['game_plays'],
									'GAME_IMAGE'	=> ($arcade->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 30, 30, 'x', 'play', $row['cat_id'], $row['game_id'], false, true) : false,

									'U_MOVE_UP'		=> $url . '&amp;action=g_move_up',
									'U_MOVE_DOWN'	=> $url . '&amp;action=g_move_down',
									'U_EDIT'		=> append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_manage&amp;mode=games&amp;action=edit&amp;g={$row['game_id']}" . (($start) ? "&amp;start=$start" : '')),
									'U_DELETE'		=> $url . '&amp;action=g_delete',
									'U_SYNC'		=> $url . '&amp;action=sync'
								));
							}
							while ($row = $db->sql_fetchrow($result));
						}
						$db->sql_freeresult($result);

						$mark_options = array('sync_marked', 'reset_scores_marked', 'reset_marked', 'move_marked', 'delete_marked', 'reset_install_date_marked');

						$s_mark_options = '';
						foreach ($mark_options as $mark_option)
						{
							if (($mark_option == 'delete_marked' && !$auth->acl_get('a_arcade_delete_game')) ||
								(($mark_option == 'reset_marked' || $mark_option == 'reset_install_date_marked') && (!$auth->acl_get('a_arcade_user') || !$auth->acl_get('a_arcade_game'))) ||
								($mark_option == 'reset_scores_marked' && !$auth->acl_get('a_arcade_user')))
							{
								continue;
							}

							$s_mark_options .= '<option' . (($action == $mark_option) ? ' selected="selected"' : '') . ' value="' . $mark_option . '">' . $user->lang['ARCADE_' . strtoupper($mark_option)] . '</option>';
						}

						unset($mark_options);

						$template->assign_vars(array(
							'S_MARK_OPTIONS'	=> $s_mark_options,
							'S_ON_PAGE'			=> on_page($total_games, $arcade_config['acp_items_per_page'], $start),
							'PAGINATION'		=> generate_pagination($this->u_action . '&amp;parent_id=' . $this->parent_id, $total_games, $arcade_config['acp_items_per_page'], $start, true)
						));

						$sql = 'SELECT *
								FROM ' . ARCADE_GAMES_TABLE . "
								WHERE cat_id = $this->parent_id
								ORDER BY game_name_clean ASC";
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrowset($result);
						$db->sql_freeresult($result);

						// Quick jump of all the games in this category.
						foreach ($row as $game)
						{
							$template->assign_block_vars('quick_jump', array(
								'GAME_ID'	=> $game['game_id'],
								'GAME_NAME'	=> $game['game_name']
							));
						}
						// Quick jump of all the games in this category.
					}

					$template->assign_vars(array(
						'S_CAT_DELETE'			=> ($auth->acl_get('a_arcade_delete_cat')) ? true : false,
						'S_GAME_EDIT'			=> ($auth->acl_get('a_arcade_game')) ? true : false,
						'S_ARCADE_CAT_GAMES'	=> ($parent_cat_type == ARCADE_CAT_GAMES) ? true : false,
						'S_PROGRESS_BAR'		=> true,

						'ERROR_MSG'				=> (sizeof($errors)) ? implode('<br />', $errors) : '',
						'NAVIGATION'			=> $navigation,
						'CAT_BOX'				=> $cat_box,

						'U_SEL_ACTION'			=> $this->u_action,
						'U_ACTION'				=> $this->u_action . '&amp;parent_id=' . $this->parent_id . (($start) ? "&amp;start=$start" : ''),
						'U_ACTION_DELETE'		=> $this->u_action . '&amp;parent_id=' . $this->parent_id . '&amp;action=g_delete' . (($start) ? "&amp;start=$start" : ''),
						'U_PROGRESS_BAR'		=> $this->u_action . '&amp;action=progress_bar',
						'UA_PROGRESS_BAR'		=> str_replace('&amp;', '&', $this->u_action) . '&action=progress_bar'
					));
				}
			break;

			case 'games':
				$form_key = 'acp_arcade_games';
				add_form_key($form_key);
				$page_title = 'ACP_ARCADE_MANAGE_GAMES';

				$start					= request_var('start', 0);
				$game_id				= request_var('g', 0);
				$cat_id					= request_var('c', 0);
				$game_type				= request_var('gt', 0);
				$gametypev3				= request_var('gametypev3', 0);
				$gametypev32			= request_var('gametypev32', 0);
				$delete_game			= (isset($_POST['delete'])) ? true : false;
				$reset_game_ins_date	= (isset($_POST['reset_game_install_date']) && $auth->acl_get('a_arcade_user') && $auth->acl_get('a_arcade_game')) ? true : false;
				$edit_game				= (isset($_POST['edit'])) ? true : false;
				$cat_games_delete		= (!empty($_POST['cat_games_delete'])) ? true : false;
				$type_games_delete		= (!empty($_POST['type_games_delete'])) ? true : false;

				$template->assign_var('S_MANAGE_GAMES', true);

				if ($action == '')
				{
					if ($edit_game)
					{
						$action = 'edit';
					}
					else if ($delete_game || $cat_games_delete || $type_games_delete)
					{
						$action = 'delete';
					}
				}
				else if ($delete_game)
				{
					$action = 'delete';
				}

				switch ($action)
				{
					case 'edit':
						$old_cat_id = request_var('old_cat_id', 0);
						$old_game_scoretype = request_var('old_game_scoretype', 0);

						$display_vars = array(
							'title'					=> 'ARCADE_EDIT_GAME',
							'title_explain'			=> 'ARCADE_EDIT_GAME_EXPLAIN',
							'vars'	=> array(
								'legend1'			=> 'ARCADE_GAME_SETTINGS',
								'game_name'			=> array('lang' => 'ARCADE_GAME_NAME',			'validate' => 'string', 	'type' => 'text:30:255', 	'explain' => true),
								'game_desc'			=> array('lang' => 'ARCADE_GAME_DESC',										'type' => 'textarea:5:25', 	'explain' => true),
								'game_control'		=> array('lang' => 'ARCADE_GAME_CONTROL',		'validate' => 'int', 		'type' => 'custom', 		'explain' => true, 	'method' => 'game_control'),
								'game_control_desc'	=> false,
								'game_image'		=> array('lang' => 'ARCADE_GAME_IMAGE',			'validate' => 'string', 	'type' => 'text:30:255',	'explain' => true),
								'game_swf'			=> array('lang' => 'ARCADE_GAME_SWF',			'validate' => 'string', 	'type' => 'custom', 		'explain' => true, 	'method' => 'game_swf'),
								'game_width'		=> array('lang' => 'ARCADE_GAME_WIDTH',			'validate' => 'int:50', 	'type' => 'text:3:4', 		'explain' => true),
								'game_height'		=> array('lang' => 'ARCADE_GAME_HEIGHT',		'validate' => 'int:50', 	'type' => 'text:3:4', 		'explain' => true),
								'game_installdate'	=> array('lang' => 'ARCADE_GAME_INSTALLDATE',	'validate' => 'string',		'type' => 'custom', 		'explain' => true, 	'method' => 'game_installdate'),
								'game_filesize'		=> array('lang' => 'ARCADE_GAME_FILESIZE',		'validate' => 'string',		'type' => 'custom', 		'explain' => true, 	'method' => 'game_filesize'),
								'game_download'		=> array('lang' => 'ARCADE_GAME_DOWNLOAD', 		'validate' => 'bool', 		'type' => 'radio:yes_no', 	'explain' => true),
								'cat_id'			=> array('lang' => 'ARCADE_GAME_CATEGORY',		'validate' => 'int', 		'type' => 'select', 		'explain' => true, 	'method' => 'game_cat_select'),

								'legend2'			=> 'ARCADE_SCORE_SETTINGS',
								'game_scorevar'		=> array('lang' => 'ARCADE_GAME_SCOREVAR', 		'validate' => 'string', 	'type' => 'text:30:255', 	'explain' => true),
								'game_type'			=> array('lang' => 'ARCADE_GAME_TYPE',			'validate' => 'int', 		'type' => 'select', 		'explain' => true,	'method' => 'game_type_select'),
								'game_scoretype'	=> array('lang' => 'ARCADE_GAME_SCORETYPE',		'validate' => 'int', 		'type' => 'select', 		'explain' => true,	'method' => 'game_scoretype_select'),
							)
						);

						if ($arcade->points['installed'] && $auth->acl_get('a_arcade_points_settings'))
						{
							$display_vars['vars'] += array(
								'legend3'			=> 'ARCADE_GAME_POINTS_SETTINGS',
								'game_download_cost'=> array('lang' => 'ARCADE_GAME_DOWNLOAD_COST',	'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
								'game_cost'			=> array('lang' => 'ARCADE_GAME_COST', 			'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
								'game_reward'		=> array('lang' => 'ARCADE_GAME_REWARD',		'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
								'game_use_jackpot'	=> array('lang' => 'ARCADE_GAME_USE_JACKPOT',	'validate' => 'bool', 		'type' => 'radio:yes_no', 		'explain' => true),
								'game_jackpot'		=> array('lang' => 'ARCADE_GAME_JACKPOT',		'validate' => 'string', 	'type' => 'text:10:11', 		'explain' => true),
							);
						}

						$this->new_game_data = $arcade->get_game_cat_data($game_id);
						if (!$this->new_game_data)
						{
							trigger_error($user->lang['NO_GAME_ID'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_game_data;

						// We validate the complete config if whished
						validate_config_vars($display_vars['vars'], $cfg_array, $errors);

						if ($update && !check_form_key($form_key))
						{
							$errors[] = $user->lang['FORM_INVALID'];
						}

						$cat_data = $arcade->get_cat_info($cfg_array['cat_id']);
						if ($cat_data['cat_type'] != ARCADE_CAT_GAMES)
						{
							$errors[] = $user->lang['WRONG_CAT_TYPE'];
						}

						if ($update && $reset_game_ins_date && !sizeof($errors))
						{
							$errors = $arcade->reset_game('reset_game_install_date', $game_id, $old_cat_id);
						}

						// Do not write values if there is an error
						if (sizeof($errors))
						{
							$update = false;
						}

						$update_install_file = false;

						// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
						foreach ($display_vars['vars'] as $config_name => $null)
						{
							if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
							{
								continue;
							}

							$config_value = $cfg_array[$config_name];
							$this->new_game_data[$config_name] = $config_value;

							if ($update)
							{
								if (in_array($config_name, array('game_download_cost', 'game_cost', 'game_reward', 'game_jackpot')))
								{
									$negative = ($config_name == 'game_jackpot') ? true : false;
									$arcade->float_num($config_value, $negative);
								}

								// We have to check if the cat_id has changed if it does we need to send the
								// new and old ids
								if ($config_name == 'cat_id')
								{
									$arcade->set_game_data($game_id, $config_name, $config_value, $this->new_game_data['cat_id'], $old_cat_id);
								}
								else if ($config_name == 'game_scoretype')
								{
									$arcade->set_game_data($game_id, $config_name, $config_value, false, false, $this->new_game_data['game_scoretype'], $old_game_scoretype);
								}
								else
								{
									if ($config_name == 'game_name')
									{
										if (Arcade_update_game_install_file($game_id))
										{
											$update_install_file = true;
										}
									}

									$arcade->set_game_data($game_id, $config_name, $config_value);
								}
							}
						}

						if ($update)
						{
							$log_key = 'LOG_ARCADE_EDIT_GAME' . (($reset_game_ins_date) ? '_RESET_INSTALL_DATE' : '');

							$arcade->add_log('admin', $log_key, '<a onclick="window.open(this.href); return false;" href="' . $arcade->url('mode=play&amp;g=' . $this->new_game_data['game_id']) .'">'. $this->new_game_data['game_name'] . '</a>');

							$manage_games_link	= '<br /><br /><a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_manage&amp;mode=games") . '">&laquo; ' . $user->lang['ACP_BACK_MANAGE_GAMES'] . '</a>';
							$manage_cat_link	= $auth->acl_get('a_arcade_cat') ? '<br /><br /><a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_manage&amp;mode=category&amp;parent_id={$old_cat_id}" . (($start) ? "&amp;start=$start" : '')) . '">&laquo; ' . $user->lang['ACP_BACK_MANAGE_CATEGORY'] . '</a>' : '';
							$game_return_links	= '<br /><br />' . sprintf($user->lang['ARCADE_RETURN_LINKS'], '<a href="' . $arcade->url("mode=play&amp;g={$game_id}") . '">&laquo; ', $this->new_game_data['game_name'], '</a>', '<a href="' . $arcade->url('mode=cat&amp;c=' . $this->new_game_data['cat_id']) . '&amp;g=' . $game_id .'#g' . $game_id .'">&laquo; ', $this->new_game_data['cat_name'], '</a>', '<a href="' . $arcade->url() . '">&laquo; ', '</a>');

							$msg_key = ($reset_game_ins_date) ? sprintf($user->lang['RESET_INSTALL_DATE_GAME_SUCCESS'], $this->new_game_data['game_name']) . '<br /><br />' : '';

							$tables = array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE);

							if ($reset_game_ins_date)
							{
								$tables[] = ARCADE_SCORES_TABLE;
								$tables[] = ARCADE_PLAYS_TABLE;

								$cache->destroy('_arcade_leaders');
								$cache->destroy('_arcade_leaders_all');
							}

							$cache->destroy('sql', $tables);
							$cache->destroy('_arcade_cats');

							if ($update_install_file)
							{
								trigger_error($msg_key . $user->lang['ARCADE_GAME_UPDATED'] . $manage_games_link . $manage_cat_link . $game_return_links);
							}
							else
							{
								trigger_error($msg_key . $user->lang['ARCADE_GAME_UPDATED_ERROR'] . $manage_games_link . $manage_cat_link . $game_return_links, E_USER_WARNING);
							}
						}

						// Output relevant page
						foreach ($display_vars['vars'] as $config_key => $vars)
						{
							if (!is_array($vars) && strpos($config_key, 'legend') === false)
							{
								continue;
							}

							if (strpos($config_key, 'legend') !== false)
							{
								$template->assign_block_vars('options', array(
									'S_LEGEND'	=> true,
									'LEGEND'	=> $user->lang[$vars])
								);

								continue;
							}

							$type = explode(':', $vars['type']);

							if (in_array($config_key, array('game_download_cost', 'game_cost', 'game_reward', 'game_jackpot')))
							{
								$this->new_game_data[$config_key] = $arcade->number_format($this->new_game_data[$config_key], true);
							}

							$template->assign_block_vars('options', array(
								'ACTIVE'		=> ($config_key == 'game_desc') ? true : false,
								'KEY'			=> (in_array($config_key, array('game_swf', 'game_installdate', 'game_filesize'))) ? false : $config_key,
								'TITLE'			=> $user->lang[$vars['lang']],
								'S_EXPLAIN'		=> $vars['explain'],
								'TITLE_EXPLAIN'	=> ($vars['explain']) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '',
								'CONTENT'		=> build_cfg_template($type, $config_key, $this->new_game_data, $config_key, $vars),
							));

							unset($display_vars['vars'][$config_key]);
						}

						if (isset($display_vars['lang']))
						{
							$user->add_lang($display_vars['lang']);
						}

						$page_title	= $display_vars['title'];
						$l_title	= $user->lang[$display_vars['title']];
						$l_title_explain = $user->lang[$display_vars['title_explain']];

						$s_hidden_fields = array(
							'old_cat_id'			=> $this->new_game_data['cat_id'],
							'old_game_scoretype'	=> $this->new_game_data['game_scoretype']
						);

						$template->assign_vars(array(
							'S_EDIT_GAME'				=> true,
							'S_RESET_GAME_INSTALL_DATE'	=> ($auth->acl_get('a_arcade_user') && $auth->acl_get('a_arcade_game')) ? true : false,
							'S_HIDDEN_FIELDS'			=> build_hidden_fields($s_hidden_fields),

							'U_ACTION'					=> $this->u_action . '&amp;action=edit&amp;g=' . $game_id . (($start) ? '&amp;start=' . $start : '')
						));

						$return = true;
					break;

					case 'delete':
						if (confirm_box(true))
						{
							$delete_files = (request_var('delete_files', false)) ? true : false;

							if ($cat_games_delete || $type_games_delete)
							{
								if ($type_games_delete && $game_type == IBPROV3_GAME && !$gametypev3 && !$gametypev32)
								{
									$errors[] = $user->lang['ARCADE_SELECT_EMPTY'];
								}
								else
								{
									$type	= ($cat_games_delete) ? 'cat' : 'gametype';
									$id		= ($cat_id) ? $cat_id : $game_type;

									$row = $arcade->get_cat_game_ids($type, $id, $gametypev3, $gametypev32);

									if (!$row)
									{
										$errors[] = $user->lang['NOT_FOUND_GAME'];
									}
									else
									{
										$errors = $arcade->delete_game($row['game_ids'], $row['cat_ids'], $delete_files, $row['games_swf']);
										$game_names = implode(', ', $row['game_names']);
									}
								}
							}
							else
							{
								$game_data	= $arcade->get_game_data($game_id);
								$errors		= $arcade->delete_game($game_id, $game_data['cat_id'], $delete_files, $game_data['game_swf']);
								$game_names	= $game_data['game_name'];
							}

							if (sizeof($errors))
							{
								break;
							}

							$s = ($cat_games_delete || $type_games_delete) ? 'S' : '';

							$arcade->add_log('admin', 'LOG_ARCADE_DELETE_GAME' . $s, $game_names);

							$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE, ARCADE_CHALLENGE_TABLE, ARCADE_CHALLENGE_CHAMP_TABLE));
							$cache->destroy('_arcade_cats');
							$cache->destroy('_arcade_leaders');
							$cache->destroy('_arcade_leaders_all');
							$cache->destroy('_arcade_games_filesize');
							$cache->destroy('_arcade_challenge_users');
							$cache->destroy('_arcade_challenge_leaders');
							$cache->destroy('_arcade_challenge_leaders_all');

							trigger_error(sprintf($user->lang['GAME' . $s . '_DELETED'], $game_names) . adm_back_link($this->u_action));
						}
						else
						{
							if (!$auth->acl_get('a_arcade_delete_game'))
							{
								trigger_error($user->lang['NO_PERMISSION_GAME_DELETE'] . adm_back_link($this->u_action), E_USER_WARNING);
							}

							if ($cat_games_delete)
							{
								$confirm_lang = 'DELETE_SELECTED_CAT_ALL_GAMES';
								$s_hidden_fields = array(
									'c'					=> $cat_id,
									'cat_games_delete'	=> true
								);
							}
							else if ($type_games_delete)
							{
								$confirm_lang = 'DELETE_SELECTED_GAMETYPE_ALL_GAMES';
								$s_hidden_fields = array(
									'gt'				=> $game_type,
									'type_games_delete'	=> true,
									'gametypev3'		=> $gametypev3,
									'gametypev32'		=> $gametypev32
								);
							}
							else
							{
								$confirm_lang = 'DELETE_SELECTED_GAME';
								$s_hidden_fields = array('g' => $game_id);
							}

							$s_hidden_fields = array_merge($s_hidden_fields, array('action' => 'delete'));

							$arcade->confirm_box('delete_files', $confirm_lang, $s_hidden_fields);
						}
					break;
				}

				if (!$return)
				{
					// Default Arcade Games Page
					// Quick jump of all the games in the arcade...
					$sql = 'SELECT game_id, game_name, game_name_clean
						FROM ' . ARCADE_GAMES_TABLE . "
						ORDER BY game_name_clean ASC";
					$result = $db->sql_query($sql);

					while ($row = $db->sql_fetchrow($result))
					{
						$template->assign_block_vars('quick_jump', array(
							'GAME_ID'	=> $row['game_id'],
							'GAME_NAME'	=> $row['game_name'],
						));
					}
					$db->sql_freeresult($result);
					// Quick jump of all the games in the arcade...

					$l_title = $user->lang['ACP_ARCADE_MANAGE_GAMES'];
					$l_title_explain = $user->lang['ACP_ARCADE_MANAGE_GAMES_EXPLAIN'];

					$template->assign_vars(array(
						'S_DEFAULT_EDIT_GAME'	=> true,
						'S_CATEGORY_OPTIONS'	=> $arcade->make_cat_select(false, false, true, false, false),
						'S_GAMETYPE_OPTIONS'	=> $arcade->game_type_select(),
						'S_IBPROV3_GAME'		=> IBPROV3_GAME,

						'U_ACTION'				=> $this->u_action
					));
				}
			break;

			case 'users':
				$form_key = 'acp_arcade_users';
				add_form_key($form_key);
				$template->assign_var('S_MANAGE_USERS', true);
				$page_title = 'ACP_ARCADE_MANAGE_USERS';

				$username	= utf8_normalize_nfc(request_var('username', '', true));
				$user_id	= request_var('u', 0);
				$start		= request_var('start', 0);
				$score		= request_var('score', 0.000);
				$score_date	= request_var('score_date', 0);
				$date_now	= request_var('date_now', false);
				$game_id	= request_var('g', 0);
				$comment	= request_var('comment', '', true);

				$action = (isset($_POST['submituser'])) ? 'prefs' : $action;
				$action = (isset($_POST['reset_user_scores'])) ? 'reset_user_scores' : $action;
				$action = (isset($_POST['reset_user_all'])) ? 'reset_user_all' : $action;
				$action = (isset($_POST['user_banned'])) ? 'user_banned' : $action;

				switch ($action)
				{
					case 'prefs':
						$user->add_lang(array('ucp', 'acp/users', 'mods/info_ucp_arcade'));
						$user_row = $arcade->userdata('options', $user_id, $username);

						if (!$user_row)
						{
							trigger_error($user->lang['NO_USER'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$page_title = 'USER_ADMIN';
						$l_title_explain = $user->lang['USER_ADMIN_EXPLAIN'];

						$cdata = array(
							'view_avatars'		=> (bool) request_var('avatars', $arcade->optionget('view_avatars', $user_row['user_arcade_options'], true)),
							'view_game_image'	=> (bool) request_var('game_image', $arcade->optionget('view_game_image', $user_row['user_arcade_options'], true)),
							'view_popup_icon'	=> (bool) request_var('popup_icon', $arcade->optionget('view_popup_icon', $user_row['user_arcade_options'], true))
						);

						if ($user_row['user_id'] != ANONYMOUS)
						{
							$cdata = array_merge($cdata, array(
								'challenge'			=> (bool) request_var('challenge', $arcade->optionget('challenge', $user_row['user_arcade_options'], true)),
								'arcade_pm'			=> (bool) request_var('arcade_pm', $arcade->optionget('arcade_pm', $user_row['user_arcade_options'], true)),
								'notifypm'			=> (bool) request_var('notifypm', $arcade->optionget('notifypm', $user_row['user_arcade_options'], true)),

								'bbcode'			=> (bool) request_var('bbcode', $arcade->optionget('bbcode', $user_row['user_arcade_options'], true)),
								'smilies'			=> (bool) request_var('smilies', $arcade->optionget('smilies', $user_row['user_arcade_options'], true))
							));
						}

						$data = array(
							'games_per_page'	=> request_var('games_per_page', $user_row['games_per_page']),
							'games_sort_order'	=> request_var('games_sort_order', $user_row['games_sort_order']),
							'games_sort_dir'	=> request_var('games_sort_dir', $user_row['games_sort_dir']),
						);

						if ($update)
						{
							if (!function_exists('validate_data'))
							{
								include($phpbb_root_path . 'includes/functions_user.' . $phpEx);
							}

							$validate_array = array(
								'games_sort_order'	=> array('string', false, 1, 1),
								'games_sort_dir'	=> array('string', false, 1, 1),
								'games_per_page'	=> array('num', false, 0, ARCADE_UCP_GAMES_PER_PAGE_MAX),
							);

							$errors = validate_data($data, $validate_array);

							if (!check_form_key($form_key))
							{
								$errors[] = 'FORM_INVALID';
							}

							if (!sizeof($errors))
							{
								$arcade->rev_optionset($user_row, 'view_avatars', $cdata['view_avatars']);
								$arcade->rev_optionset($user_row, 'view_game_image', $cdata['view_game_image']);
								$arcade->rev_optionset($user_row, 'view_popup_icon', $cdata['view_popup_icon']);

								if ($user_row['user_id'] != ANONYMOUS)
								{
									$arcade->rev_optionset($user_row, 'arcade_pm', $cdata['arcade_pm']);
									$arcade->rev_optionset($user_row, 'notifypm', $cdata['notifypm']);
									$arcade->rev_optionset($user_row, 'challenge', $cdata['challenge']);

									$arcade->rev_optionset($user_row, 'bbcode', $cdata['bbcode']);
									$arcade->rev_optionset($user_row, 'smilies', $cdata['smilies']);
								}

								$data['user_arcade_options'] = $user_row['user_arcade_options'];

								$sql = 'UPDATE ' . ARCADE_USERS_TABLE . '
										SET ' . $db->sql_build_array('UPDATE', $data) . '
										WHERE user_id = ' . (int) $user_row['user_id'];
								$db->sql_query($sql);

								$cache->destroy('sql', ARCADE_CHALLENGE_CHAMP_TABLE);

								$arcade->add_log('admin', 'LOG_ARCADE_USER_SET_UPDATE', $user_row['username']);

								trigger_error($user->lang['USER_OVERVIEW_UPDATED'] . adm_back_link($this->u_action . '&amp;action=prefs&amp;u=' . $user_row['user_id']));
							}

							$errors = preg_replace('#^([A-Z_]+)$#e', "(!empty(\$user->lang['\\1'])) ? \$user->lang['\\1'] : '\\1'", $errors);
						}

						$s_hidden_fields = array(
							'u' => $user_row['user_id']
						);

						$template->assign_vars(array(
							'S_USER_PREFS'			=> true,
							'S_ANONYMOUS'			=> ($user_row['user_id'] == ANONYMOUS) ? true : false,
							'S_HIDDEN_FIELDS'		=> build_hidden_fields($s_hidden_fields),
							'S_ARCADE_BANNED'		=> ($user_row['user_type'] != USER_FOUNDER && !in_array($user_row['user_id'], array($user->data['user_id'], ANONYMOUS))) ? true : false,

							'S_AVATARS_ENABLE'		=> $cdata['view_avatars'],
							'S_GAME_IMAGE_ENABLED'	=> $cdata['view_game_image'],
							'S_POPUP_ICON_ENABLED'	=> $cdata['view_popup_icon'],

							'U_ACTION'				=> $this->u_action . '&amp;action=prefs',

							'MANAGE_USERNAME'		=> $user_row['username'],
							'GAMES_PER_PAGE'		=> $data['games_per_page'],
							'GAMES_SORT_ORDER'		=> arcade_games_sort_order_select($data['games_sort_order']),
							'GAMES_SORT_DIR'		=> arcade_games_sort_dir_select($data['games_sort_dir'])
						));

						if ($user_row['user_id'] != ANONYMOUS)
						{
							$template->assign_vars(array(
								'S_ARCADE_PM_ENABLED'	=> $cdata['arcade_pm'],
								'S_ARCADE_NOTIFY_PM'	=> $cdata['notifypm'],
								'S_CHALLENGE_ENABLED'	=> $cdata['challenge'],

								'S_BBCODE_ENABLED'		=> $cdata['bbcode'],
								'S_SMILIES_ENABLED'		=> $cdata['smilies'],

								'ARCADE_HIGHSCORES'		=> $arcade->number_format(display_arcade_highscores('', $user_row['user_id'], 'arcade')),
								'CHALLENGE_WINS'		=> $arcade->number_format(display_arcade_highscores('', $user_row['user_id'], 'challenge'))
							));
						}

						$return = true;
					break;

					case 'reset_user_scores':
						if ((!$total_champ_data || !request_var('process', false)) && !confirm_box(true))
						{
							if (!$total_champ_data)
							{
								$total_champ_data = $arcade->get_total('update_super_champion', $user_id);
							}

							$s_hidden_fields = array(
								'u'					=> $user_id,
								'reset_user_scores'	=> true,
								'total_champ_data'	=> $total_champ_data
							);

							confirm_box(false, 'RESET_USER_SCORES', build_hidden_fields($s_hidden_fields));
						}
						else
						{
							$super_champion_process = false;
							if ($total_champ_data && $arcade->proces_champion_users($action, $total_champ_data, $this->u_action, $user_id))
							{
								$super_champion_process = true;
							}

							if (!$super_champion_process)
							{
								$arcade->reset('user_scores', $user_id);
								$username = $arcade->userdata('username', $user_id);

								$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
								$cache->destroy('_arcade_leaders');
								$cache->destroy('_arcade_leaders_all');

								$arcade->sync('total_data', 'plays');

								$arcade->add_log('admin', 'LOG_ARCADE_RESET_USER_SCORES', $username);
								trigger_error(sprintf($user->lang['ARCADE_RESET_USER_SCORES_DONE'], $username) . adm_back_link($this->u_action));
							}

							$return = true;
						}
					break;

					case 'reset_user_all':
						if (confirm_box(true))
						{
							$arcade->reset('user', $user_id);
							$username = $arcade->userdata('username', $user_id);

							$arcade->add_log('admin', 'LOG_ARCADE_RESET_USER_ALL', $username);

							$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
							$cache->destroy('_arcade_leaders');
							$cache->destroy('_arcade_leaders_all');

							$arcade->sync('total_data', 'plays');

							trigger_error(sprintf($user->lang['ARCADE_RESET_USER_ALL_DONE'], $username) . adm_back_link($this->u_action));
						}
						else
						{
							$s_hidden_fields = array(
								'u'					=> $user_id,
								'reset_user_all'	=> true,
							);
							confirm_box(false, 'RESET_USER_ALL', build_hidden_fields($s_hidden_fields));
						}
					break;

					case 'user_banned':
						if (!$user_id || $user_id == ANONYMOUS)
						{
							break;
						}

						if (confirm_box(true))
						{
							$user_data = $arcade->userdata('set_auth', $user_id);
							$arcade->banned(ARCADE_BANNED_ADMIN, $user_id, $user_data['user_ip']);

							$arcade->add_log('admin', 'LOG_ARCADE_USER_BANNED', $user_data['username']);
							trigger_error(sprintf($user->lang['ARCADE_USER_BANNED_DONE'], $user_data['username']) . adm_back_link($this->u_action));
						}
						else
						{
							$user->add_lang('acp/users');

							if ($user->data['user_id'] == $user_id)
							{
								trigger_error($user->lang['CANNOT_BAN_YOURSELF'] . adm_back_link($this->u_action), E_USER_WARNING);
							}

							if ($arcade->userdata('user_type', $user_id) == USER_FOUNDER)
							{
								trigger_error($user->lang['CANNOT_BAN_FOUNDER'] . adm_back_link($this->u_action), E_USER_WARNING);
							}

							$s_hidden_fields = array(
								'u'				=> $user_id,
								'user_banned'	=> true,
							);
							confirm_box(false, 'ARCADE_USER_BANNED', build_hidden_fields($s_hidden_fields));
						}
					break;

					case 'delete':
						if (!$game_id || !$user_id)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						if (confirm_box(true))
						{
							$game_data = $arcade->get_game_data($game_id);
							$arcade->delete_score($game_data, $user_id);
							$username = $arcade->userdata('username', $user_id);

							$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
							$cache->destroy('_arcade_leaders');
							$cache->destroy('_arcade_leaders_all');

							$arcade->add_log('admin', 'LOG_ARCADE_DELETE_SCORE', $username, $game_data['game_name']);
							trigger_error(sprintf($user->lang['ARCADE_SCORE_DELETED'], $username, $game_data['game_name']) . adm_back_link($this->u_action));
						}
						else
						{
							$arcade->confirm_box('retain_plays', 'DELETE_SELECTED_SCORE');
						}
					break;

					case 'progress_bar':
						$arcade->display_progress_bar();
					break;

					case 'sync':
						$game_data = $arcade->get_game_data($game_id);
						$arcade->sync('game', $game_id);
						$arcade->update_highscore($game_data);

						$arcade->add_log('admin', 'LOG_ARCADE_SYNC_GAME', $game_data['game_name']);

						$template->assign_vars(array(
							'S_RESYNCED'	=> true,
							'GAME_RESYNCED'	=> sprintf($user->lang['GAME_RESYNCED'], $game_data['game_name'])
						));
					break;

					case 'edit':
						if (!$game_id || !$user_id)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						if ($update)
						{
							if (!check_form_key($form_key))
							{
								trigger_error('FORM_INVALID', E_USER_WARNING);
							}

							$score_data = array(
								'score'				=> $score,
								'score_date'		=> ($date_now) ? time(): $score_date,
								'comment_text'		=> $comment,
								'comment_bitfield'	=> '',
								'comment_options'	=> 7,
								'comment_uid'		=> ''
							);

							$game_data = $arcade->get_game_data($game_id);
							$arcade->update_score($score_data, $user_id, $game_id);
							$username = $arcade->userdata('username', $user_id);

							$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE));
							$cache->destroy('_arcade_leaders');
							$cache->destroy('_arcade_leaders_all');

							$arcade->add_log('admin', 'LOG_ARCADE_EDIT_SCORE', $username, '<a onclick="window.open(this.href); return false;" href="'. $arcade->url('mode=play&amp;g=' . $game_data['game_id']) .'">'. $game_data['game_name'] .'</a>');
							trigger_error(sprintf($user->lang['ARCADE_SCORE_UPDATED'], $username, $game_data['game_name']) . adm_back_link($this->u_action));
						}

						$sql_array = array(
							'SELECT'	=> 'g.*, s.*, p.total_time, p.total_plays, u.username, u.user_colour',
							'FROM'		=> array(
								ARCADE_SCORES_TABLE	=> 's',
							),
							'LEFT_JOIN'	=> array(
								array(
									'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
									'ON'	=> 's.game_id = g.game_id'
								),
								array(
									'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
									'ON'	=> 's.game_id = p.game_id AND s.user_id = p.user_id'
								),
								array(
									'FROM'	=> array(USERS_TABLE => 'u'),
									'ON'	=> 's.user_id = u.user_id'
								),
							),
							'WHERE'		=> 's.user_id = ' . (int) $user_id . ' AND g.game_id = ' . (int) $game_id,
						);

						$sql = $db->sql_build_query('SELECT', $sql_array);
						$result = $db->sql_query($sql);
						$row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$row)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$l_title_explain	= $user->lang['ACP_ARCADE_EDIT_SCORES_EDIT_EXPLAIN'];
						$comment_data		= generate_text_for_edit($row['comment_text'], $row['comment_uid'], $row['comment_options']);

						$template->assign_vars(array(
							'S_EDIT_SCORES'		=> true,

							'U_EDIT_ACTION'		=> $this->u_action . "&amp;g=$game_id&amp;u=$user_id&amp;action=$action",

							'SCORE_USERNAME'	=> $arcade->adm_username($row['user_id'], $row['username'], $row['user_colour'], $this->u_action),
							'GAME_NAME'			=> $row['game_name'],
							'SCORE'				=> $row['score'],
							'COMMENT'			=> $comment_data['text'],
							'DATE_RAW'			=> $row['score_date'],
							'DATE'				=> $user->format_date($row['score_date']),
							'TOTAL_PLAYS'		=> $row['total_plays'],
							'TOTAL_TIME'		=> $arcade->time_format($row['total_time']),
							'AVG_TIME'			=> $arcade->time_format($row['total_time']/$row['total_plays'])
						));

						$return = true;
					break;

					case 'show_scores':
						$game_data = $arcade->get_game_data($game_id);
						$score_order = ($game_data['game_scoretype'] == SCORETYPE_HIGH) ? 'DESC' : 'ASC';

						$sql_array = array(
							'SELECT'	=> 's.*, p.total_time, p.total_plays, u.user_id, u.user_colour, u.username',
							'FROM'		=> array(
								ARCADE_SCORES_TABLE	=> 's',
							),
							'LEFT_JOIN'	=> array(
								array(
									'FROM'	=> array(ARCADE_PLAYS_TABLE => 'p'),
									'ON'	=> 's.game_id = p.game_id AND s.user_id = p.user_id'
								),
								array(
									'FROM'	=> array(USERS_TABLE => 'u'),
									'ON'	=> 's.user_id = u.user_id'
								),
							),
							'WHERE'		=> 's.game_id = ' . (int) $game_id,
							'ORDER_BY'	=> 's.score ' . $score_order . ', s.score_date ASC',
						);

						$sql = $db->sql_build_query('SELECT', $sql_array);
						$result = $db->sql_query_limit($sql, $arcade_config['acp_items_per_page'], $start);

						$position = ($start > 0 ) ? $start : 0;
						$actual_position = ($start > 0 ) ? $start : 0;
						$lastscore = '';
						while ($row = $db->sql_fetchrow($result))
						{
							$actual_position++;

							if ($lastscore != $row['score'])
							{
								$position = $actual_position;
							}
							$lastscore = $row['score'] ;

							$url = $this->u_action . '&amp;g=' . $row['game_id'] . '&amp;u=' . $row['user_id'];

							$template->assign_block_vars('scorerow', array(
								'POS'			=> $position,
								'USERNAME'		=> $arcade->adm_username($row['user_id'], $row['username'], $row['user_colour'], $this->u_action),
								'SCORE'			=> $arcade->number_format($row['score']),
								'COMMENT'		=> generate_text_for_display($row['comment_text'], $row['comment_uid'], $row['comment_bitfield'], $row['comment_options']),
								'DATE'			=> $user->format_date($row['score_date']),
								'TOTAL_PLAYS'	=> $row['total_plays'],
								'TOTAL_TIME'	=> $arcade->time_format($row['total_time']),
								'U_EDIT'		=> $url . '&amp;action=edit',
								'U_DELETE'		=> $url . '&amp;action=delete',
							));
						}
						$db->sql_freeresult($result);

						if ($super_champion = $arcade->super_champion($game_id))
						{
							$url = $this->u_action . '&amp;g=' . $game_data['game_id'] . '&amp;u=' . $super_champion['user_id'];

							$template->assign_vars(array(
								'S_SUPER_CHAMPION'			=> true,

								'U_EDIT_SUPER_CHAMPION'		=> $url . '&amp;action=champ_edit',
								'U_DELETE_SUPER_CHAMPION'	=> $url . '&amp;action=champ_delete',

								'SUPER_CHAMPION_USERNAME'	=> $arcade->adm_username($super_champion['user_id'], $super_champion['username'], $super_champion['user_colour'], $this->u_action),
								'SUPER_CHAMPION_SCORE'		=> (!$arcade->hidden_score($game_data['game_id'], $super_champion['user_id'])) ? $arcade->number_format($super_champion['score']) : $user->lang['HIDDEN'],
								'SUPER_CHAMPION_DATE'		=> $user->format_date($super_champion['score_date'])
							));
						}

						$l_title_explain	= $user->lang['ACP_ARCADE_EDIT_SCORES_LIST_EXPLAIN'];
						$total_scores		= $arcade->get_total('scores', $game_id);

						$template->assign_vars(array(
							'S_GAME_SCORES'		=> true,

							'GAME_TITLE'		=> sprintf($user->lang['ARCADE_GAME_SCORES'], $game_data['game_name']),

							'S_ON_PAGE'			=> on_page($total_scores, $arcade_config['acp_items_per_page'], $start),
							'PAGINATION'		=> generate_pagination($this->u_action . '&amp;action=show_scores&amp;g=' . $game_id, $total_scores, $arcade_config['acp_items_per_page'], $start, true),
						));

						$return = true;
					break;

					case 'champ_edit':
						if (!$game_id || !$user_id)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						if ($update)
						{
							if (!check_form_key($form_key))
							{
								trigger_error('FORM_INVALID', E_USER_WARNING);
							}

							$score_data = array(
								'score'			=> $score,
								'score_date'	=> ($date_now) ? time(): $score_date
							);

							$game_data = $arcade->get_game_data($game_id);
							$arcade->update_super_score($score_data, $user_id, $game_id);
							$username = $arcade->userdata('username', $user_id);

							$arcade->add_log('admin', 'LOG_ARCADE_EDIT_SUPER_CHAMPION_SCORE', $username, '<a onclick="window.open(this.href); return false;" href="'. $arcade->url('mode=play&amp;g=' . $game_data['game_id']) .'">'. $game_data['game_name'] .'</a>');
							trigger_error(sprintf($user->lang['ARCADE_SUPER_CHAMPION_SCORE_UPDATED'], $username, $game_data['game_name']) . adm_back_link($this->u_action));
						}

						$super_champion = $arcade->super_champion($game_id);
						$game_data = $arcade->get_game_data($game_id);

						if (!$super_champion || !$game_data)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$l_title_explain = $user->lang['ACP_ARCADE_EDIT_SCORES_EDIT_EXPLAIN'];

						$template->assign_vars(array(
							'S_EDIT_CHAMP_SCORE'	=> true,

							'U_EDIT_CHAMP_ACTION'	=> $this->u_action . "&amp;g=$game_id&amp;u=$user_id&amp;action=$action",

							'CHAMP_USERNAME'		=> $arcade->adm_username($super_champion['user_id'], $super_champion['username'], $super_champion['user_colour'], $this->u_action),
							'CHAMP_GAME_NAME'		=> $game_data['game_name'],
							'CHAMP_SCORE'			=> $super_champion['score'],
							'CHAMP_DATE_RAW'		=> $super_champion['score_date'],
							'CHAMP_DATE'			=> $user->format_date($super_champion['score_date'])
						));

						$return = true;
					break;

					case 'champ_delete':
						if (!$game_id || !$user_id)
						{
							trigger_error($user->lang['NO_USERS_OR_GAMES'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						if (confirm_box(true))
						{
							$game_data = $arcade->get_game_data($game_id);
							$arcade->delete_super_score($game_id, $user_id);
							$username = $arcade->userdata('username', $user_id);

							$arcade->add_log('admin', 'LOG_ARCADE_DELETE_SUPER_CHAMPION_SCORE', $username, '<a onclick="window.open(this.href); return false;" href="'. $arcade->url('mode=play&amp;g=' . $game_data['game_id']) .'">'. $game_data['game_name'] .'</a>');
							trigger_error(sprintf($user->lang['ARCADE_SUPER_CHAMPION_SCORE_DELETED'], $username, $game_data['game_name']) . adm_back_link($this->u_action));
						}
						else
						{
							confirm_box(false, 'DELETE_SELECTED_SUPER_CHAMPION_SCORE');
						}
					break;
				}

				if (!$return)
				{
					$sql_array = array(
						'SELECT'	=> 'g.game_id , g.game_name, g.game_name_clean, g.game_image, g.game_desc, g.game_plays, g.cat_id',
						'FROM'		=> array(
							ARCADE_GAMES_TABLE	=> 'g',
						),
						'LEFT_JOIN'	=> array(
							array(
								'FROM'	=> array(ARCADE_SCORES_TABLE => 's'),
								'ON'	=> 'g.game_id = s.game_id'
							),
							array(
								'FROM'	=> array(ARCADE_SUPER_CHAMP_TABLE => 'sc'),
								'ON'	=> 'g.game_id = sc.game_id'
							),
						),
						'WHERE'		=> 'g.game_id = s.game_id OR g.game_id = sc.game_id',
						'ORDER_BY'	=> 'g.game_name_clean ASC',
					);

					$sql = $db->sql_build_query('SELECT_DISTINCT', $sql_array);
					$result = $db->sql_query_limit($sql, $arcade_config['acp_items_per_page'], $start);

					while($row = $db->sql_fetchrow($result))
					{
						$url = $this->u_action . '&amp;g=' . $row['game_id'];

						$template->assign_block_vars('games', array(
							'GAME_ID'		=> $row['game_id'],
							'GAME_NAME'		=> get_arcade_game_name($row['game_name'], false, 'play', $row['cat_id'], $row['game_id'], 'x', false, false, false, true),
							'GAME_DESC'		=> $row['game_desc'],
							'GAME_IMAGE'	=> ($arcade->optionget('view_game_image')) ? get_arcade_game_image($row['game_image'], 30, 30, 'x', 'play', $row['cat_id'], $row['game_id'], false, true) : false,
							'GAME_PLAYS'	=> $row['game_plays'],

							'U_EDIT'		=> $url . '&amp;action=show_scores',
							'U_SYNC'		=> $url . '&amp;action=sync&amp;start=' . $start
						));
					}
					$db->sql_freeresult($result);

					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					// Start quick jump of all the games in the arcade with scores...
					foreach ($row as $game)
					{
						$template->assign_block_vars('quick_jump', array(
							'GAME_ID'	=> $game['game_id'],
							'GAME_NAME'	=> $game['game_name']
						));
					}

					$s_hidden_fields = array('action' => 'show_scores');

					// End quick jump of all the games in the arcade...

					// This next bit of code is used to grab all the users that have some type of arcade data stored
					$user_ids = array();
					// These tables could all possibly contain user related arcade data
					$tables = array(ARCADE_FAVS_TABLE, ARCADE_PLAYS_TABLE, ARCADE_RATING_TABLE, ARCADE_SCORES_TABLE, ARCADE_SUPER_CHAMP_TABLE);
					foreach ($tables as $table)
					{
						$sql = "SELECT DISTINCT user_id from $table ORDER BY user_id ASC";
						$result = $db->sql_query($sql);

						while ($row = $db->sql_fetchrow($result))
						{
							$user_ids[] = $row['user_id'];
						}
						$db->sql_freeresult($result);
					}

					// Order and remove the duplicated user ids
					$user_ids = array_unique($user_ids);
					$l_title_explain = $user->lang['ACP_ARCADE_MANAGE_USERS_EXPLAIN'];

					if (sizeof($user_ids))
					{
						// Now lets get the usernames for all the users
						$sql = 'SELECT user_id, username, username_clean
								FROM ' . USERS_TABLE . '
								WHERE ' . $db->sql_in_set('user_id', $user_ids) . '
								ORDER BY username_clean ASC';
						$result = $db->sql_query($sql);

						$user_data = $db->sql_fetchrowset($result);
						$db->sql_freeresult($result);

						foreach ($user_data as $item)
						{
							$template->assign_block_vars('user_jump', array(
								'USER_ID'	=> $item['user_id'],
								'USERNAME'	=> $item['username']
							));
						}
					}

					$total_scores = $arcade->get_total('scores');

					$template->assign_vars(array(
						'S_PROGRESS_BAR'	=> true,
						'S_USERS_OPTIONS'	=> true,
						'S_HIDDEN_FIELDS'	=> build_hidden_fields($s_hidden_fields),
						'S_SHOW_POINTS'		=> ($arcade->points['installed'] && $auth->acl_get('a_arcade_points_settings')),

						'U_ACTION'			=> $this->u_action,
						'U_PROGRESS_BAR'	=> $this->u_action . '&amp;action=progress_bar',
						'UA_PROGRESS_BAR'	=> str_replace('&amp;', '&', $this->u_action) . '&action=progress_bar',
						'U_FIND_USERNAME'	=> append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=searchuser&amp;form=select_user&amp;field=username&amp;select_single=true'),

						'ANONYMOUS_USER_ID'	=> ANONYMOUS,

						'S_ON_PAGE'			=> on_page($total_scores, $arcade_config['acp_items_per_page'], $start),
						'PAGINATION'		=> generate_pagination($this->u_action, $total_scores, $arcade_config['acp_items_per_page'], $start, true)
					));
				}
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}

		$this->page_title = $page_title;

		if (sizeof($errors))
		{
			$template->assign_vars(array(
				'S_ERROR'	=> true,
				'ERROR_MSG'	=> implode('<br />', $errors)
			));
		}

		$template->assign_vars(array(
			'S_GAME_DELETE'			=> ($auth->acl_get('a_arcade_delete_game')) ? true : false,
			'S_DISPLAY_GAME_IMAGES'	=> ($arcade_config['display_game_image'] && $arcade->optionget('view_game_image')) ? true : false,

			'L_TITLE'				=> (!empty($l_title)) ? $l_title : '',
			'L_TITLE_EXPLAIN'		=> (!empty($l_title_explain)) ? $l_title_explain : ''
		));
	}

	function cat_display_select($value = '')
	{
			global $user;

			return '<option value="' . ARCADE_CAT_DISPLAY_BOTH . '"' . (($value == ARCADE_CAT_DISPLAY_BOTH) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_CAT_DISPLAY_BOTH'] . '</option><option value="' . ARCADE_CAT_DISPLAY_NAME . '"' . (($value == ARCADE_CAT_DISPLAY_NAME) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_CAT_DISPLAY_NAME'] . '</option><option value="' . ARCADE_CAT_DISPLAY_IMAGE . '"' . (($value == ARCADE_CAT_DISPLAY_IMAGE) ? ' selected="selected"' : '') . '>' . $user->lang['ARCADE_CAT_DISPLAY_IMAGE'] . '</option>';
	}

	/**
	* Update category data
	*/
	function update_cat_data(&$cat_data)
	{
		global $db, $user, $arcade;

		$errors = array();

		if ($cat_data['cat_name'] == '')
		{
			$errors[] = $user->lang['CAT_NAME_EMPTY'];
		}

		if (utf8_strlen($cat_data['cat_desc']) > 4000)
		{
			$errors[] = $user->lang['CAT_DESC_TOO_LONG'];
		}

		if (utf8_strlen($cat_data['cat_rules']) > 4000)
		{
			$errors[] = $user->lang['CAT_RULES_TOO_LONG'];
		}

		if ($cat_data['cat_password'] || $cat_data['cat_password_confirm'])
		{
			if ($cat_data['cat_password'] != $cat_data['cat_password_confirm'])
			{
				$cat_data['cat_password'] = $cat_data['cat_password_confirm'] = '';
				$errors[] = $user->lang['CAT_PASSWORD_MISMATCH'];
			}
		}

		// Set category flags
		// 1 = link tracking
		$cat_data['cat_flags'] = 0;
		$cat_data['cat_flags'] += ($cat_data['cat_link_track']) ? ARCADE_FLAG_LINK_TRACK : 0;

		// Unset data that are not database fields
		$cat_data_sql = $cat_data;

		unset($cat_data_sql['cat_link_track']);
		unset($cat_data_sql['cat_password_confirm']);

		// What are we going to do tonight Brain? The same thing we do everynight,
		// try to take over the world ... or decide whether to continue update
		// and if so, whether it's a new category/cat/link or an existing one
		if (sizeof($errors))
		{
			return $errors;
		}

		// As we don't know the old password, it's kinda tricky to detect changes
		if ($cat_data_sql['cat_password_unset'])
		{
			$cat_data_sql['cat_password'] = '';
		}
		else if (empty($cat_data_sql['cat_password']))
		{
			unset($cat_data_sql['cat_password']);
		}
		else
		{
			$cat_data_sql['cat_password'] = phpbb_hash($cat_data_sql['cat_password']);
		}
		unset($cat_data_sql['cat_password_unset']);

		if (!isset($cat_data_sql['cat_id']))
		{
			// no cat_id means we're creating a new category
			unset($cat_data_sql['type_action']);

			if ($cat_data_sql['parent_id'])
			{
				$sql = 'SELECT left_id, right_id, cat_type
						FROM ' . ARCADE_CATS_TABLE . '
						WHERE cat_id = ' . $cat_data_sql['parent_id'];
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					trigger_error($user->lang['ARCADE_PARENT_NOT_EXIST'] . adm_back_link($this->u_action . '&amp;' . $this->parent_id), E_USER_WARNING);
				}

				if ($row['cat_type'] == ARCADE_LINK)
				{
					$errors[] = $user->lang['PARENT_IS_LINK_ARCADE'];
					return $errors;
				}

				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET left_id = left_id + 2, right_id = right_id + 2
						WHERE left_id > ' . $row['right_id'];
				$db->sql_query($sql);

				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET right_id = right_id + 2
						WHERE ' . $row['left_id'] . ' BETWEEN left_id AND right_id';
				$db->sql_query($sql);

				$cat_data_sql['left_id'] = $row['right_id'];
				$cat_data_sql['right_id'] = $row['right_id'] + 1;
			}
			else
			{
				$sql = 'SELECT MAX(right_id) AS right_id
						FROM ' . ARCADE_CATS_TABLE;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$cat_data_sql['left_id'] = $row['right_id'] + 1;
				$cat_data_sql['right_id'] = $row['right_id'] + 2;
			}

			$db->sql_query('INSERT INTO ' . ARCADE_CATS_TABLE . ' ' . $db->sql_build_array('INSERT', $cat_data_sql));

			$cat_data['cat_id'] = $db->sql_nextid();

			$arcade->add_log('admin', 'LOG_ARCADE_ADD_CAT', $cat_data['cat_name']);
		}
		else
		{
			$row = $arcade->get_cat_info($cat_data_sql['cat_id']);

			if ($row['cat_type'] == ARCADE_CAT_GAMES && $row['cat_type'] != $cat_data_sql['cat_type'])
			{
				// Has subcategories and want to change into a link?
				if ($row['right_id'] - $row['left_id'] > 1 && $cat_data_sql['cat_type'] == ARCADE_LINK)
				{
					$errors[] = $user->lang['CAT_WITH_SUBCATS_NOT_TO_LINK'];
					return $errors;
				}

				// we're turning a postable category into a non-postable category
				if ($cat_data_sql['type_action'] == 'move')
				{
					$to_cat_id = request_var('to_cat_id', 0);

					if ($to_cat_id)
					{
						$errors = $this->move_cat_content($cat_data_sql['cat_id'], $to_cat_id);
					}
					else
					{
						return array($user->lang['NO_DESTINATION_CAT']);
					}
				}
				else if ($cat_data_sql['type_action'] == 'delete')
				{
					$delete_files = (request_var('delete_files', false)) ? true : false;

					$errors = $this->delete_cat_content($cat_data_sql['cat_id'], $delete_files);
				}
				else
				{
					return array($user->lang['NO_CAT_ACTION']);
				}

				$cat_data_sql['cat_games'] = $cat_data_sql['cat_plays'] = $cat_data_sql['cat_last_play_game_id'] = $cat_data_sql['cat_last_game_installdate'] = $cat_data_sql['cat_last_play_user_id'] = $cat_data_sql['cat_last_play_score'] = $cat_data_sql['cat_last_play_time'] = 0;
				$cat_data_sql['cat_last_play_game_name'] = $cat_data_sql['cat_last_play_username'] = $cat_data_sql['cat_last_play_user_colour'] = '';
			}
			else if ($row['cat_type'] == ARCADE_CAT && $cat_data_sql['cat_type'] == ARCADE_LINK)
			{
				// Has subcategories?
				if ($row['right_id'] - $row['left_id'] > 1)
				{
					// We are turning a category into a link - but need to decide what to do with the subcategories.
					$action_subcats = request_var('action_subcats', '');
					$subcats_to_id = request_var('subcats_to_id', 0);

					if ($action_subcats == 'delete')
					{
						$rows = $arcade->get_cat_branch($row['cat_id'], 'children', 'descending', false);

						$sub_delete_files = (request_var('sub_delete_files', false)) ? true : false;

						foreach ($rows as $_row)
						{
							// Do not remove the category id we are about to change. ;)
							if ($_row['cat_id'] == $row['cat_id'])
							{
								continue;
							}

							$cat_ids[] = $_row['cat_id'];
							$errors = array_merge($errors, $this->delete_cat_content($_row['cat_id'], $sub_delete_files));
						}

						if (sizeof($errors))
						{
							return $errors;
						}

						if (sizeof($cat_ids))
						{
							$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . '
									WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
							$db->sql_query($sql);

							$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . '
									WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
							$db->sql_query($sql);

							$sql = 'DELETE FROM ' . ACL_ARCADE_USERS_TABLE . '
									WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
							$db->sql_query($sql);
						}
					}
					else if ($action_subcats == 'move')
					{
						if (!$subcats_to_id)
						{
							return array($user->lang['NO_DESTINATION_CAT']);
						}

						$sql = 'SELECT cat_name
								FROM ' . ARCADE_CATS_TABLE . '
								WHERE cat_id = ' . $subcats_to_id;
						$result = $db->sql_query($sql);
						$_row = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);

						if (!$_row)
						{
							return array($user->lang['NO_CAT_ID']);
						}

						$subcats_to_name = $_row['cat_name'];

						$sql = 'SELECT cat_id
								FROM ' . ARCADE_CATS_TABLE . "
								WHERE parent_id = {$row['cat_id']}";
						$result = $db->sql_query($sql);

						while ($_row = $db->sql_fetchrow($result))
						{
							$this->move_cat($_row['cat_id'], $subcats_to_id);
						}
						$db->sql_freeresult($result);

						$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
								SET parent_id = $subcats_to_id
								WHERE parent_id = {$row['cat_id']}";
						$db->sql_query($sql);
					}

					// Adjust the left/right id
					$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
							SET right_id = left_id + 1
							WHERE cat_id = ' . $row['cat_id'];
					$db->sql_query($sql);
				}
			}
			else if ($row['cat_type'] == ARCADE_CAT && $cat_data_sql['cat_type'] == ARCADE_CAT_GAMES)
			{
				// Changing a category to a category? Reset the data (you can't post directly in a cat, you must use a category)
				$cat_data_sql['cat_games'] = 0;
				$cat_data_sql['cat_plays'] = 0;
				$cat_data_sql['cat_last_play_game_id'] = 0;
				$cat_data_sql['cat_last_play_game_name'] = '';
				$cat_data_sql['cat_last_game_installdate'] = 0;
				$cat_data_sql['cat_last_play_user_id'] = 0;
				$cat_data_sql['cat_last_play_score'] = 0;
				$cat_data_sql['cat_last_play_time'] = 0;
				$cat_data_sql['cat_last_play_username'] = '';
				$cat_data_sql['cat_last_play_user_colour'] = '';
			}

			if (sizeof($errors))
			{
				return $errors;
			}

			if ($row['parent_id'] != $cat_data_sql['parent_id'])
			{
				if ($row['cat_id'] != $cat_data_sql['parent_id'])
				{
					$errors = $this->move_cat($cat_data_sql['cat_id'], $cat_data_sql['parent_id']);
				}
				else
				{
					$cat_data_sql['parent_id'] = $row['parent_id'];
				}
			}

			if (sizeof($errors))
			{
				return $errors;
			}

			unset($cat_data_sql['type_action']);

			if ($row['cat_name'] != $cat_data_sql['cat_name'])
			{
				// the category name has changed, clear the parents list of all categories (for safety)
				$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
						SET cat_parents = ''";
				$db->sql_query($sql);
			}

			// Setting the category id to the category id is not really received well by some dbs. ;)
			$cat_id = $cat_data_sql['cat_id'];
			unset($cat_data_sql['cat_id']);

			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
					SET ' . $db->sql_build_array('UPDATE', $cat_data_sql) . '
					WHERE cat_id = ' . $cat_id;
			$db->sql_query($sql);

			// Add it back
			$cat_data['cat_id'] = $cat_id;

			$arcade->add_log('admin', 'LOG_ARCADE_CAT_EDIT', $cat_data['cat_name']);
		}

		return $errors;
	}

	/**
	* Move category
	*/
	function move_cat($from_id, $to_id)
	{
		global $db, $user, $arcade;

		$to_data = $moved_ids = $errors = array();

		// Check if we want to move to a parent with link type
		if ($to_id > 0)
		{
			$to_data = $arcade->get_cat_info($to_id);

			if ($to_data['cat_type'] == ARCADE_LINK)
			{
				$errors[] = $user->lang['PARENT_IS_LINK_ARCADE'];
				return $errors;
			}
		}

		$moved_categories = $arcade->get_cat_branch($from_id, 'children', 'descending');
		$from_data = $moved_categories[0];
		$diff = sizeof($moved_categories) * 2;

		$moved_ids = array();
		for ($i = 0; $i < sizeof($moved_categories); ++$i)
		{
			$moved_ids[] = $moved_categories[$i]['cat_id'];
		}

		// Resync parents
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET right_id = right_id - $diff, cat_parents = ''
				WHERE left_id < " . $from_data['right_id'] . "
				AND right_id > " . $from_data['right_id'];
		$db->sql_query($sql);

		// Resync righthand side of tree
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id - $diff, right_id = right_id - $diff, cat_parents = ''
				WHERE left_id > " . $from_data['right_id'];
		$db->sql_query($sql);

		if ($to_id > 0)
		{
			// Retrieve $to_data again, it may have been changed...
			$to_data = $arcade->get_cat_info($to_id);

			// Resync new parents
			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
					SET right_id = right_id + $diff, cat_parents = ''
					WHERE " . $to_data['right_id'] . ' BETWEEN left_id AND right_id
					AND ' . $db->sql_in_set('cat_id', $moved_ids, true);
			$db->sql_query($sql);

			// Resync the righthand side of the tree
			$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
					SET left_id = left_id + $diff, right_id = right_id + $diff, cat_parents = ''
					WHERE left_id > " . $to_data['right_id'] . '
					AND ' . $db->sql_in_set('cat_id', $moved_ids, true);
			$db->sql_query($sql);

			// Resync moved branch
			$to_data['right_id'] += $diff;

			if ($to_data['right_id'] > $from_data['right_id'])
			{
				$diff = '+ ' . ($to_data['right_id'] - $from_data['right_id'] - 1);
			}
			else
			{
				$diff = '- ' . abs($to_data['right_id'] - $from_data['right_id'] - 1);
			}
		}
		else
		{
			$sql = 'SELECT MAX(right_id) AS right_id
					FROM ' . ARCADE_CATS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $moved_ids, true);
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$diff = '+ ' . ($row['right_id'] - $from_data['left_id'] + 1);
		}

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id $diff, right_id = right_id $diff, cat_parents = ''
				WHERE " . $db->sql_in_set('cat_id', $moved_ids);
		$db->sql_query($sql);

		return $errors;
	}

	/**
	* Move category content from one to another category
	*/
	function move_cat_content($from_id, $to_id, $sync = true)
	{
		global $db, $arcade;

		$table_ary = array(ARCADE_GAMES_TABLE);

		foreach ($table_ary as $table)
		{
			$sql = "UPDATE $table
					SET cat_id = $to_id
					WHERE cat_id = $from_id";
			$db->sql_query($sql);
		}
		unset($table_ary);

		if ($sync)
		{
			// Sync total games and plays for the categories...
			$arcade->sync('category');
		}
		return array();
	}

	/**
	* Remove complete category
	*/
	function delete_cat($cat_id, $action_games = 'delete', $delete_files = false, $action_subcats = 'delete', $sub_delete_files = false, $games_to_id = 0, $subcats_to_id = 0)
	{
		global $db, $user, $arcade;

		$cat_data = $arcade->get_cat_info($cat_id);

		$errors = array();
		$log_action_games = $log_action_cats = $games_to_name = $subcats_to_name = '';
		$cat_ids = array($cat_id);

		if ($action_games == 'delete')
		{
			$log_action_games = 'GAMES';
			$errors = array_merge($errors, $this->delete_cat_content($cat_id, $delete_files));
		}
		else if ($action_games == 'move')
		{
			if (!$games_to_id)
			{
				$errors[] = $user->lang['NO_DESTINATION_CAT'];
			}
			else
			{
				$log_action_games = 'MOVE_GAMES';

				$sql = 'SELECT cat_name
						FROM ' . ARCADE_CATS_TABLE . '
						WHERE cat_id = ' . $games_to_id;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					$errors[] = $user->lang['NO_CAT_ID'];
				}
				else
				{
					$posts_to_name = $row['cat_name'];
					$errors = array_merge($errors, $this->move_cat_content($cat_id, $games_to_id));
				}
			}
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		if ($action_subcats == 'delete')
		{
			$log_action_cats = 'CATS';
			$rows = $arcade->get_cat_branch($cat_id, 'children', 'descending', false);

			foreach ($rows as $row)
			{
				$cat_ids[] = $row['cat_id'];
				$errors = array_merge($errors, $this->delete_cat_content($row['cat_id'], $sub_delete_files));
			}

			if (sizeof($errors))
			{
				return $errors;
			}

			$diff = sizeof($cat_ids) * 2;

			$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
			$db->sql_query($sql);

			$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
			$db->sql_query($sql);

			$sql = 'DELETE FROM ' . ACL_ARCADE_USERS_TABLE . '
					WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
			$db->sql_query($sql);
		}
		else if ($action_subcats == 'move')
		{
			if (!$subcats_to_id)
			{
				$errors[] = $user->lang['NO_DESTINATION_CAT'];
			}
			else
			{
				$log_action_cats = 'MOVE_CATS';

				$sql = 'SELECT cat_name
						FROM ' . ARCADE_CATS_TABLE . '
						WHERE cat_id = ' . $subcats_to_id;
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				if (!$row)
				{
					$errors[] = $user->lang['NO_CAT_ID'];
				}
				else
				{
					$subcats_to_name = $row['cat_name'];

					$sql = 'SELECT cat_id
							FROM ' . ARCADE_CATS_TABLE . "
							WHERE parent_id = $cat_id";
					$result = $db->sql_query($sql);

					while ($row = $db->sql_fetchrow($result))
					{
						$this->move_cat($row['cat_id'], $subcats_to_id);
					}
					$db->sql_freeresult($result);

					// Grab new category data for correct tree updating later
					$cat_data = $arcade->get_cat_info($cat_id);

					$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
							SET parent_id = $subcats_to_id
							WHERE parent_id = $cat_id";
					$db->sql_query($sql);

					$diff = 2;
					$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . "
							WHERE cat_id = $cat_id";
					$db->sql_query($sql);

					$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . "
							WHERE cat_id = $cat_id";
					$db->sql_query($sql);

					$sql = 'DELETE FROM ' . ACL_ARCADE_USERS_TABLE . "
							WHERE cat_id = $cat_id";
					$db->sql_query($sql);
				}
			}

			if (sizeof($errors))
			{
				return $errors;
			}
		}
		else
		{
			$diff = 2;
			$sql = 'DELETE FROM ' . ARCADE_CATS_TABLE . "
					WHERE cat_id = $cat_id";
			$db->sql_query($sql);

			$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . "
					WHERE cat_id = $cat_id";
			$db->sql_query($sql);

			$sql = 'DELETE FROM ' . ACL_ARCADE_USERS_TABLE . "
					WHERE cat_id = $cat_id";
			$db->sql_query($sql);
		}

		// Resync tree
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET right_id = right_id - $diff
				WHERE left_id < {$cat_data['right_id']} AND right_id > {$cat_data['right_id']}";
		$db->sql_query($sql);

		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id - $diff, right_id = right_id - $diff
				WHERE left_id > {$cat_data['right_id']}";
		$db->sql_query($sql);

		$log_action = implode('_', array($log_action_games, $log_action_cats));

		switch ($log_action)
		{
			case 'MOVE_GAMES_MOVE_CATS':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_MOVE_GAMES_MOVE_CATS', $posts_to_name, $subcats_to_name, $cat_data['cat_name']);
			break;

			case 'MOVE_GAMES_CATS':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_MOVE_GAMES_CATS', $posts_to_name, $cat_data['cat_name']);
			break;

			case 'GAMES_MOVE_CATS':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_GAMES_MOVE_CATS', $subcats_to_name, $cat_data['cat_name']);
			break;

			case '_MOVE_CATS':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_MOVE_CATS', $subcats_to_name, $cat_data['cat_name']);
			break;

			case 'MOVE_GAMES_':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_MOVE_GAMES', $posts_to_name, $cat_data['cat_name']);
			break;

			case 'GAMES_CATS':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_GAMES_CATS', $cat_data['cat_name']);
			break;

			case '_CATS':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_CATS', $cat_data['cat_name']);
			break;

			case 'GAMES_':
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_GAMES', $cat_data['cat_name']);
			break;

			default:
				$arcade->add_log('admin', 'LOG_ARCADE_DEL_CAT', $cat_data['cat_name']);
			break;
		}

		return $errors;
	}

	/**
	* Delete category content
	*/
	function delete_cat_content($cat_id, $delete_files = false)
	{
		global $db, $user, $cache, $arcade;

		$errors = array();
		if (!$cat_id)
		{
			$errors[] = $user->lang['NO_CAT_ID'];
		}

		if (sizeof($errors))
		{
			return $errors;
		}

		$sql = 'SELECT game_id, game_swf FROM ' . ARCADE_GAMES_TABLE . '
				WHERE cat_id = ' . $cat_id;
		$result = $db->sql_query($sql);
		while ($row = $db->sql_fetchrow($result))
		{
			$errors = array_merge($errors, $arcade->delete_game($row['game_id'], $cat_id, $delete_files, $row['game_swf'], false));
		}
		$db->sql_freeresult($result);

		$cache->destroy('sql', array(ARCADE_CATS_TABLE, ARCADE_GAMES_TABLE, ARCADE_SCORES_TABLE, ARCADE_PLAYS_TABLE, ARCADE_CHALLENGE_TABLE, ARCADE_CHALLENGE_CHAMP_TABLE));
		$cache->destroy('_arcade_cats');
		$cache->destroy('_arcade_leaders');
		$cache->destroy('_arcade_leaders_all');
		$cache->destroy('_arcade_games_filesize');
		$cache->destroy('_arcade_challenge_leaders');
		$cache->destroy('_arcade_challenge_leaders_all');

		$arcade->sync('total_data', 'all');
		$arcade->set_last_play($cat_id);
		$arcade->sync('category', $cat_id);

		return $errors;
	}

	/**
	* Move category position by $steps up/down
	*/
	function move_cat_by($cat_row, $action = 'move_up', $steps = 1)
	{
		global $db;

		/**
		* Fetch all the siblings between the module's current spot
		* and where we want to move it to. If there are less than $steps
		* siblings between the current spot and the target then the
		* module will move as far as possible
		*/
		$sql = 'SELECT cat_id, cat_name, left_id, right_id
				FROM ' . ARCADE_CATS_TABLE . "
				WHERE parent_id = {$cat_row['parent_id']}
				AND " . (($action == 'move_up') ? "right_id < {$cat_row['right_id']} ORDER BY right_id DESC" : "left_id > {$cat_row['left_id']} ORDER BY left_id ASC");
		$result = $db->sql_query_limit($sql, $steps);

		$target = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$target = $row;
		}
		$db->sql_freeresult($result);

		if (!sizeof($target))
		{
			// The category is already on top or bottom
			return false;
		}

		/**
		* $left_id and $right_id define the scope of the nodes that are affected by the move.
		* $diff_up and $diff_down are the values to substract or add to each node's left_id
		* and right_id in order to move them up or down.
		* $move_up_left and $move_up_right define the scope of the nodes that are moving
		* up. Other nodes in the scope of ($left_id, $right_id) are considered to move down.
		*/
		if ($action == 'move_up')
		{
			$left_id = $target['left_id'];
			$right_id = $cat_row['right_id'];

			$diff_up = $cat_row['left_id'] - $target['left_id'];
			$diff_down = $cat_row['right_id'] + 1 - $cat_row['left_id'];

			$move_up_left = $cat_row['left_id'];
			$move_up_right = $cat_row['right_id'];
		}
		else
		{
			$left_id = $cat_row['left_id'];
			$right_id = $target['right_id'];

			$diff_up = $cat_row['right_id'] + 1 - $cat_row['left_id'];
			$diff_down = $target['right_id'] - $cat_row['right_id'];

			$move_up_left = $cat_row['right_id'] + 1;
			$move_up_right = $target['right_id'];
		}

		// Now do the dirty job
		$sql = 'UPDATE ' . ARCADE_CATS_TABLE . "
				SET left_id = left_id + CASE
				WHEN left_id BETWEEN {$move_up_left} AND {$move_up_right} THEN -{$diff_up}
				ELSE {$diff_down}
				END,
				right_id = right_id + CASE
				WHEN right_id BETWEEN {$move_up_left} AND {$move_up_right} THEN -{$diff_up}
				ELSE {$diff_down}
				END,
				cat_parents = ''
				WHERE
				left_id BETWEEN {$left_id} AND {$right_id}
				AND right_id BETWEEN {$left_id} AND {$right_id}";
		$db->sql_query($sql);

		return $target['cat_name'];
	}

	function game_control($value, $key = '')
	{
		global $arcade;

		return '<select id="' . $key . '" name="config[' . $key . ']">' . $arcade->game_control_select($value) . '</select>
				<textarea id="game_control_desc" name="config[game_control_desc]" rows="5" cols="25">' . $this->new_game_data['game_control_desc'] . '</textarea>';
	}

	function game_swf()
	{
		return $this->new_game_data['game_swf'];
	}

	function game_installdate()
	{
		global $user;

		return $user->format_date($this->new_game_data['game_installdate']);
	}

	function game_filesize()
	{
		return get_formatted_filesize($this->new_game_data['game_filesize']);
	}

	function game_type_select($value)
	{
		global $arcade;

		return $arcade->game_type_select($value);
	}

	function game_scoretype_select($value)
	{
		global $arcade;

		return $arcade->game_scoretype_select($value);
	}

	function game_cat_select($value)
	{
		global $arcade;
		return $arcade->make_cat_select($value, false, false, true);
	}
}

?>