<?php
/**
*
* @package phpBB Arcade acp
* @version $Id: acp_arcade_plugin.php 182 2013-01-06 01:40:35Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class acp_arcade_plugin
{
	var $u_action;
	var $tpl_name;
	var $page_title;
	var $plugin_id = 0;
	var $new_config = array();

	function main($id, $mode)
	{
		global $db, $user, $cache, $auth, $config, $template, $phpbb_admin_path, $phpbb_root_path, $phpEx;
		global $arcade, $auth_arcade, $arcade_cache, $arcade_config, $arcade_plugin, $file_functions;

		define('IN_PHPBB_ARCADE', true);
		$arcade->begin();

		$this->tpl_name = 'arcade/acp_plugin';
		$action = request_var('action', '');
		$submit = (isset($_POST['submit'])) ? true : false;
		$inc_install = (in_array($action, array('requirements', 'install', 'update', 'remove', 'verify'))) ? true : false;

		if ($inc_install)
		{
			global $db_tools, $umil, $arcade_install;

			$user->add_lang(array('install', 'mods/arcade/install'));

			if (!file_exists($phpbb_root_path . 'umil/umil.' . $phpEx))
			{
				trigger_error(sprintf($user->lang['ARCADE_UMIL_DOWNLOAD'], '<a onclick="window.open(this.href); return false;" href="http://www.phpbb.com/mods/umil/">phpBB.com/mods/umil</a>'), E_USER_ERROR);
			}

			include($phpbb_root_path . 'umil/umil.' . $phpEx);
			include($phpbb_root_path . 'arcade/includes/class_install.' . $phpEx);
			$arcade_install = new arcade_install(($action == 'update') ? true : false);
			$umil			= new arcade_umil(true);
		}

		$errors = array();
		$l_title_explain = '';

		switch ($mode)
		{
			case 'install':
				$this->page_title = 'ACP_ARCADE_PLUGIN_INSTALL';

				if ($submit)
				{
					$plugin_filename = request_var('plugin_filename', '');

					if (empty($plugin_filename))
					{
						$errors[] = $user->lang['ACP_ARCADE_NO_SELECT_PLUGIN'];
					}

					if (!sizeof($errors))
					{
						$plugin_install = $this->plugin_ini('install', $plugin_filename);

						if ($plugin_install != false)
						{
							$sql = 'SELECT plugin_filename
									FROM ' . ARCADE_PLUGINS_TABLE . "
									WHERE plugin_filename = '" . $db->sql_escape($plugin_filename) . "'";
							$result = $db->sql_query($sql);
							if ($db->sql_fetchfield('plugin_filename'))
							{
								$submit = false;
							}
							$db->sql_freeresult($result);

							if ($submit)
							{
								$this->validate_install_file($plugin_install);
								$install = $plugin_install['plugin_install_db'];
								$arcade_plugin->add_lang($plugin_filename);
								$plugin_name = $arcade->lang_value($plugin_install['plugin_name']);
							}
						}
						else
						{
							$errors[] = $user->lang['ACP_ARCADE_PLUGIN_INSTALL_ERROR'];
						}
					}
				}

				if (sizeof($errors))
				{
					$submit = false;
				}

				if ($submit)
				{
					switch ($action)
					{
						case 'requirements':
							$this->page_title = 'REQUIREMENTS_INSTALL_TITLE';

							$s_hidden_fields = build_hidden_fields(array(
								'action'			=> 'install',
								'plugin_filename'	=> $plugin_filename
							));

							$arcade_install->requirements('install', 'install', $plugin_install['plugin_requirements'], $this->u_action, $s_hidden_fields);
						break;

						case 'install':
							$sql = 'SELECT MAX(plugin_order) AS max_order
									FROM ' . ARCADE_PLUGINS_TABLE;
							$result = $db->sql_query($sql);
							$max_order = (int) $db->sql_fetchfield('max_order');
							$db->sql_freeresult($result);

							if (sizeof($install))
							{
								if ($install['custom_file'] !== false)
								{
									redirect(append_sid($phpbb_root_path . $install['custom_file'] . '.' . $phpEx));
								}

								if (sizeof($install['arcade_config']['add']))
								{
									$umil->arcade_config_add($install['arcade_config']['add']);
								}

								if (sizeof($install['phpbb_config']['add']))
								{
									$umil->config_add($install['phpbb_config']['add']);
								}

								if (sizeof($install['schema_data']) || sizeof($install['schema_changes']))
								{
									if (sizeof($install['schema_data']))
									{
										foreach ($install['schema_data'] as $table_name => $table_data)
										{
											$umil->table_add($table_name, $table_data);
										}
									}

									if (sizeof($install['schema_changes']))
									{
										$db_tools->perform_schema_changes($install['schema_changes']);
									}
								}

								if (sizeof($install['arcade_permission']['add']))
								{
									$umil->arcade_permission_add($install['arcade_permission']['add']);
								}

								if (sizeof($install['arcade_roles']['add']))
								{
									$umil->arcade_permission_role_add($install['arcade_roles']['add']);
								}

								if (sizeof($install['arcade_roles_set']))
								{
									$umil->arcade_permission_set($install['arcade_roles_set']);
								}

								if (sizeof($install['phpbb_permission']['add']))
								{
									$umil->permission_add($install['phpbb_permission']['add']);
								}

								if (sizeof($install['phpbb_roles']['add']))
								{
									$umil->permission_role_add($install['phpbb_roles']['add']);
								}

								if (sizeof($install['phpbb_roles_set']))
								{
									$umil->permission_set($install['phpbb_roles_set']);
								}

								if (sizeof($install['modules']['add']))
								{
									$umil->module_add($install['modules']['add']);
								}

								if (sizeof($plugin_install['plugin_requirements']['bbcodes']))
								{
									$arcade_install->add_bbcodes($plugin_install['plugin_requirements']['bbcodes']);
								}

								if (sizeof($install['purge_cache']))
								{
									$umil->cache_purge($install['purge_cache']);
								}
							}

							$class = $arcade_plugin->class_name($plugin_filename, 'install');
							//$class::install();
							call_user_func(array($class, 'install'));

							$max_order += 10;

							$plugin_data = array(
								'plugin_enabled'	=> 1,
								'plugin_name'		=> $plugin_install['plugin_name'],
								'plugin_filename'	=> $plugin_filename,
								'plugin_use'		=> $plugin_install['plugin_use'],
								'plugin_page'		=> $plugin_install['plugin_page'],
								'plugin_position'	=> $plugin_install['plugin_position'],
								'plugin_version'	=> $plugin_install['plugin_version'],
								'plugin_order'		=> $max_order,
								'plugin_date'		=> time()
							);

							$db->sql_query('INSERT INTO ' . ARCADE_PLUGINS_TABLE . ' ' . $db->sql_build_array('INSERT', $plugin_data));
							$plugin_id = $db->sql_nextid();

							$cache->destroy('_arcade_plugins');
							$arcade->add_log('admin', 'LOG_ARCADE_INSTALL_PLUGIN', $plugin_install['plugin_name']);

							$install_verify = '<br /><br /><a href="' . append_sid("{$phpbb_admin_path}index.$phpEx", "i=arcade_plugin&amp;mode=manage&amp;p={$plugin_id}&amp;action=verify") . '">&laquo; ' . $user->lang['ARCADE_INSTALL_CHECK'] . '</a>';
							trigger_error(sprintf($user->lang['ARCADE_INSTALL_PLUGIN_SUCCESS'], $plugin_name) . $install_verify);
						break;
					}
				}
				else
				{
					$s_plugin_options = '';
					$arcade_plugins = $this->plugin_data();

					// Now search for arcade plugins which is not yet installed...
					$am = @opendir($arcade_plugin->file_path(false, 'install'));

					if ($am)
					{
						while (($file = readdir($am)) !== false)
						{
							if (substr($file, -(strlen($phpEx) + 1)) === '.' . $phpEx)
							{
								$plugin_filename = substr($file, 0, -(strlen($phpEx) + 1));

								$installed_plugin = false;
								foreach ($arcade_plugins as $plugin_id => $row)
								{
									if ($row['plugin_filename'] == $plugin_filename)
									{
										$installed_plugin = true;
										break;
									}
								}

								if ($installed_plugin)
								{
									continue;
								}

								$plugin_install = $this->plugin_ini('install', $plugin_filename);

								if ($plugin_install != false)
								{
									$arcade_plugin->add_lang($plugin_filename);
									$s_plugin_options .= '<option value="' . $plugin_filename . '">' . $arcade->lang_value($plugin_install['plugin_name']) . "</option>\n";
								}
							}
						}
						closedir($am);
					}

					$s_hidden_fields = array(
						'action' => 'requirements'
					);

					$template->assign_vars(array(
						'S_ARCADE_PLUGIN_INSTALL'	=> true,
						'S_PLUGIN_OPTIONS'			=> ($s_plugin_options != '') ? $s_plugin_options : false,
						'S_HIDDEN_FIELDS'			=> build_hidden_fields($s_hidden_fields)
					));
				}
			break;

			case 'manage':
				$this->plugin_id	= intval(request_var('p', 0));
				$submit				= (isset($_POST['submit'])) ? true : false;

				$this->page_title	= 'ACP_ARCADE_PLUGIN_MANAGE';
				$form_key = 'acp_arcade_plugin_manage';
				add_form_key($form_key);

				$template->assign_var('S_ARCADE_PLUGIN_MANAGE', true);

				$arcade_plugins = $this->plugin_data();
				$arcade_plugin->add_lang(false, true, $arcade_plugins);

				switch ($action)
				{
					case 'enable':
						if ($this->plugin_update_check($arcade_plugins))
						{
							$errors[] = $user->lang['ARCADE_PLUGIN_BEING_UPDATED'];
						}
					case 'disable':
						if (!sizeof($errors))
						{
							$this->plugin_switching($action);
						}
					case 'move_up':
					case 'move_down':
						if (!sizeof($errors))
						{
							$reload_data = false;

							if (in_array($action, array('enable', 'disable')))
							{
								$reload_data = true;
								$arcade->add_log('admin', 'LOG_ARCADE_PLUGIN_' . strtoupper($action), $arcade_plugins[$this->plugin_id]['plugin_name']);
							}
							else
							{
								$move_plugin_name = $this->move_plugin_by($action, $arcade_plugins[$this->plugin_id]['plugin_order']);

								if ($move_plugin_name != false)
								{
									$reload_data = true;
									$arcade->add_log('admin', 'LOG_ARCADE_PLUGIN_' . strtoupper($action), $arcade_plugins[$this->plugin_id]['plugin_name'], $move_plugin_name);
								}
							}

							if ($reload_data)
							{
								$arcade_plugins = $this->plugin_data();
								$cache->destroy('_arcade_plugins');
							}
						}

						$action = false;
					break;

					case 'edit':
						$plugin_page		= '';
						$display_vars		= $this->plugin_check('acp', $arcade_plugins);
						$this->page_title	= 'ACP_ARCADE_PLUGIN_EDIT';
						$plugin_enabled		= (bool) request_var('plugin_enabled', $arcade_plugins[$this->plugin_id]['plugin_enabled']);
						$plugin_name		= utf8_normalize_nfc(request_var('plugin_langname', $arcade_plugins[$this->plugin_id]['plugin_name'], true));

						if ($arcade_plugins[$this->plugin_id]['plugin_use'] == ARCADE_PLUGIN_USE_ARCADE &&
							in_array($arcade_plugins[$this->plugin_id]['plugin_page'], array(ARCADE_INDEX_PAGE, ARCADE_ALL_PAGE)))
						{
							$plugin_page = request_var('plugin_page', $arcade_plugins[$this->plugin_id]['plugin_page']);
						}

						if (!in_array($arcade_plugins[$this->plugin_id]['plugin_position'], array(ARCADE_DISABLED, ARCADE_POSITION_STAT)))
						{
							$plugin_position = (int) request_var('plugin_position', $arcade_plugins[$this->plugin_id]['plugin_position']);
						}
						else
						{
							$plugin_position = ARCADE_DISABLED;
						}

						if ($submit)
						{
							if (!check_form_key($form_key))
							{
								$errors[] = $user->lang['FORM_INVALID'];
							}

							if (!$plugin_name)
							{
								$errors[] = $user->lang['ACP_ARCADE_PLUGIN_NO_LANGNAME'];
							}

							// Do not write values if there is an errors
							if (sizeof($errors))
							{
								$submit = false;
							}
						}

						if ($display_vars != false && sizeof($display_vars))
						{
							$display_vars['vars'] += array(
								'legend_' => 'ACP_SUBMIT_CHANGES'
							);

							if (isset($display_vars['lang']))
							{
								//$user->add_lang($display_vars['lang']);
								$arcade_plugin->add_lang($display_vars['lang']);
							}

							$this->new_config = $arcade_config;
							$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_config;

							// We validate the complete config if whished
							validate_config_vars($display_vars['vars'], $cfg_array, $errors);

							// Do not write values if there is an errors
							if (sizeof($errors))
							{
								$submit = false;
							}

							// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
							foreach ($display_vars['vars'] as $config_name => $value)
							{
								if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
								{
									continue;
								}

								$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

								if ($submit)
								{
									$arcade->set_config($config_name, $config_value);
								}
							}

							if (!$submit)
							{
								// Output relevant page
								foreach ($display_vars['vars'] as $config_key => $vars)
								{
									if (!is_array($vars) && strpos($config_key, 'legend') === false)
									{
										continue;
									}

									if (strpos($config_key, 'legend') !== false)
									{
										$template->assign_block_vars('options', array(
											'S_LEGEND'	=> true,
											'LEGEND'	=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars
										));

										continue;
									}

									$type = explode(':', $vars['type']);

									$l_explain = '';
									if ($vars['explain'] && isset($vars['lang_explain']))
									{
										$l_explain = (isset($user->lang[$vars['lang_explain']])) ? $user->lang[$vars['lang_explain']] : $vars['lang_explain'];
									}
									else if ($vars['explain'])
									{
										$l_explain = (isset($user->lang[$vars['lang'] . '_EXPLAIN'])) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '';
									}

									if ($vars['type'] == 'custom')
									{
										$call = false;

										if (isset($vars['method']))
										{
											$call = array($arcade_plugin->class_name($arcade_plugins[$this->plugin_id]['plugin_filename'], 'acp'), $vars['method']);
										}
										else if (isset($vars['function']))
										{
											$call = $vars['function'];
										}

										if ($call !== false)
										{
											if (isset($vars['params']))
											{
												$args = array();

												foreach ($vars['params'] as $value)
												{
													switch ($value)
													{
														case '{CONFIG_VALUE}':
															$value = $this->new_config[$config_key];
														break;

														case '{KEY}':
															$value = $key;
														break;
													}

													$args[] = $value;
												}
											}
											else
											{
												$args = array($this->new_config[$config_key], $config_key);
											}

											$content = call_user_func_array($call, $args);
										}
									}
									else
									{
										$content = build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);
									}

									if (empty($content))
									{
										continue;
									}

									$template->assign_block_vars('options', array(
										'KEY'			=> $config_key,
										'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
										'S_EXPLAIN'		=> $vars['explain'],
										'TITLE_EXPLAIN'	=> $l_explain,
										'CONTENT'		=> $content
									));

									unset($display_vars['vars'][$config_key]);
								}
							}
						}

						if ($submit)
						{
							$sql_array = array('plugin_name' => $plugin_name);

							if (!$this->plugin_update_check($arcade_plugins))
							{
								$sql_array['plugin_enabled'] = ($plugin_enabled) ? 1 : 0;
							}

							if ($plugin_page != '')
							{
								$sql_array['plugin_page'] = ($plugin_page == ARCADE_ALL_PAGE) ? ARCADE_ALL_PAGE : ARCADE_INDEX_PAGE;
							}

							if ($plugin_position != ARCADE_DISABLED)
							{
								$sql_array['plugin_position'] = (int) $plugin_position;
							}

							$sql = 'UPDATE ' . ARCADE_PLUGINS_TABLE . '
									SET ' . $db->sql_build_array('UPDATE', $sql_array) . '
									WHERE plugin_id = ' . $this->plugin_id;
							$db->sql_query($sql);

							$plugin_original_name = $arcade->lang_value($arcade_plugins[$this->plugin_id]['plugin_name']);
							$plugin_langname = $arcade->lang_value($plugin_name);
							$cache->destroy('_arcade_plugins');

							if ($plugin_original_name == $plugin_langname)
							{
								$arcade->add_log('admin', 'LOG_ARCADE_PLUGIN_EDIT', $plugin_name);
							}
							else
							{
								$arcade->add_log('admin', 'LOG_ARCADE_PLUGIN_NAME_EDIT', $arcade_plugins[$this->plugin_id]['plugin_name'], $plugin_name);
							}

							trigger_error(sprintf($user->lang['ACP_ARCADE_PLUGIN_EDIT_SUCCESS'], $plugin_langname) . adm_back_link($this->u_action . "&amp;action=edit&amp;p={$this->plugin_id}"));
						}

						$s_hidden_fields = array(
							'p'			=> $this->plugin_id,
							'action'	=> 'edit'
						);

						$template->assign_vars(array(
							'S_ARCADE_PLUGIN_EDIT'		=> true,
							'S_PLUGIN_ENABLED'			=> ($plugin_enabled) ? true : false,
							'S_PLUGIN_PAGE'				=> ($plugin_page != '') ? true : false,
							'S_PLUGIN_PAGE_INDEX'		=> ($plugin_page == ARCADE_INDEX_PAGE) ? true : false,
							'S_PLUGIN_POSITION'			=> ($plugin_position != ARCADE_DISABLED) ? true : false,
							'S_PLUGIN_POSITION_MIDDLE'	=> ($arcade_plugins[$this->plugin_id]['plugin_use'] == ARCADE_PLUGIN_USE_ARCADE) ? true : false,
							'S_HIDDEN_FIELDS'			=> build_hidden_fields($s_hidden_fields),

							'PLUGIN_POSITION_T_CHECKED'	=> ($plugin_position == ARCADE_POSITION_TOP) ? ' checked="checked"' : '',
							'PLUGIN_POSITION_M_CHECKED'	=> ($plugin_position == ARCADE_POSITION_MIDDLE) ? ' checked="checked"' : '',
							'PLUGIN_POSITION_B_CHECKED'	=> ($plugin_position == ARCADE_POSITION_BOTTOM) ? ' checked="checked"' : '',

							'PLUGIN_NAME'				=> $arcade->lang_value($plugin_name),
							'PLUGIN_EXPLAIN'			=> (!empty($user->lang[$plugin_name. '_EXPLAIN'])) ? $user->lang[$plugin_name. '_EXPLAIN'] : false,
							'PLUGIN_LANGNAME'			=> $plugin_name
						));
					break;

					case 'remove':
						if (!$auth->acl_get('a_arcade_plugin_remove'))
						{
							trigger_error($user->lang['NO_AUTH_OPERATION'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$plugin_install = $this->plugin_check('install', $arcade_plugins);

						if (confirm_box(true))
						{
							if (request_var('delete_files', false))
							{
								$arcade->delete_plugin_files($arcade_plugins[$this->plugin_id]['plugin_filename'], $plugin_install['plugin_files']['extra_files'], $errors);

								if (sizeof($errors))
								{
									trigger_error(implode('<br />', $errors) . adm_back_link($this->u_action), E_USER_WARNING);
								}
							}

							$uninstall = $plugin_install['plugin_install_db'];

							if (sizeof($uninstall))
							{
								if ($uninstall['custom_file'] !== false)
								{
									redirect(append_sid($phpbb_root_path . $install['custom_file'] . '.' . $phpEx));
								}

								if (sizeof($uninstall['schema_data']) || sizeof($uninstall['schema_changes']))
								{
									if (sizeof($uninstall['schema_data']))
									{
										$umil->table_remove(array_keys($uninstall['schema_data']));
									}

									if (sizeof($uninstall['schema_changes']))
									{
										$drop_data = $arcade_install->drop_columns_data($uninstall['schema_changes']);

										if (sizeof($drop_data))
										{
											$db_tools->perform_schema_changes($drop_data);
										}
									}
								}

								if (sizeof($uninstall['arcade_roles']['add']))
								{
									$umil->arcade_permission_role_remove($uninstall['arcade_roles']['add']);
								}

								if (sizeof($uninstall['arcade_permission']['add']))
								{
									$umil->arcade_permission_remove($uninstall['arcade_permission']['add']);
								}

								if (sizeof($uninstall['phpbb_roles']['add']))
								{
									$umil->permission_role_remove($uninstall['phpbb_roles']['add']);
								}

								if (sizeof($uninstall['phpbb_permission']['add']))
								{
									$umil->permission_remove($uninstall['phpbb_permission']['add']);
								}

								if (sizeof($uninstall['modules']['add']))
								{
									$umil->module_remove(array_reverse($uninstall['modules']['add']));
								}

								if (sizeof($uninstall['arcade_config']['add']))
								{
									$umil->arcade_config_remove($uninstall['arcade_config']['add']);
								}

								if (sizeof($uninstall['phpbb_config']['add']))
								{
									$umil->config_remove($uninstall['phpbb_config']['add']);
								}

								if (sizeof($uninstall['purge_cache']))
								{
									$umil->cache_purge($uninstall['purge_cache']);
								}
							}

							$class = $arcade_plugin->class_name($arcade_plugins[$this->plugin_id]['plugin_filename'], 'install');
							//$class::uninstall($arcade_plugins[$this->plugin_id]['plugin_version']);
							call_user_func_array(array($class, 'uninstall'), array($arcade_plugins[$this->plugin_id]['plugin_version']));

							$db->sql_query('DELETE FROM ' . ARCADE_PLUGINS_TABLE . ' WHERE plugin_id = ' . $this->plugin_id);

							$cache->destroy('_arcade_plugins');

							$arcade->add_log('admin', 'LOG_ARCADE_PLUGIN_REMOVED', $arcade_plugins[$this->plugin_id]['plugin_name']);
							trigger_error(sprintf($user->lang['ACP_ARCADE_PLUGIN_REMOVED_SUCCESS'], $arcade->lang_value($arcade_plugins[$this->plugin_id]['plugin_name'])) . adm_back_link($this->u_action));
						}
						else
						{
							$s_hidden_fields = array(
								'i'			=> $id,
								'mode'		=> $mode,
								'plugin_id'	=> $this->plugin_id,
								'action'	=> 'remove'
							);

							$arcade->confirm_box('delete_files', 'ACP_ARCADE_PLUGIN_REMOVE', $s_hidden_fields);
						}

						$action = false;
					break;

					case 'details':
						$plugin_info		= $this->plugin_check('info', $arcade_plugins);
						$plugin_install		= $this->plugin_check('install', $arcade_plugins);
						$this->page_title	= 'ARCADE_PLUGIN_DETAILS';
						$url				= $this->u_action . '&amp;p=' . $this->plugin_id;

						$plugin_version		= $arcade_plugins[$this->plugin_id]['plugin_version'];
						$plugin_install_xml	= $arcade_plugin->file_path($arcade_plugins[$this->plugin_id]['plugin_filename'], 'install_xml');

						$latest_version_info = $up_to_date = false;
						if (!empty($plugin_info['version_check']['domain']))
						{
							if (($latest_version_info = $this->obtain_latest_version_info($plugin_info['version_check']['domain'], $plugin_info['version_check']['path'], $plugin_info['version_check']['filename'], request_var('versioncheck_force', false))) !== false)
							{
								$latest_version_info = explode("\n", $latest_version_info);
								$up_to_date = phpbb_version_compare($plugin_version, trim($latest_version_info[0]), '<') ? false : true;
							}
							else
							{
								$template->assign_var('S_VERSIONCHECK_FAIL', true);
							}
						}

						if ($latest_version_info !== false)
						{
							$plugin_version = '<strong style="color:#'. (($up_to_date) ? '228822' : 'BC2A4D') .';">' . $plugin_version . '</strong>';
							$plugin_version = '<a href="' . $url . '&amp;action=version' . '" title="' . $user->lang['MORE_INFORMATION'] . '">' . $plugin_version . '</a>&nbsp;[&nbsp;<a href="' . $url . '&amp;action=details&amp;versioncheck_force=1' . '">' . $user->lang['VERSIONCHECK_FORCE_UPDATE'] . '</a>&nbsp;]';
						}

						$plugin_check = (($latest_version_info !== false) ? '&nbsp;-&nbsp;' : '&nbsp;') . '[&nbsp;<a href="' . $url . '&amp;action=verify' . '">' . $user->lang['ARCADE_PLUGIN_INSTALL_VERIFY'] . '</a>&nbsp;]';

						$template->assign_vars(array(
							'S_PLUGIN_DETAILS'				=> true,
							'S_PLUGIN_INSTALL_XML'			=> ($plugin_install_xml !== false) ? true : false,

							'U_PLUGIN_INSTALL_XML'			=> $plugin_install_xml,
							'U_VERSIONCHECK'				=> $url . '&amp;action=version',
							'U_VERSIONCHECK_FORCE'			=> $url . '&amp;action=details&amp;versioncheck_force=1',

							'ARCADE_PLUGIN_NAME'			=> $arcade->lang_value($arcade_plugins[$this->plugin_id]['plugin_name']),
							'ARCADE_PLUGIN_VERSION'			=> $plugin_version . $plugin_check,
							'ARCADE_PLUGIN_INSTALL_TIME'	=> $user->format_date($arcade_plugins[$this->plugin_id]['plugin_date']),

							'ARCADE_PLUGIN_SUPPORT'			=> $plugin_info['plugin_support'],
							'ARCADE_PLUGIN_DOWNLOAD'		=> $plugin_info['plugin_download'],

							'ARCADE_PLUGIN_AUTHOR_NAME'		=> $plugin_info['author_name'],
							'ARCADE_PLUGIN_AUTHOR_EMAIL'	=> $plugin_info['author_email'],
							'ARCADE_PLUGIN_AUTHOR_URL'		=> $plugin_info['author_url'],
							'ARCADE_PLUGIN_AUTHOR_NOTE'		=> $plugin_info['author_note'],
							'ARCADE_PLUGIN_COPYRIGHT'		=> $plugin_info['plugin_copyright']
						));
					break;

					case 'version':
						$plugin_info		= $this->plugin_check('info', $arcade_plugins);
						$plugin_name		= $arcade->lang_value($arcade_plugins[$this->plugin_id]['plugin_name']);
						$this->page_title	= sprintf($user->lang['ARCADE_PLUGIN_VERSION_CHECK'], $plugin_name);
						$l_title_explain	= sprintf($user->lang['ARCADE_PLUGIN_VERSION_CHECK_EXPLAIN'], $plugin_name);

						if (empty($plugin_info['version_check']['domain']))
						{
							trigger_error(sprintf($user->lang['ACP_ARCADE_PLUGIN_NO_VERSION_CHECK'], $plugin_name) . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$info = $this->obtain_latest_version_info($plugin_info['version_check']['domain'], $plugin_info['version_check']['path'], $plugin_info['version_check']['filename'], request_var('versioncheck_force', false), true);

						if ($info === false)
						{
							trigger_error($user->lang['VERSIONCHECK_FAIL'] . adm_back_link($this->u_action), E_USER_WARNING);
						}

						$version		= $arcade_plugins[$this->plugin_id]['plugin_version'];
						$info			= explode("\n", $info);
						$latest_version	= trim($info[0]);
						$up_to_date		= (phpbb_version_compare($version, $latest_version, '<')) ? false : true;

						$template->assign_vars(array(
							'S_PLUGIN_VERSION'		=> true,
							'S_UP_TO_DATE'			=> $up_to_date,

							'U_VERSIONCHECK_FORCE'	=> $this->u_action . '&amp;p=' . $this->plugin_id . '&amp;action=version&amp;versioncheck_force=1',

							'LATEST_VERSION'		=> '<strong style="color:#228822;">' . $latest_version . '</strong>',
							'CURRENT_VERSION'		=> '<strong style="color:#'. (($up_to_date) ? '228822' : 'BC2A4D') .';">' . $version . '</strong>'
						));
					break;

					case 'verify':
						$plugin_install		= $this->plugin_check('install', $arcade_plugins);
						$plugin_name		= $arcade->lang_value($arcade_plugins[$this->plugin_id]['plugin_name']);
						$this->page_title	= sprintf($user->lang['ARCADE_PLUGIN_INSTALL_CHECK'], $plugin_name);
						$l_title_explain	= sprintf($user->lang['ARCADE_PLUGIN_INSTALL_CHECK_EXPLAIN'], $plugin_name);

						$s_hidden_fields = build_hidden_fields(array(
							'action'	=> 'verify',
							'p'			=> $this->plugin_id
						));

						if (sizeof(array_keys($plugin_install['plugin_requirements']['bbcodes'])))
						{
							$user->add_lang('acp/posting');
						}

						$passed = $arcade_install->requirements('verify', 'verify', $plugin_install['plugin_requirements'], $this->u_action, $s_hidden_fields);

						$template->assign_block_vars('checks', array(
							'S_LEGEND'			=> true,
							'LEGEND'			=> $user->lang['ARCADE_INSTALL_CHECK'],
							'LEGEND_EXPLAIN'	=> false
						));

						$error = $arcade->plugin_cool_files($arcade_plugins[$this->plugin_id]['plugin_filename'], $plugin_install['plugin_files']['extra_files'], true, true, $plugin_install['plugin_files']['use_default_path']);

						if (sizeof($error))
						{
							$passed['mod'] = false;
							$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_MISSING_FILES'], $arcade_install->color_text($arcade->root_key(implode('<br />', $error)), 'black')), 'red');
						}
						else
						{
							$result = $arcade_install->color_text($user->lang['VERIFY_ALL_FILES']);
						}

						$data = array('VERIFY_FILES_EXIST' => $result);

						if (!empty($plugin_install['plugin_files']['file_edits']))
						{
							$error = array();
							$result = $arcade_install->file_edit_check($plugin_install['plugin_files']['file_edits'], $error);

							if (sizeof($error))
							{
								$passed['mod'] = false;
							}

							$data = array_merge($data, array('VERIFY_FILES_EDITED' => $result));
						}

						$db_verify = $plugin_install['plugin_install_db'];

						if (sizeof($db_verify))
						{
							if (!empty($db_verify['schema_data']))
							{
								$error = array();
								$result = $arcade_install->verify_tables($db_verify['schema_data'], $error);

								if (sizeof($error))
								{
									$passed['mod'] = false;
								}

								$data = array_merge($data, array('VERIFY_TABLES_EXIST' => $result));
							}

							if (!empty($db_verify['modules']['add']))
							{
								$error = array();

								foreach ($plugin_install['plugin_verify_db']['modules'] as $module_data)
								{
									$module_class		= array_shift($module_data);
									$module_parent		= array_shift($module_data);
									$module_langname	= array_shift($module_data);

									if (!$umil->module_exists($module_class, $module_parent, $module_langname))
									{
										$error[] = (($module_parent) ? $arcade->lang_value($module_parent) . ' / ' : '') . $arcade->lang_value($module_langname);
									}
								}

								if (sizeof($error))
								{
									$passed['mod'] = false;
									$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_MISSING_MODULES'], $arcade_install->color_text(implode('<br />', $error), 'black')), 'red');
								}
								else
								{
									$result = $arcade_install->color_text($user->lang['VERIFY_ALL_MODULES']);
								}

								$data = array_merge($data, array('VERIFY_MODULES' => $result));
							}

							if (!empty($db_verify['arcade_permission']['add']))
							{
								$error = array();
								$result = $arcade_install->verify_permission($db_verify['arcade_permission']['add'], $error);

								if (sizeof($error))
								{
									$passed['mod'] = false;
								}

								$data = array_merge($data, array('VERIFY_PERMISSIONS_ARCADE' => $result));
							}

							if (!empty($db_verify['phpbb_permission']['add']))
							{
								$error = array();
								$result = $arcade_install->verify_permission($db_verify['phpbb_permission']['add'], $error, 'phpbb');

								if (sizeof($error))
								{
									$passed['mod'] = false;
								}

								$data = array_merge($data, array('VERIFY_PERMISSIONS_PHPBB' => $result));
							}

							if (!empty($db_verify['arcade_config']['add']))
							{
								$error = array();
								$result = $arcade_install->verify_config($db_verify['arcade_config']['add'], $error);

								if (sizeof($error))
								{
									$passed['mod'] = false;
								}

								$data = array_merge($data, array('VERIFY_ARCADE_CONFIGS' => $result));
							}

							if (!empty($db_verify['phpbb_config']['add']))
							{
								$error = array();
								$result = $arcade_install->verify_config($db_verify['phpbb_config']['add'], $error, 'phpbb');

								if (sizeof($error))
								{
									$passed['mod'] = false;
								}

								$data = array_merge($data, array('VERIFY_PHPBB_CONFIGS' => $result));
							}

							if (!empty($plugin_install['plugin_verify_db']['alter']['current']))
							{
								$results = array();
								$arcade_install->verify_alter_table('current', $plugin_install['plugin_verify_db']['alter']['current'], $results);

								$data = array_merge($data, array('VERIFY_OTHER_DB_DATA' => implode('<br />', $results)));
							}
						}

						if (!empty($plugin_install['plugin_verify_db']['old_plugin_files']))
						{
							$error = $arcade->plugin_cool_files(false, $plugin_install['plugin_verify_db']['old_plugin_files'], false, false);

							if (sizeof($error))
							{
								$passed['mod'] = false;
								$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_OLD_FILES_PRESENT'], $arcade_install->color_text($arcade->root_key(implode('<br />', $error)), 'black')), 'red');
								$data = array_merge($data, array('VERIFY_OLD_FILES' => $result));
							}
						}

						if (!empty($plugin_install['plugin_verify_db']['alter']['old']))
						{
							$results = array();
							$arcade_install->verify_alter_table('old', $plugin_install['plugin_verify_db']['alter']['old'], $results);
							$data = array_merge($data, array('VERIFY_OLD_OTHER_DB_DATA' => implode('<br />', $results)));
						}

						$plugin_install_xml = $arcade_plugin->file_path($arcade_plugins[$this->plugin_id]['plugin_filename'], 'install_xml');

						if (in_array(false, $passed))
						{
							if (!empty($db_verify['purge_cache']))
							{
								$umil->cache_purge($db_verify['purge_cache']);
							}
							else
							{
								$cache->destroy('config');
								$cache->destroy('_arcade');
							}

							if ($plugin_install_xml !== false)
							{
								$template->assign_block_vars('checks', array(
									'TITLE'		=> $user->lang['ARCADE_VIEW_INSTALL_FILE'],
									'RESULT'	=> '<a onclick="window.open(this.href); return false;" href="' . $plugin_install_xml . '">' . $user->lang['ARCADE_VIEW_INSTALL_FILE'] . '</a>',

									'S_EXPLAIN'	=> false,
									'S_LEGEND'	=> false,
								));
							}
						}

						foreach ($data as $title => $result)
						{
							$template->assign_block_vars('checks', array(
								'TITLE'		=> $user->lang[$title],
								'RESULT'	=> $result,

								'S_EXPLAIN'	=> false,
								'S_LEGEND'	=> false,
							));
						}

						$submit = (in_array(false, $passed)) ? $user->lang['INSTALL_TEST'] : false;

						if (in_array(false, $passed))
						{
							$s_hidden_fields = str_replace(array('install', 'update'), 'requirements', $s_hidden_fields);
						}

						$template->assign_vars(array(
							'S_HIDDEN'				=> $s_hidden_fields,
							'U_ACTION'				=> $this->u_action,
							'L_SUBMIT'				=> $submit
						));
					break;

					case 'requirements':
						$this->page_title = 'REQUIREMENTS_UPDATE_TITLE';
					case 'update':
						$plugin_update = $this->plugin_check('install', $arcade_plugins);
						$plugin_name = $arcade->lang_value($plugin_update['plugin_name']);
						$installed_version = $arcade_plugins[$this->plugin_id]['plugin_version'];

						if ($action == 'requirements')
						{
							$req = array_merge($plugin_update['plugin_requirements'], array(
								'plugin_name'				=> $plugin_name,
								'plugin_install_version'	=> $installed_version
							));

							$s_hidden_fields = build_hidden_fields(array(
								'action'	=> 'update',
								'p'			=> $this->plugin_id
							));

							$arcade_install->requirements('update', 'update', $req, $this->u_action, $s_hidden_fields);
							break;
						}

						if ($plugin_update['plugin_install_db']['custom_file'] !== false)
						{
							redirect(append_sid($phpbb_root_path . $plugin_update['plugin_install_db']['custom_file'] . '.' . $phpEx));
						}

						$this->page_title = $user->lang['ARCADE_PLUGIN_UPDATE'] . ' :: ' . $plugin_name;

						$error_ary = array();
						$errored = false;
						$no_updates = true;

						$db_msg = '<h1>' . $user->lang['UPDATING_TO_LATEST_STABLE'] . '</h1>';
						$db_msg .= $arcade_install->def_db($installed_version, $plugin_update['plugin_version']);
						$versions = array_keys($plugin_update['plugin_update_db']['versions']);

						for ($i = 0; $i < sizeof($versions); $i++)
						{
							$version = $versions[$i];
							$next_version = (isset($versions[$i + 1])) ? $versions[$i + 1] : $plugin_update['plugin_version'];

							if (phpbb_version_compare($version, $installed_version, '<') || phpbb_version_compare($installed_version, $next_version, '>='))
							{
								continue;
							}

							$this->validate_install_file($plugin_update, $version);

							$update_version = $plugin_update['plugin_update_db']['versions'][$version];

							if (sizeof($update_version['schema_data']) || sizeof($update_version['schema_changes']))
							{
								if (sizeof($update_version['schema_data']))
								{
									foreach ($update_version['schema_data'] as $table_name => $table_data)
									{
										$statements = $db_tools->sql_create_table($table_name, $table_data);

										foreach ($statements as $sql)
										{
											$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
										}
									}
								}

								if (sizeof($update_version['schema_changes']))
								{
									$statements = $db_tools->perform_schema_changes($update_version['schema_changes']);

									foreach ($statements as $sql)
									{
										$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
									}
								}

								$no_updates = false;
							}
						}

						$db_msg .= $arcade_install->_write_result($no_updates, $errored, $error_ary);

						$error_ary = array();
						$errored = false;
						$no_updates = true;

						$db_msg .= '<h1><br />' . $user->lang['UPDATING_DATA'] . '</h1>';
						$db_msg .= '<p><br />' . $user->lang['PROGRESS'] . ' :: <br /><strong>';

						for ($i = 0; $i < sizeof($versions); $i++)
						{
							$version = $versions[$i];
							$next_version = (isset($versions[$i + 1])) ? $versions[$i + 1] : $plugin_update['plugin_version'];

							if (phpbb_version_compare($version, $installed_version, '<') || phpbb_version_compare($installed_version, $next_version, '>='))
							{
								continue;
							}

							$update_version = $plugin_update['plugin_update_db']['versions'][$version];

							if (sizeof($update_version['arcade_config']['remove']))
							{
								$umil->arcade_config_remove($update_version['arcade_config']['remove']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_config']['add']))
							{
								$umil->arcade_config_add($update_version['arcade_config']['add']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_config']['update']))
							{
								$umil->arcade_config_update($update_version['arcade_config']['update']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_config']['remove']))
							{
								$umil->config_remove($update_version['phpbb_config']['remove']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_config']['add']))
							{
								$umil->config_add($update_version['phpbb_config']['add']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_config']['update']))
							{
								$umil->config_update($update_version['phpbb_config']['update']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_permission']['remove']))
							{
								$umil->arcade_permission_remove($update_version['arcade_permission']['remove']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_permission']['add']))
							{
								$umil->arcade_permission_add($update_version['arcade_permission']['add']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_permission']['remove']))
							{
								$umil->permission_remove($update_version['phpbb_permission']['remove']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_permission']['add']))
							{
								$umil->permission_add($update_version['phpbb_permission']['add']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_roles']['remove']))
							{
								$umil->arcade_permission_role_remove($update_version['arcade_roles']['remove']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_roles']['add']))
							{
								$umil->arcade_permission_role_add($update_version['arcade_roles']['add']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_roles']['remove']))
							{
								$umil->permission_role_remove($update_version['phpbb_roles']['remove']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_roles']['add']))
							{
								$umil->permission_role_add($update_version['phpbb_roles']['add']);
								$no_updates = false;
							}

							if (sizeof($update_version['arcade_roles_set']))
							{
								$umil->arcade_permission_set($update_version['arcade_roles_set']);
								$no_updates = false;
							}

							if (sizeof($update_version['phpbb_roles_set']))
							{
								$umil->permission_set($update_version['phpbb_roles_set']);
								$no_updates = false;
							}

							if (sizeof($update_version['modules']['remove']))
							{
								$umil->module_remove(array_reverse($update_version['modules']['remove']));
								$no_updates = false;
							}

							if (sizeof($update_version['modules']['add']))
							{
								$umil->module_add($update_version['modules']['add']);
								$no_updates = false;
							}
						}

						$db_msg .= $arcade_install->_write_result($no_updates, $errored, $error_ary);

						$class = $arcade_plugin->class_name($arcade_plugins[$this->plugin_id]['plugin_filename'], 'install');
						//$class::update($installed_version);
						call_user_func_array(array($class, 'update'), array($installed_version));

						$plugin_data = array(
							'plugin_enabled'	=> 1,
							'plugin_name'		=> $plugin_update['plugin_name'],
							'plugin_use'		=> $plugin_update['plugin_use'],
							'plugin_page'		=> $plugin_update['plugin_page'],
							'plugin_position'	=> $plugin_update['plugin_position'],
							'plugin_version'	=> $plugin_update['plugin_version']
						);

						$error_ary = array();
						$errored = false;
						$no_updates = false;

						$db_msg .= '<h1><br />' . $user->lang['UPDATE_VERSION_OPTIMIZE'] . '</h1>';
						$db_msg .= '<p><br />' . $user->lang['PROGRESS'] . ' :: <br /><strong>';

						// update the plugin data
						$sql = 'UPDATE ' . ARCADE_PLUGINS_TABLE . '
								SET ' . $db->sql_build_array('UPDATE', $plugin_data) . '
								WHERE plugin_id = ' . $this->plugin_id;
						$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);

						if (sizeof($plugin_update['plugin_install_db']['schema_data']))
						{
							$arcade_install->table_optimize(array_keys($plugin_update['plugin_install_db']['schema_data']), $db_msg, $errored, $error_ary);
						}

						$db_msg .= $arcade_install->_write_result($no_updates, $errored, $error_ary);

						$db_msg .= '<h1><br />' . $user->lang['UPDATE_COMPLETED'] . '</h1>';

						if (sizeof($plugin_update['plugin_install_db']['purge_cache']))
						{
							$umil->cache_purge($plugin_update['plugin_install_db']['purge_cache']);
						}

						$cache->destroy('_arcade_plugins');
						$arcade->add_log('admin', 'LOG_ARCADE_PLUGIN_UPDATED', $plugin_update['plugin_name'], $installed_version, $plugin_update['plugin_version']);

						$message = sprintf($user->lang['ACP_ARCADE_PLUGIN_UPDATED_SUCCESS'], $plugin_name);

						$s_hidden_fields = build_hidden_fields(array(
							'action'	=> 'verify',
							'p'			=> $this->plugin_id
						));

						$template->assign_vars(array(
							'S_ARCADE_PLUGIN_UPDATE'	=> true,
							'S_HIDDEN_FIELDS'			=> $s_hidden_fields,

							'ARCADE_PLUGIN_UPDATE'		=> $db_msg
						));
					break;
				}

				if (!$action)
				{
					// installed plugins
					foreach ($arcade_plugins as $plugin_id => $row)
					{
						$details_file = $remove_enable = true;
						if (!$this->plugin_ini('info', $row['plugin_filename'], true))
						{
							$details_file = false;
							$errors [] = sprintf($user->lang['ARCADE_FILE_NOT_FOUND'], $arcade->root_key($arcade_plugin->file_path($row['plugin_filename'], 'info')));
						}

						$plugin_install = $this->plugin_ini('install', $row['plugin_filename'], false, true);

						if (!is_array($plugin_install))
						{
							$details_file = false;

							if ($plugin_install == 'file_not_found')
							{
								$remove_enable = false;
							}

							$msg = 'ARCADE_FILE_' . (($plugin_install == 'file_not_found') ? 'NOT_FOUND' : 'WRONG');

							$errors [] = sprintf($user->lang[$msg], $arcade->root_key($arcade_plugin->file_path($row['plugin_filename'], 'install')));
						}

						$url = $this->u_action . '&amp;p=' . $plugin_id;
						$plugin_update = $this->plugin_update_check($plugin_install, $row['plugin_version']);

						if ($plugin_update && $row['plugin_enabled'])
						{
							$this->plugin_switching('disable', $plugin_id);
							$row['plugin_enabled'] = 0;
							$cache->destroy('_arcade_plugins');
						}

						$template->assign_block_vars('plugins', array(
							'S_PLUGIN_UPDATE'	=> $plugin_update,
							'S_PLUGIN_REMOVE'	=> $remove_enable,

							'U_MOVE_UP'			=> $url . '&amp;action=move_up',
							'U_MOVE_DOWN'		=> $url . '&amp;action=move_down',
							'U_EDIT'			=> $url . '&amp;action=edit',
							'U_REMOVE'			=> $url . '&amp;action=remove',
							'U_ENABLE'			=> $url . '&amp;action=enable',
							'U_DISABLE'			=> $url . '&amp;action=disable',
							'U_UPDATE'			=> $url . '&amp;action=requirements',

							'IMAGE'		=> ($row['plugin_enabled']) ? '<img src="images/icon_folder.gif" alt="" />' : '<img src="images/icon_folder_lock.gif" alt="" />',
							'NAME'		=> $arcade->lang_value($row['plugin_name']),
							'DETAILS'	=> ($details_file) ? '<a href="' . $url . '&amp;action=details">' . $user->lang['ARCADE_PLUGIN_DETAILS'] . '</a>' : false,
							'ENABLED'	=> ($row['plugin_enabled']) ? true : false
						));
					}

					$template->assign_vars(array(
						'S_PLUGIN_REMOVE'		=> ($auth->acl_get('a_arcade_plugin_remove')) ? true : false,

						'ICON_REMOVE'			=> '<img src="' . $phpbb_admin_path . 'images/icon_delete.gif" alt="' . $user->lang['ARCADE_PLUGIN_REMOVED'] . '" title="' . $user->lang['ARCADE_PLUGIN_REMOVED'] . '" />',
						'ICON_REMOVE_DISABLED'	=> '<img src="' . $phpbb_admin_path . 'images/icon_delete_disabled.gif" alt="' . $user->lang['ARCADE_PLUGIN_REMOVED'] . '" title="' . $user->lang['ARCADE_PLUGIN_REMOVED'] . '" />',
						'ICON_UPDATE'			=> '<img src="' . $phpbb_admin_path . 'images/icon_sync.gif" alt="' . $user->lang['ARCADE_PLUGIN_UPDATE'] . '" title="' . $user->lang['ARCADE_PLUGIN_UPDATE'] . '" />',
						'ICON_UPDATE_DISABLED'	=> '<img src="' . $phpbb_admin_path . 'images/icon_sync_disabled.gif" alt="' . $user->lang['ARCADE_PLUGIN_UPDATE'] . '" title="' . $user->lang['ARCADE_PLUGIN_UPDATE'] . '" />'
					));
				}
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}

		if (sizeof($errors))
		{
			$template->assign_vars(array(
				'S_ERROR'	=> true,
				'ERROR_MSG'	=> implode('<br />', $errors)
			));
		}

		if (empty($l_title_explain))
		{
			$l_title_explain = (!empty($user->lang[$this->page_title . '_EXPLAIN'])) ? $user->lang[$this->page_title . '_EXPLAIN'] : false;
		}
		else
		{
			$l_title_explain = (!empty($user->lang[$l_title_explain])) ? $user->lang[$l_title_explain] : $l_title_explain;
		}

		$template->assign_vars(array(
			'L_TITLE'			=> (!empty($user->lang[$this->page_title])) ? $user->lang[$this->page_title] : $this->page_title,
			'L_TITLE_EXPLAIN'	=> $l_title_explain,

			'U_ACTION'			=> $this->u_action
		));
	}

	function plugin_ini($type, $filename, $file_ex = false, $not_found = false)
	{
		global $user, $arcade_plugin;

		$class = $arcade_plugin->class_name($filename, $type);

		$plugin_file = $arcade_plugin->file_path($filename, $type);

		if (!file_exists($plugin_file))
		{
			if ($not_found)
			{
				return 'file_not_found';
			}

			return false;
		}

		if ($file_ex)
		{
			return true;
		}

		if (!class_exists($class))
		{
			include($plugin_file);
		}

		if (!class_exists($class))
		{
			if ($not_found)
			{
				return 'class_not_found';
			}

			return false;
		}

		$return = false;

		switch ($type)
		{
			case 'install':
				//$install = $class::arcade_plugin_install();
				$install = call_user_func(array($class, 'arcade_plugin_install'));

				if ($install != false && sizeof($install))
				{
					if (empty($install['plugin_name']) || empty($install['plugin_version']))
					{
						break;
					}

					$return = $install;
				}
			break;

			case 'acp':
				//$config_tpl = $class::arcade_plugin_config_tpl();
				$config_tpl = call_user_func(array($class, 'arcade_plugin_config_tpl'));

				if ($config_tpl != false && sizeof($config_tpl))
				{
					$return = $config_tpl;
				}
			break;

			case 'info':
				//$info = $class::arcade_plugin_info();
				$info = call_user_func(array($class, 'arcade_plugin_info'));

				if ($info != false && sizeof($info))
				{
					$return = $info;
				}
			break;
		}

		if (!is_array($return))
		{
			return false;
		}

		return $return;
	}

	function plugin_data($p_id = false)
	{
		global $db, $user;

		$where_sql = '';

		if ($p_id != false)
		{
			if (!$this->plugin_id)
			{
				trigger_error($user->lang['ACP_ARCADE_PLUGIN_NO_ID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}

			$where_sql = ' WHERE plugin_id = ' . $this->plugin_id . ' ';
		}

		$sql = 'SELECT *
				FROM ' . ARCADE_PLUGINS_TABLE . "
				$where_sql
				ORDER BY plugin_order ASC";
		$result = $db->sql_query($sql);
		$plugins_data = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$plugins_data[$row['plugin_id']] = array(
				'plugin_enabled'	=> $row['plugin_enabled'],
				'plugin_name'		=> $row['plugin_name'],
				'plugin_filename'	=> $row['plugin_filename'],
				'plugin_use'		=> $row['plugin_use'],
				'plugin_page'		=> $row['plugin_page'],
				'plugin_position'	=> $row['plugin_position'],
				'plugin_version'	=> $row['plugin_version'],
				'plugin_order'		=> $row['plugin_order'],
				'plugin_date'		=> $row['plugin_date']
			);
		}
		$db->sql_freeresult($result);

		if ($where_sql && !sizeof($plugins_data))
		{
			trigger_error($user->lang['ARCADE_NO_PLUGIN'] . adm_back_link($this->u_action), E_USER_WARNING);
		}

		return $plugins_data;
	}

	function move_plugin_by($action, $plugin_order)
	{
		global $db;

		switch($action)
		{
			case 'move_up':
				$move = -15;
				$move_order = ($plugin_order - 10);
			break;

			case 'move_down':
				$move = 15;
				$move_order = ($plugin_order + 10);
			break;
		}

		$sql = 'SELECT plugin_name FROM ' . ARCADE_PLUGINS_TABLE . '
				WHERE plugin_order = ' . (int) $move_order;
		$result = $db->sql_query($sql);
		$move_plugin_name = $db->sql_fetchfield('plugin_name');
		$db->sql_freeresult($result);

		if (!$move_plugin_name)
		{
			return false;
		}

		$sql = 'UPDATE ' . ARCADE_PLUGINS_TABLE . '
				SET plugin_order = plugin_order + ' . $move . '
				WHERE plugin_id = ' . $this->plugin_id;
		$db->sql_query($sql);

		$sql = 'SELECT plugin_id, plugin_order FROM ' . ARCADE_PLUGINS_TABLE . '
				ORDER BY plugin_order ASC';
		$result = $db->sql_query($sql);

		$i = 10;
		while ($row = $db->sql_fetchrow($result))
		{
			$sql = 'UPDATE ' . ARCADE_PLUGINS_TABLE . '
					SET plugin_order = ' . $i . '
					WHERE plugin_id = ' . (int) $row['plugin_id'];
			$db->sql_query($sql);
			$i += 10;
		}
		$db->sql_freeresult($result);

		return $move_plugin_name;
	}

	function plugin_check($type, $arcade_plugins)
	{
		global $user, $phpbb_root_path;
		global $arcade, $arcade_plugin;

		if (!$this->plugin_id)
		{
			trigger_error($user->lang['ACP_ARCADE_PLUGIN_NO_ID'] . adm_back_link($this->u_action), E_USER_WARNING);
		}
		else if (empty($arcade_plugins[$this->plugin_id]))
		{
			trigger_error($user->lang['ACP_ARCADE_PLUGIN_ID_NOT_EXIST'] . adm_back_link($this->u_action), E_USER_WARNING);
		}

		$acp_file = false;
		if ($type == 'acp' && !file_exists($arcade_plugin->file_path($arcade_plugins[$this->plugin_id]['plugin_filename'], $type)))
		{
			$plugin_install = $this->plugin_check('install', $arcade_plugins);
			$acp_file = (!empty($plugin_install['plugin_files']['use_default_path']['acp'])) ? true : false;
		}

		$not_found = (($type != 'acp') || ($type == 'acp' && $acp_file)) ? true : false;

		$plugin_data = $this->plugin_ini($type, $arcade_plugins[$this->plugin_id]['plugin_filename'], false, $not_found);

		if ($not_found)
		{
			if ($plugin_data == 'file_not_found')
			{
				if ($type == 'info')
				{
					$plugin_data = $this->plugin_ini('install', $arcade_plugins[$this->plugin_id]['plugin_filename'], false, true);

					if ($plugin_data == 'file_not_found')
					{
						$type = 'install';
					}
					else if (!$plugin_data || !is_array($plugin_data))
					{
						$error = 'ACP_ARCADE_PLUGIN_READ_INSTALL_ERROR';
					}
					else if (!$plugin_data['plugin_files']['use_default_path'][$type])
					{
						trigger_error(sprintf($user->lang['ARCADE_PLUGIN_NOT_FUNCTION'], $arcade->lang_value($arcade_plugins[$this->plugin_id]['plugin_name'])) . adm_back_link($this->u_action), E_USER_WARNING);
					}
				}

				if (empty($error))
				{
					$error = 'ARCADE_FILE_NOT_FOUND';
				}
			}
			else if (!$plugin_data || (!is_array($plugin_data) && $plugin_data != 'class_not_found'))
			{
				$error = 'ACP_ARCADE_PLUGIN_READ_' . strtoupper($type) . '_ERROR';
			}
			else
			{
				$error = false;
			}

			if ($error || $plugin_data == 'class_not_found')
			{
				$filename = $arcade->root_key($arcade_plugin->file_path($arcade_plugins[$this->plugin_id]['plugin_filename'], $type));
				$message = ($error) ? sprintf($user->lang[$error], $filename) : sprintf($user->lang['ARCADE_PLUGIN_CLASS_NOT_FOUND'], $arcade_plugin->class_name($arcade_plugins[$this->plugin_id]['plugin_filename'], $type), $filename);

				trigger_error($message . adm_back_link($this->u_action), E_USER_WARNING);
			}

			if ($type != 'acp')
			{
				$func = 'validate_' . $type . '_file';
				$this->$func($plugin_data);
			}
		}

		return $plugin_data;
	}

	function validate_install_file(&$install_data, $version = false)
	{
		$isc = array(
			'schema_data'		=> array(),
			'schema_changes'	=> array(),
			'arcade_config'		=> array(
				'add'			=> array(),
				'update'		=> array(),
				'remove'		=> array()
			),
			'phpbb_config'		=> array(
				'add'			=> array(),
				'update'		=> array(),
				'remove'		=> array()
			),
			'arcade_permission'	=> array(
				'add'			=> array(),
				'remove'		=> array()
			),
			'phpbb_permission'	=> array(
				'add'			=> array(),
				'remove'		=> array()
			),
			'arcade_roles'		=> array(
				'add'			=> array(),
				'remove'		=> array()
			),
			'phpbb_roles'		=> array(
				'add'			=> array(),
				'remove'		=> array()
			),
			'arcade_roles_set'	=> array(),
			'phpbb_roles_set'	=> array(),
			'modules'			=> array(
				'add'			=> array(),
				'remove'		=> array()
			),
			'purge_cache'		=> array()
		);

		$default_install_data = array(
			//'plugin_name'				=> '',
			//'plugin_version'			=> '',
			'plugin_requirements'		=> array(
				'phpbb_version'			=> '3.0.11',
				'arcade_version'		=> '2.0.RC4',
				'plugin_version'		=> '',
				'writable_directories'	=> array(),
				'bbcodes'				=> array()
			),
			'plugin_use'			=> ARCADE_PLUGIN_USE_BOARD,
			'plugin_page'			=> ARCADE_INDEX_PAGE,
			'plugin_position'		=> ARCADE_DISABLED,

			'plugin_files'			=> array(
				'use_default_path'	=> array(
					'acp'			=> false,
					'language'		=> false,
					'template'		=> false
				),
				'extra_files'		=> array(),
				'file_edits'		=> array()
			),

			'plugin_install_db'		=> array_merge(array(
				'custom_file'		=> false
				), $isc
			),

			'plugin_update_db'		=> array(
				'versions'			=> array()
			),

			'plugin_verify_db'		=> array(
				'modules'			=> array(),
				'old_plugin_files'	=> array(),
				'alter'				=> array(
					'current'		=> array(),
					'old'			=> array()
				)
			)
		);

		if ($version)
		{
			$default_install_data['plugin_update_db']['versions'][$version] = $isc;
		}

		$this->validate_install_data(array_keys($default_install_data), $default_install_data, $install_data);
	}

	function validate_install_data($keys, $default_install_data, &$install_data, $type = false)
	{
		foreach ($keys as $key)
		{
			if (empty($install_data[$key]))
			{
				$install_data[$key] = $default_install_data[$key];
			}

			if ($type)
			{
				$this->validate_config($install_data[$key]);
			}

			if (is_array($default_install_data[$key]))
			{
				$vc = (in_array($key, array('arcade_config', 'phpbb_config'))) ? 'config' : false;
				$this->validate_install_data(array_keys($default_install_data[$key]), $default_install_data[$key], $install_data[$key], $vc);
			}
		}
	}

	function validate_config(&$data)
	{
		if (sizeof($data))
		{
			foreach ($data as $key => $config_data)
			{
				if (!is_array($config_data))
				{
					unset($data[$key]);
					continue;
				}

				$config_name = array_shift($config_data);

				if (strpos($config_name, 'plugin_') === false)
				{
					unset($data[$key]);
					continue;
				}

				$config_value	= array_shift($config_data);
				$config_data	= (is_array($config_data)) ? array_shift($config_data) : $config_data;
				$is_dynamic		= (!empty($config_data)) ? 1 : 0;
				$data[$key]		= array($config_name, $config_value, $is_dynamic);
			}
		}
	}

	function validate_info_file(&$info_data)
	{
		global $user;

		$default_info_data = array(
			'author_name'		=> $user->lang['ARCADE_UNKNOWN'],
			'author_profile'	=> '',
			'author_email'		=> '',
			'author_url'		=> '',
			'author_note'		=> $user->lang['ARCADE_PLUGIN_AUTHOR_NO_NOTE'],
			'plugin_support'	=> '',
			'plugin_download'	=> '',
			'plugin_copyright'	=> '',

			'version_check'		=> array(
				'domain'		=> '',
				'path'			=> '',
				'filename'		=> ''
			)
		);

		foreach (array_keys($default_info_data) as $dk1)
		{
			$replace = false;

			if (empty($info_data[$dk1]))
			{
				$info_data[$dk1] = $default_info_data[$dk1];
			}
			else if ($dk1 == 'author_name' && !empty($info_data['author_profile']))
			{
				$replace = array($info_data['author_profile'], $info_data[$dk1]);
			}
			else if ($dk1 == 'author_email')
			{
				$replace = array('mailto:' . $info_data[$dk1], $info_data[$dk1]);
			}
			else if ($dk1 == 'author_url')
			{
				$replace = $info_data[$dk1];
			}
			else if ($dk1 == 'plugin_support')
			{
				$replace = array($info_data[$dk1], sprintf($user->lang['ARCADE_SUPPORT'], $user->lang['ARCADE_PLUGIN']));
			}
			else if ($dk1 == 'plugin_download')
			{
				$replace = array($info_data[$dk1], $user->lang['ARCADE_PLUGIN'] . ' ' . $user->lang['ARCADE_DOWNLOAD']);
			}

			if ($replace)
			{
				$info_data[$dk1] = str_replace(array('{URL}', '{MSG}'), $replace, '<a href="{URL}" onclick="window.open(this.href); return false;">{MSG}</a>');
			}

			if (is_array($default_info_data[$dk1]))
			{
				foreach (array_keys($default_info_data[$dk1]) as $dk2)
				{
					if (empty($info_data[$dk1][$dk2]))
					{
						$info_data[$dk1][$dk2] = $default_info_data[$dk1][$dk2];
					}

					if ($dk1 == 'version_check' && ($dk2 == 'path' || $dk2 == 'filename') && !$info_data[$dk1][$dk2])
					{
						$info_data[$dk1]['domain'] = '';
					}
				}
			}
		}
	}

	function plugin_switching($action, $plugin_id = false)
	{
		global $db;

		$plugin_id = ($plugin_id) ? $plugin_id : $this->plugin_id;

		$sql = 'UPDATE ' . ARCADE_PLUGINS_TABLE . '
				SET plugin_enabled = ' . (($action == 'enable') ? 1 : 0) . '
				WHERE plugin_id = ' . (int) $plugin_id;
		$db->sql_query($sql);
	}

	function plugin_update_check($plugin_data, $plugin_version = false)
	{
		if (!$plugin_version && $this->plugin_id)
		{
			$plugin_version = $plugin_data[$this->plugin_id]['plugin_version'];
			$plugin_data = $this->plugin_ini('install', $plugin_data[$this->plugin_id]['plugin_filename']);
		}

		if (empty($plugin_data['plugin_version']))
		{
			return false;
		}

		return (phpbb_version_compare($plugin_version, $plugin_data['plugin_version'], '<')) ? true : false;
	}

	function obtain_latest_version_info($domain, $path, $file, $force_update = false, $warn_fail = false, $ttl = 86400)
	{
		global $cache;

		$info = $cache->get('_arcade_plugin_' . $this->plugin_id . '_vc');

		if ($info === false || $force_update)
		{
			$errstr = '';
			$errno = 0;

			$info = get_remote_file($domain, $path, $file, $errstr, $errno);

			if ($info === false)
			{
				$cache->destroy('_arcade_plugin_' . $this->plugin_id . '_vc');
				if ($warn_fail)
				{
					trigger_error($errstr . adm_back_link($this->u_action), E_USER_WARNING);
				}
				return false;
			}

			$cache->put('_arcade_plugin_' . $this->plugin_id . '_vc', $info, $ttl);
		}

		return $info;
	}
}

?>