<?php
/**
*
* @package phpBB Arcade install
* @version $Id: arcade_ins_convert_file.php 167 2013-01-02 14:11:03Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_INSTALL') || !defined('IN_PHPBB'))
{
	exit;
}

if (!empty($setmodules))
{
	if (!$this->arcade_version || (phpbb_version_compare($this->arcade_version, $arcade_mod_config['version']['current'], '<')))
	{
		return;
	}

	define('ARCADE_CONVERT_GAME_INSTALL_FILE_LIMIT', 500);

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'CONVERT_FILE',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 50,
		'module_subs'		=> '',
		'module_stages'		=> array('INTRO', 'CONVERT_FILE'),
		'module_reqs'		=> ''
	);
}

class arcade_ins_convert_file extends module
{
	function arcade_ins_convert_file(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $db, $user, $template, $phpbb_root_path, $phpEx;
		global $arcade, $arcade_config, $file_functions, $arcade_install;

		$this->tpl_name = 'install_install';

		switch ($sub)
		{
			case 'intro':
				$this->page_title = 'SUB_INTRO';

				$template->assign_vars(array(
					'U_ACTION'	=> $this->p_master->u_module("mode=$mode&amp;sub=convert_file"),

					'TITLE'		=> $user->lang['CONVERT_GAME_INSTALL_FILE'],
					'BODY'		=> $user->lang['CONVERT_GAME_INSTALL_FILE_BODY'],
					'L_SUBMIT'	=> $user->lang['CONVERT_GAME_INSTALL_FILE_START']
				));
			break;

			case 'convert_file':
				$this->page_title = 'STAGE_CONVERT_FILE';
				$arcade->begin(true, false, false);

				$limit = ARCADE_CONVERT_GAME_INSTALL_FILE_LIMIT;

				$total_games		= (int) request_var('total_games', 0);
				$start				= (int) request_var('start', 0);
				$convert_file		= (int) request_var('convert_file', 0);
				$updated_files		= (int) request_var('updated_files', 0);
				$file_exists		= (int) request_var('file_exists', 0);
				$unwritable_file	= (int) request_var('unwritable_file', 0);

				if (!$total_games)
				{
					$sql = 'SELECT COUNT(game_id) as total_games
							FROM ' . ARCADE_GAMES_TABLE;
					$result = $db->sql_query($sql);
					$total_games = (int) $db->sql_fetchfield('total_games');
					$db->sql_freeresult($result);
				}

				if ($total_games)
				{
					$sql = 'SELECT game_id, game_name, game_name_clean, game_desc, game_control, game_control_desc, game_type, game_width, game_height, game_scoretype, game_image, game_swf, game_scorevar
							FROM ' . ARCADE_GAMES_TABLE . '
							ORDER BY game_name_clean';
					$result = $db->sql_query_limit($sql, $limit, $start);
					while ($row = $db->sql_fetchrow($result))
					{
						$path = $file_functions->remove_extension($row['game_swf']);
						$updated_file = $phpbb_root_path . $arcade_config['game_path'] . $path . '/' . $path . '.' . $phpEx;

						if (file_exists($updated_file))
						{
							@include($updated_file);

							if (isset($game_data['game_control']))
							{
								$updated_files++;
								continue;
							}
						}
						else
						{
							$file_exists++;
							$arcade->add_log('critical', $row['game_id'], 'LOG_ARCADE_GAME_INSTALL_FILE_NOT_FOUND', $row['game_name'], $arcade->root_key($updated_file));
							continue;
						}

						if (Arcade_update_game_install_file(false, $updated_file, $row))
						{
							$convert_file++;
						}
						else
						{
							$unwritable_file++;
							$arcade->add_log('critical', $row['game_id'], 'LOG_ARCADE_GAME_INSTALL_FILE_UNWRITABLE', $row['game_name'], $arcade->root_key($updated_file));
						}
					}
					$db->sql_freeresult($result);

					$start += $limit;

					if ($limit && $start < $total_games && $limit < $total_games)
					{
						$redirect = $this->p_master->u_module("mode=convert_file&amp;sub=convert_file&amp;start=$start&amp;total_games=$total_games&amp;convert_file=$convert_file&amp;updated_files=$updated_files&amp;file_exists=$file_exists&amp;unwritable_file=$unwritable_file");
						$redirect = meta_refresh(3, $redirect);
						$msg_box = '</p><div class="errorbox notice"><p>' . $user->lang['CONVERT_GAME_INSTALL_FILE_EXPLAIN'] . '<br /><br />' . sprintf($user->lang['CONVERT_GAME_INSTALL_FILE_PROCESSING'], $arcade->get_image('full', 'img', 'loading2.gif', 'STAGE_CONVERT_FILE', true), $start, $total_games) . '</p></div><p>';
					}

					if (!isset($msg_box))
					{
						$msg_box  = sprintf($user->lang['ARCADE_TOTAL_INSTALL_GAMES'], $total_games) . '<br /><br />';
						$msg_box .= sprintf($user->lang['CONVERT_TOTAL_GAME_INSTALL_FILES'], $convert_file) . '<br />';
						$msg_box .= sprintf($user->lang['GAME_INSTALL_FILES_ALREADY_CONVERTED'], $updated_files) . '<br />';
						$msg_box .= sprintf($user->lang['GAME_INSTALL_FILES_UNWRITABLE'], $unwritable_file) . '<br />';
						$msg_box .= sprintf($user->lang['GAME_INSTALL_FILES_NOT_FOUND'], $file_exists);

						$msg_box = '</p><div class="successbox"><h3>' . $user->lang['INFORMATION'] . '</h3><p>' . $user->lang['CONVERT_GAME_INSTALL_FILE_END'] . '</p></div><p>' . $msg_box;

						if ($file_exists || $unwritable_file)
						{
							$msg_box .= '<br /><br />' . $arcade_install->color_text($user->lang['GAME_INSTALL_FILE_ERROR_LOG'], 'red');
						}
					}
				}
				else
				{
					$msg_box = '</p><div class="errorbox"><h3>' . $user->lang['INFORMATION'] . '</h3><p>' . $user->lang['ARCADE_NO_GAMES'] . '</p></div><p>';
				}

				$template->assign_vars(array(
					'TITLE'		=> $user->lang['CAT_CONVERT_FILE'],
					'BODY'		=> $msg_box,
					'L_SUBMIT'	=> false
				));
			break;
		}
	}
}

?>