<?php
/**
*
* @package phpBB Arcade install
* @version $Id: arcade_ins_update.php 182 2013-01-06 01:40:35Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_INSTALL') || !defined('IN_PHPBB'))
{
	exit;
}

if (!empty($setmodules))
{
	if (!$this->arcade_version || (phpbb_version_compare($this->arcade_version, $arcade_mod_config['version']['current'], '>=')))
	{
		return;
	}

	if ($this->arcade_auth_sync)
	{
		$arcade_module_stages = array('INTRO', 'REQUIREMENTS', 'ARCADE_AUTH_SYNC', 'UPDATE');
	}
	else
	{
		$arcade_module_stages = array('INTRO', 'REQUIREMENTS', 'UPDATE');
	}

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'UPDATE',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 20,
		'module_subs'		=> '',
		'module_stages'		=> $arcade_module_stages,
		'module_reqs'		=> ''
	);
}

class arcade_ins_update extends module
{
	function arcade_ins_update(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $user, $template, $phpbb_root_path, $phpEx;
		global $arcade_config, $arcade_install, $file_functions;

		$this->tpl_name = 'install_install';

		switch ($sub)
		{
			case 'intro':
				$this->page_title = 'SUB_INTRO';

				// challenge mod check
				$challenge_instructions = '';
				if (!empty($arcade_config['ul_challenge_version']) && $content = get_file_contents("{$phpbb_root_path}includes/constants.{$phpEx}"))
				{
					if (strpos($content, 'includes/challenge/constants') !== false)
					{
						$challenge_instructions .= '</p><div class="errorbox"><h3>' . $user->lang['WARNING'] . '</h3><p>' . $user->lang['CHALLENGE_INSTRUCTIONS'] . '</p></div><p>';
					}
				}

				$template->assign_vars(array(
					'U_ACTION'	=> $this->p_master->u_module("mode=$mode" . (($challenge_instructions == '') ? "&amp;sub=requirements" : '')),

					'TITLE'		=> $user->lang['UPDATE_INTRO'],
					'BODY'		=> $user->lang['UPDATE_INTRO_BODY'] . $challenge_instructions,
					'L_SUBMIT'	=> ($challenge_instructions == '') ? $user->lang['NEXT_STEP'] : false
				));
			break;

			case 'requirements':
				$this->page_title = 'STAGE_REQUIREMENTS';
				$s_hidden_fields = build_hidden_fields(array('arcade' => true));
				$sub = ($this->p_master->arcade_auth_sync) ? 'arcade_auth_sync' : 'update';
				$arcade_install->requirements($mode, $sub, $this->p_master->req, $this->p_master->u_module(), $s_hidden_fields);
			break;

			case 'arcade_auth_sync':
				$this->arcade_auth_sync();
			break;

			case 'update':
				if (isset($_POST['arcade']))
				{
					$this->update($mode, $sub);
				}
			break;
		}
	}

	/**
	* Obtain the information required to connect to the database
	*/
	function update($mode, $sub)
	{
		global $db, $user, $template, $db_tools, $table_prefix;
		global $umil, $arcade_mod_config, $arcade_config, $arcade_install;

		$this->page_title = 'STAGE_UPDATE_ARCADE';

		$db_msg = $arcade_install->def_db($arcade_config['version'], $arcade_mod_config['version']['current']);

		$error_ary = array();
		$errored = false;
		$no_updates = true;
		$sc = $this->schema_changes(false, $db_msg, $no_updates, $errored, $error_ary);
		$updates_to_version = $arcade_mod_config['version']['current'];
		$versions = array_keys($sc);

		switch ($this->p_master->arcade_version)
		{
			case '1.0.0':
			case '2.0.RC1':
				switch ($db->sql_layer)
				{
					case 'sqlite':
					case 'firebird':
						$arcade_install->_sql($db_msg, 'DELETE FROM ' . ARCADE_SESSIONS_TABLE, $errored, $error_ary);
					break;

					default:
						$arcade_install->_sql($db_msg, 'TRUNCATE TABLE ' . ARCADE_SESSIONS_TABLE, $errored, $error_ary);
					break;
				}
				$no_updates = false;
			break;
		}

		for ($i = 0; $i < sizeof($versions); $i++)
		{
			$version = $versions[$i];
			$schema_changes = $sc[$version];

			$next_version = (isset($versions[$i + 1])) ? $versions[$i + 1] : $updates_to_version;

			if (phpbb_version_compare($version, $this->p_master->arcade_version, '<') || phpbb_version_compare($this->p_master->arcade_version, $next_version, '>='))
			{
				continue;
			}

			$this->schema_changes($version, $db_msg, $no_updates, $errored, $error_ary);

			if (!sizeof($schema_changes))
			{
				continue;
			}

			$no_updates = false;

			foreach ($schema_changes as $key => $changes)
			{
				$statements = $db_tools->perform_schema_changes(array($key => $changes));

				foreach ($statements as $sql)
				{
					$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
				}
			}
		}

		$db_msg .= $arcade_install->_write_result($no_updates, $errored, $error_ary);

		$error_ary = array();
		$errored = false;
		$no_updates = true;
		$db_msg .= '<h1><br />' . $user->lang['UPDATING_DATA'] . '</h1>';
		$db_msg .= '<p><br />' . $user->lang['PROGRESS'] . ' :: <br /><strong>';

		for ($i = 0; $i < sizeof($versions); $i++)
		{
			$version = $versions[$i];
			$next_version = (isset($versions[$i + 1])) ? $versions[$i + 1] : $updates_to_version;

			if (phpbb_version_compare($version, $this->p_master->arcade_version, '<') || phpbb_version_compare($this->p_master->arcade_version, $next_version, '>='))
			{
				continue;
			}

			$this->database_changes($version, $db_msg, $no_updates, $errored, $error_ary);
		}

		$db_msg .= $arcade_install->_write_result($no_updates, $errored, $error_ary);

		$error_ary = array();
		$errored = false;
		$no_updates = false;
		$db_msg .= '<h1><br />' . $user->lang['UPDATE_VERSION_OPTIMIZE'] . '</h1>';
		$db_msg .= '<p><br />' . $user->lang['PROGRESS'] . ' :: <br /><strong>';

		// update the version
		$sql = "UPDATE " . ARCADE_CONFIG_TABLE . "
				SET config_value = '$updates_to_version'
				WHERE config_name = 'version'";
		$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);

/*
		// Reset permissions
		$sql = 'UPDATE ' . USERS_TABLE . "
				SET user_permissions = '', user_perm_from = 0";
		$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);

		// Reset Arcade permissions
		$sql = 'UPDATE ' . ARCADE_USERS_TABLE . "
				SET user_arcade_permissions = '', user_arcade_perm_from = 0";
		$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
*/

		$arcade_install->table_optimize(array_keys($arcade_mod_config['schema_data']), $db_msg, $errored, $error_ary);
		$db_msg .= $arcade_install->_write_result($no_updates, $errored, $error_ary);
		$db_msg .= '<h1><br />' . $user->lang['UPDATE_COMPLETED'] . '</h1>';
		$db_msg .= '<p><br />' . $user->lang['INLINE_UPDATE_SUCCESSFUL'] . '<br />';

		// Purge the cache
		$umil->cache_purge(array('template', 'theme', 'imageset', 'auth', ''));

		add_log('admin', 'LOG_ARCADE_UPDATE', $this->p_master->arcade_version, $updates_to_version);

		if (phpbb_version_compare($this->p_master->arcade_version, '2.0.RC4', '<'))
		{
			$perm_msg = '</p><div class="errorbox"><h3>' . $user->lang['WARNING'] . '</h3><p>' . $user->lang['PERMISSIONS_WARNING'] . '</p></div>';
		}
		else
		{
			$perm_msg = '</p>';
		}

		$donate_button = '<br /><a onclick="window.open(this.href); return false;" href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&amp;hosted_button_id=UA2VMFQD6GKFW"><img src="http://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif" alt="' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '" title="' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '" /></a>';
		$donate = '<div class="successbox"><h3>' . $user->lang['PHPBB_ARCADE_DONATE_TITLE'] . '</h3><p>' . $user->lang['PHPBB_ARCADE_DONATE'] . '</p><p>' . $donate_button . '</p></div>';

		$template->assign_vars(array(
			'U_ACTION'	=> $this->p_master->u_module("mode=verify&amp;sub=verify"),

			'TITLE'		=> $user->lang['ACP_ARCADE'] . ' - ' . $user->lang['UPDATING_TO_LATEST_STABLE'],
			'BODY'		=> $user->lang['STAGE_UPDATE_ARCADE_EXPLAIN'] . '<br /><br />' . $perm_msg . $donate . $db_msg,
			'L_SUBMIT'	=> $user->lang['ARCADE_UPDATE_CHECK']
		));
	}

	function schema_changes($version = false, &$db_msg, &$no_updates, &$errored, &$error_ary)
	{
		if (!$version)
		{
			return array(
				'1.0.0' => array(
					'drop_columns'						=> array(
						USERS_TABLE						=> array('user_arcade_permissions', 'user_arcade_perm_from', 'user_arcade_pm', 'games_per_page', 'games_sort_dir', 'games_sort_order'),
						ARCADE_GAMES_TABLE				=> array('game_files'),
					),
					'add_columns'		=> array(
						ARCADE_CONFIG_TABLE				=> array('is_dynamic'	=> array('BOOL', 0)),
					),
					'add_index'			=> array(
						ARCADE_CONFIG_TABLE				=> array(
							'is_dynamic'				=> array('is_dynamic')
						),

						ARCADE_SESSIONS_TABLE			=> array(
							'game_id'					=> array('game_id'),
						),

						ARCADE_PLAYS_TABLE				=> array(
							'uid'						=> array('user_id'),
						),

						ARCADE_GAMES_TABLE				=> array(
							'highuser'					=> array('game_highuser'),
							'installdate'				=> array('game_installdate'),
							'highdate'					=> array('game_highdate'),
							'plays'						=> array('game_plays'),
							'download'					=> array('game_download_total')
						),
					),
					'change_columns'					=> array(
						ARCADE_SESSIONS_TABLE			=> array(
							'randchar1'					=> array('UINT', 0),
							'randchar2'					=> array('UINT', 0),
							'randgid1'					=> array('UINT', 0),
							'randgid2'					=> array('UINT', 0),
						),
					),
				),
				'2.0.RC1' => array(
					'drop_columns'						=> array(
						ARCADE_USERS_TABLE				=> array('user_arcade_pm'),
					),
					'add_columns'						=> array(
						ARCADE_SESSIONS_TABLE			=> array(
							'game_popup'				=> array('TINT:1', 0),
							'microtime'					=> array('TIMESTAMP', 0),
						),
						ARCADE_USERS_TABLE				=> array(
							'user_arcade_options'		=> array('UINT:11', 223),
						),
						ARCADE_GAMES_TABLE				=> array(
							'game_download_cost'		=> array('PDECIMAL:15', 0.00),
						),
						ARCADE_CATS_TABLE				=> array(
							'cat_download_cost'			=> array('PDECIMAL:15', 0.00),
						),
					),
					'add_index'							=> array(
						ARCADE_CATS_TABLE				=> array(
							'status'					=> array('cat_status'),
							'ctest'						=> array('cat_test'),
						),
						ARCADE_SESSIONS_TABLE			=> array(
							'phpbb_sid'					=> array('phpbb_session_id'),
						),
					),
				),
				// No changes
				'2.0.RC2' => array(),
				'2.0.RC3' => array(
					'add_columns'						=> array(
						ARCADE_USERS_BANNED_TABLE		=> array(
							'formtime'					=> array('UINT:11', 0),
						),
						ARCADE_GAMES_TABLE	=> array(
							'game_control'				=> array('TINT:1', 0),
							'game_control_desc'			=> array('TEXT', '')
						),
					),
				),
				'2.0.RC4' => array(
					'drop_keys'		=> array(
						ACL_ARCADE_OPTIONS_TABLE		=> array('au_op'),
						ACL_ARCADE_USERS_TABLE			=> array('autho_id', 'authr_id'),
						ARCADE_GAMES_TABLE				=> array('download'),
						ARCADE_PLAYS_TABLE				=> array('uid'),
						ARCADE_CHALLENGE_TABLE			=> array('ca', 'cid', 'oid', 'gid')
					),
					'add_index'	=> array(
						ACL_ARCADE_USERS_TABLE			=> array(
							'ath_opid'					=> array('auth_option_id'),
							'ath_roid'					=> array('auth_role_id')
						),
						ARCADE_GAMES_TABLE				=> array(
							'downloads'					=> array('game_download_total')
						),
						ARCADE_PLAYS_TABLE				=> array(
							'user_id'					=> array('user_id')
						),
						ARCADE_CHALLENGE_TABLE			=> array(
							'accept'					=> array('challenge_accept'),
							'chall_id'					=> array('challenger_id'),
							'opp_id'					=> array('opponent_id'),
							'game_id'					=> array('game_id'),
						),
					),
					'add_unique_index'	=> array(
						ACL_ARCADE_OPTIONS_TABLE		=> array(
							'ath_op'					=> array('auth_option'),
						),
					),
					'change_columns'					=> array(
						ARCADE_PLUGINS_TABLE			=> array(
							'plugin_page'				=> array('VCHAR:100', '')
						),
						ARCADE_CHALLENGE_TABLE			=> array(
							'challenger_id'				=> array('UINT', 0),
							'opponent_id'				=> array('UINT', 0),
							'game_id'					=> array('UINT', 0)
						),
						ARCADE_CHALLENGE_CHAMP_TABLE	=> array(
							'champ_challenger_id'		=> array('UINT', 0),
							'champ_opponent_id'			=> array('UINT', 0),
							'game_id'					=> array('UINT', 0),
							'champ_winner'				=> array('UINT', 0),
						),
					),
					'drop_columns'						=> array(
						ARCADE_USERS_TABLE				=> array('user_arcade_last_search', 'user_arcade_last_search_term')
					),
					'add_columns'						=> array(
						ARCADE_USERS_TABLE				=> array(
							'games_last_search'			=> array('TIMESTAMP', 0),
							'games_last_search_term'	=> array('VCHAR:255', ''),
						),
					),
				),
			);
		}

		global $user, $db_tools, $table_prefix;
		global $arcade_install, $arcade_mod_config;

		switch ($version)
		{
			case '1.0.0':
				$statements = $db_tools->sql_create_table(ARCADE_USERS_TABLE, $arcade_mod_config['schema_data'][ARCADE_USERS_TABLE]);

				foreach ($statements as $sql)
				{
					$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
				}

				$no_updates = false;
			break;

			case '2.0.RC1':
				foreach ($arcade_mod_config['schema_data'] as $table_name => $table_data)
				{
					if (!in_array($table_name, array(ARCADE_USERS_BANNED_TABLE, ARCADE_ANNOUNCE_TABLE)))
					{
						continue;
					}

					$statements = $db_tools->sql_create_table($table_name, $table_data);

					foreach ($statements as $sql)
					{
						$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
					}
				}
				$no_updates = false;
			break;

			case '2.0.RC2':
				//$no_updates = false;
			break;

			case '2.0.RC3':
				$statements = $db_tools->sql_table_drop("{$table_prefix}arcade_errors");

				foreach ($statements as $sql)
				{
					$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
				}

				foreach ($arcade_mod_config['schema_data'] as $table_name => $table_data)
				{
					if (!in_array($table_name, array(ARCADE_LOGS_TABLE, ARCADE_SUPER_CHAMP_TABLE, ARCADE_DELETE_GAMES_TABLE, ARCADE_PLUGINS_TABLE)))
					{
						continue;
					}

					$statements = $db_tools->sql_create_table($table_name, $table_data);

					foreach ($statements as $sql)
					{
						$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
					}
				}
				$no_updates = false;
			break;

			case '2.0.RC4':
				$arcade_install->_sql($db_msg, 'INSERT INTO ' .  ARCADE_CONFIG_TABLE . " (config_name, config_value, is_dynamic) VALUES ('cat_style', 0, 0)", $errored, $error_ary);
				$arcade_install->_sql($db_msg, 'INSERT INTO ' .  ARCADE_CONFIG_TABLE . " (config_name, config_value, is_dynamic) VALUES ('download_per_day', 0, 0)", $errored, $error_ary);
				$arcade_install->_sql($db_msg, 'INSERT INTO ' .  ARCADE_CONFIG_TABLE . " (config_name, config_value, is_dynamic) VALUES ('announce_author', '" . $user->data['username'] . "', 0)", $errored, $error_ary);

				$no_updates = false;
			break;
		}
	}

	function database_changes($version, &$db_msg, &$no_updates, &$errored, &$error_ary)
	{
		global $db, $db_tools, $table_prefix;
		global $umil, $arcade_config, $arcade_install, $arcade_mod_config;

		switch ($version)
		{
			case '1.0.0':
				$sql = 'SELECT game_installdate
						FROM ' . ARCADE_GAMES_TABLE . '
						ORDER BY game_installdate ASC';
				$result = $db->sql_query_limit($sql, 1);
				$startdate = (int) $db->sql_fetchfield('game_installdate');
				$db->sql_freeresult($result);

				$startdate = ($startdate) ? $startdate : time();

				$sql = 'SELECT SUM(total_plays) AS games_played, SUM(total_time) AS games_time
						FROM ' . ARCADE_PLAYS_TABLE;
				$result = $db->sql_query($sql);
				$play_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$sql = 'SELECT SUM(total) as downloads
						FROM ' . ARCADE_DOWNLOAD_TABLE;
				$result = $db->sql_query($sql);
				$total_downloads = (int) $db->sql_fetchfield('downloads');
				$db->sql_freeresult($result);

				if ($db->sql_layer === 'sqlite')
				{
					$sql = 'SELECT COUNT(user_id) as total_users_played
							FROM (
								SELECT DISTINCT user_id
								FROM ' . ARCADE_PLAYS_TABLE . '
							)';
				}
				else
				{
					$sql = 'SELECT COUNT(DISTINCT user_id) as total_users_played
							FROM ' . ARCADE_PLAYS_TABLE;
				}

				$result = $db->sql_query($sql);
				$total_users_played = (int) $db->sql_fetchfield('total_users_played');
				$db->sql_freeresult($result);

				$sql = 'SELECT COUNT(report_id) as reports_open
						FROM ' . ARCADE_REPORTS_TABLE . '
						WHERE report_closed = ' . ARCADE_REPORT_OPEN;
				$result = $db->sql_query($sql);
				$reports_open = (int) $db->sql_fetchfield('reports_open');
				$db->sql_freeresult($result);

				$total_plays = (int) $play_data['games_played'];
				$total_times = (int) $play_data['games_time'];

				// Lets move the old data over...
				$db->sql_return_on_error(true);
				$sql = 'SELECT user_id, user_arcade_permissions, user_arcade_perm_from, user_arcade_pm, games_per_page, games_sort_dir, games_sort_order
						FROM ' . USERS_TABLE . '
						ORDER BY user_id';
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					// We will only move the data over if the user changed the default values.
					// The other users will be created when they first visit the arcade.
					if ($row['user_arcade_pm'] != true || $row['games_per_page'] != 0 || $row['games_sort_dir'] != 'a' || $row['games_sort_order'] != 'n')
					{
						$sql_ary = array(
							'user_id'						=> $row['user_id'],
							'user_arcade_permissions'		=> $row['user_arcade_permissions'],
							'user_arcade_perm_from'			=> $row['user_arcade_perm_from'],
							'user_arcade_pm'				=> $row['user_arcade_pm'],
							'games_per_page'				=> $row['games_per_page'],
							'games_sort_dir'				=> $row['games_sort_dir'],
							'games_sort_order'				=> $row['games_sort_order']
						);

						$db->sql_query('INSERT INTO ' . ARCADE_USERS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary));
					}
				}
				$db->sql_freeresult($result);
				$db->sql_return_on_error(false);

				$arcade_configs = array(array('arcade_startdate', $startdate),
										array('arcade_resetdate', '0'),
										array('backup_limit', '50'),
										array('cat_backup_path', 'arcade/backup/'),
										array('download_anonymous_interval', '0'),
										array('download_interval', '0'),
										array('download_on_demand', '1'),
										array('facebook_font_type', 'arial'),
										array('founder_exempt', '0'),
										array('game_popup_icon_enabled', '1'),
										array('load_list', '0'),
										array('playfree_reward', '0'),
										array('play_anonymous_interval', '0'),
										array('play_interval', '0'),
										array('reports_open', $reports_open),
										array('search_anonymous_interval', '0'),
										array('search_filter_age', '1'),
										array('search_filter_password', '1'),
										array('search_interval', '0'),
										array('session_length', '72000'),
										array('total_downloads', $total_downloads, true),
										array('total_plays', $total_plays, true),
										array('total_plays_time', $total_times, true),
										array('flash_version', '10.0.0.0'),
				);

				$umil->arcade_config_add($arcade_configs);

				$delete_arcade_configs = array(
					array('pm_subject'),
					array('pm_message'),
				);
				// Remove unneeded config values
				$umil->arcade_config_remove($delete_arcade_configs);

				$umil->permission_add(array('a_arcade', 'a_arcade_points_settings', 'a_arcade_backup'));

				$umil->arcade_permission_add(array(
					'c_ignoreflood_download', 'c_ignoreflood_play', 'c_re_rate',
					'u_', 'u_arcade', 'u_arcade_favorites', 'u_arcade_ignoreflood_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats'
				));

				$umil->arcade_permission_set(array(
					array('ROLE_ARCADE_STANDARD', array('c_re_rate')),
					array('ROLE_ARCADE_FULL'	, array('c_ignoreflood_download', 'c_ignoreflood_play', 'c_re_rate'))
				));

				$no_updates = false;
			break;

			case '2.0.RC1':
				// update challenge
				if (!empty($arcade_config['ul_challenge_version']))
				{
					$sql = 'SELECT SUM(champ_challenger_time + champ_opponent_time) AS total_times, COUNT(champ_id) AS total_plays
							FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
							WHERE champ_close = ' . CHALLENGE_CLOSE;
					$result = $db->sql_query($sql);
					$total = $db->sql_fetchrow($result);
					$db->sql_freeresult($result);

					$arcade_configs = array(
						array('challenge_acc_exp_date'		, $arcade_config['challenge_time_delete']),
						array('challenge_disable'			, (($arcade_config['challenge_enable']) ? 0 : 1)),
						array('challenge_latest_winners'	, $arcade_config['challenge_first_user']),
						array('challenge_links_index'		, $arcade_config['challenge_index_link']),
						array('challenge_links_stats'		, $arcade_config['challenge_stat_link']),
						array('challenge_ong_exp_date'		, $arcade_config['challenge_time_delete_ong']),
						array('challenge_bet_Minimum'		, $arcade_config['challenge_points_min']),
						array('challenge_bet_Maximum'		, $arcade_config['challenge_points_max']),
						array('challenge_bet_fix'			, $arcade_config['challenge_points_fix']),
						array('challenge_users_select'		, $arcade_config['challenge_user_select']),
						array('challenge_top_games_header'	, $arcade_config['challenge_top_games']),
						array('challenge_resetdate'			, (($arcade_config['challenge_reset_time']) ? $arcade_config['challenge_reset_time'] : $arcade_config['challenge_install_time'])),
						array('challenge_total_plays'		, (int) $total['total_plays'], true),
						array('challenge_total_plays_time'	, (int) $total['total_times'], true),
						array('challenge_welcome_index'		, $arcade_config['challenge_index_top']),
						array('challenge_welcome_stats'		, $arcade_config['challenge_stat_top']),
					);

					$delete_arcade_configs = array(
						array('challenge_index_search'),
						array('challenge_stat_search'),
						array('challenge_statement_search'),
						array('challenge_par_reset_user'),
						array('challenge_v_good'),
						array('challenge_flash_menu'),
						array('challenge_flash_width'),
						array('challenge_flash_height'),
						array('challenge_copyright'),
						array('ul_challenge_version'),
						array('challenge_flash_navig'),
						array('challenge_index_user_play'),
						array('challenge_stat_user_play'),
						array('challenge_statement_user_play'),
						array('challenge_install_time'),
						array('challenge_update_time'),
						array('challenge_reset_user'),
						array('challenge_par_reset_time'),
						array('challenge_random_enable'),
						array('challenge_anonym'),
						array('challenge_cstt_enable'),
						array('challenge_astt_enable'),
						array('challenge_stat_enable'),
						array('challenge_points_enable'),
						array('challenge_stat_user'),
						array('challenge_stat_row'),
						array('challenge_index_link'),
						array('challenge_stat_top'),
						array('challenge_index_top'),
						array('challenge_stat_link'),
						array('challenge_statement_top'),
						array('challenge_statement_link'),
						array('challenge_first_user'),
						array('challenge_points_min'),
						array('challenge_points_max'),
						array('challenge_points_fix'),
						array('challenge_reset_time'),
						array('challenge_enable'),
						array('challenge_time_delete'),
						array('challenge_time_delete_ong'),
						array('challenge_favorite_enable'),
						array('challenge_friends_enable'),
						array('challenge_top_games')
					);

					$arcade_install->remove_config($delete_arcade_configs);

					$umil->arcade_permission_remove(array('c_challenge_opcio','c_challenge_play','c_challenge_play_treat','c_challenge_report','c_challenge_perm', 'c_popup', 'c_resolution'));
					$umil->permission_remove(array('a_challenge_config', 'a_challenge_reset', 'a_challenge_report'));

					$umil->module_remove(array(
						array('acp', 'ACP_ARCADE_CHALLENGE'	, 'ACP_ARCADE_CHALLENGE_MAIN'),
						array('acp', 'ACP_ARCADE_CHALLENGE'	, 'ACP_ARCADE_CHALLENGE_CONFIG'),
						array('acp', 'ACP_ARCADE_CHALLENGE'	, 'ACP_ARCADE_CHALLENGE_PAGE_CONFIG'),
						array('acp', 'ACP_ARCADE_CHALLENGE'	, 'ACP_ARCADE_CHALLENGE_REPORT'),
						array('acp', 'ACP_CAT_ARCADE'		, 'ACP_ARCADE_CHALLENGE'),
						array('ucp', 'UCP_CAT_ARCADE'		, 'UCP_ARCADE_CHALLENGE')
					));

					$statements = $db_tools->sql_table_drop("{$table_prefix}arcade_challenge_report");

					foreach ($statements as $sql)
					{
						$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
					}

					$statements = $db_tools->sql_table_drop("{$table_prefix}arcade_challenge_stats");

					foreach ($statements as $sql)
					{
						$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
					}

					$sql = 'SELECT champ_game_id
							FROM ' . ARCADE_CHALLENGE_CHAMP_TABLE;
					$result = $db->sql_query($sql);
					while ($row = $db->sql_fetchrow($result))
					{
						$sql = 'UPDATE ' . ARCADE_CHALLENGE_CHAMP_TABLE . '
								SET game_id = ' . (int) $row['champ_game_id'] . '
								WHERE champ_game_id = ' . (int) $row['champ_game_id'];
						$db->sql_query($sql);
					}
					$db->sql_freeresult($result);

					$challenge_schema_changes = array(
						'drop_columns'			=> array(
							USERS_TABLE						=> array('user_challenge_on', 'user_challenge_pm_on', 'user_challenge_stat'),
							ARCADE_CHALLENGE_CHAMP_TABLE	=> array('champ_game_id', 'champ_points'),
						),
						'add_columns'						=> array(
							ARCADE_CHALLENGE_TABLE			=> array(
								'challenger_game_cost'		=> array('PDECIMAL:25', 0.000),
								'opponent_game_cost'		=> array('PDECIMAL:25', 0.000),
							),
							ARCADE_CHALLENGE_CHAMP_TABLE	=> array(
								'game_id'					=> array('UINT', NULL),
							),
						),
						'add_index'					=> array(
							ARCADE_CHALLENGE_TABLE	=> array(
								'ca'		=> array('challenge_accept'),
								'cid'		=> array('challenger_id'),
								'oid'		=> array('opponent_id'),
								'gid'		=> array('game_id'),
							),
							ARCADE_CHALLENGE_CHAMP_TABLE	=> array(
								'ci'		=> array('champ_challenger_id'),
								'oi'		=> array('champ_opponent_id'),
								'cg'		=> array('game_id'),
								'cw'		=> array('champ_winner'),
								'cc'		=> array('champ_close'),
							),
						),
					);

					// Start challenge schema changes
					foreach ($challenge_schema_changes as $key => $changes)
					{
						$statements = $db_tools->perform_schema_changes(array($key => $changes));

						foreach ($statements as $sql)
						{
							$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
						}
					}
					// End challenge schema changes
				}
				else
				{
					foreach ($arcade_mod_config['schema_data'] as $table_name => $table_data)
					{
						if (strpos($table_name, 'challenge') === false)
						{
							continue;
						}

						$statements = $db_tools->sql_create_table($table_name, $table_data);

						foreach ($statements as $sql)
						{
							$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);
						}
					}

					$arcade_configs = array(
						array('challenge_acc_exp_date', '604800'),
						array('challenge_disable', '0'),
						array('challenge_latest_winners', '5'),
						array('challenge_links_index', '1'),
						array('challenge_links_stats', '1'),
						array('challenge_ong_exp_date', '1209600'),
						array('challenge_bet_Minimum', '0'),
						array('challenge_bet_Maximum', '0'),
						array('challenge_bet_fix', '0'),
						array('challenge_users_select', '1'),
						array('challenge_resetdate', time()),
						array('challenge_top_games_header', '10'),
						array('challenge_total_plays', '0', true),
						array('challenge_total_plays_time', '0', true),
						array('challenge_welcome_index', '1'),
						array('challenge_welcome_stats', '1'),
					);
				}

				$delete_arcade_configs = array(
						array('jackpot_maximum'),
						array('jackpot_minimum'),
						array('game_popup_icon_enabled'),
						array('facebook_table_height'),
						array('facebook_table_width'),
						array('facebook_enable_like'),
						array('facebook_show_faces'),
						array('facebook_layout_style'),
						array('facebook_color_scheme'),
						array('facebook_background_color'),
						array('facebook_font_type')
				);

				$umil->arcade_config_remove($delete_arcade_configs);

				$arcade_configs = array_merge($arcade_configs, array(
					array('arcade_flash_intro', '1'),
					array('cat_leaders', '5'),
					array('cat_least_downloaded', '5'),
					array('cat_least_popular', '5'),
					array('cat_longest_held_scores', '5'),
					array('cat_most_downloaded', '5'),
					array('cat_most_popular', '5'),
					array('cat_least_played_users', '5'),
					array('cat_most_played_users', '5'),
					array('challenge_exp_last_gc', time(), true),
					array('challenge_least_played_users', '5'),
					array('challenge_disable_msg', ''),
					array('challenge_most_played_users', '5'),
					array('challenge_leaders', '5'),
					array('challenge_leaders_header', '3'),
					array('challenge_least_popular', '5'),
					array('challenge_most_popular', '5'),
					array('corrupt_score_banned', '0'),
					array('display_game_image', '1'),
					array('display_user_avatar', '1'),
					array('display_game_popup_icon', ((isset($arcade_config['game_popup_icon_enabled'])) ? $arcade_config['game_popup_icon_enabled'] : '1')),
					array('download_file_type', ''),
					array('enable_image_link', '1'),
					array('enable_user_avatar_link', '1'),
					array('gamename_maxchars', '12'),
					array('game_comment', '1'),
					array('game_desc_parse_html', '0'),
					array('game_download_cost', '0'),
					array('global_announce_enable', '0'),
					array('gc_time', '3600'),
					array('jackpot_Maximum', ((isset($arcade_config['jackpot_maximum'])) ? $arcade_config['jackpot_maximum'] : '0')),
					array('jackpot_Minimum', ((isset($arcade_config['jackpot_minimum'])) ? $arcade_config['jackpot_minimum'] : '0')),
					array('least_played_users', '5'),
					array('least_jackpot', '10'),			// *****
					array('least_users_playtime', '10'),	// *****
					array('most_played_users', '5'),
					array('most_jackpot', '10'),			// *****
					array('most_users_playtime', '10'),		// *****
					array('protect_phpbbarcade', '1'),
					array('record_playing_date', time(), true),
					array('record_playing_users', '0', true),
					array('score_security_level', '2'),
					array('session_last_gc', time(), true),
					array('smilies_popup_height', 350),
					array('smilies_popup_width', 500)
				));

				$umil->arcade_config_add($arcade_configs);

				$options = array(
					array('u_favorites'			, 'u_arcade_favorites'),
					array('u_ignoreflood_search', 'u_arcade_ignoreflood_search'),
					array('u_view_whoisplaying'	, 'u_arcade_view_whoisplaying'),
					array('u_viewstats'			, 'u_arcade_viewstats')
				);

				$arcade_install->arcade_permissions_rename($options);

				$umil->arcade_permission_remove(array('c_popup', 'c_resolution'));
				$umil->permission_remove(array('a_arcade_scores'));

				$umil->arcade_permission_add(array(
					'c_challenge_playing_games', 'c_challenge_start_game_warning', 'c_downloadfree',
					'u_arcade_challenge', 'u_arcade_download', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search'
				));

				$umil->arcade_permission_set(array(
					array('ROLE_ARCADE_FULL'			, array('c_challenge_playing_games', 'c_challenge_start_game_warning', 'c_downloadfree')),
					array('ROLE_ARCADE_USER_FULL'		, array('u_arcade_challenge', 'u_arcade_download', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search')),
					array('ROLE_ARCADE_USER_STANDARD'	, array('u_arcade_challenge', 'u_arcade_download', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search')),
					array('ROLE_ARCADE_USER_LIMITED'	, array('u_arcade_challenge', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution'))
				));

				$umil->permission_add('a_arcade_user');

				$umil->permission_set(array(
					array('ROLE_ADMIN_FULL'				, array('a_arcade_user')),
					array('ROLE_ADMIN_STANDARD'			, array('a_arcade_user'))
				));

				$sql_array = array(
					'user_arcade_options'			=> 220,
					'games_per_page'				=> 0,
					'games_sort_dir'				=> 'd',
					'games_sort_order'				=> 'd'
				);

				$sql = 'UPDATE ' . ARCADE_USERS_TABLE . '
						SET ' . $db->sql_build_array('UPDATE', $sql_array) . '
						WHERE user_id = ' . ANONYMOUS;
				$arcade_install->_sql($db_msg, $sql, $errored, $error_ary);

				$arcade_install->add_bbcodes($arcade_mod_config['bbcodes']);

				$no_updates = false;
			break;

			case '2.0.RC2':
				$umil->config_add($arcade_mod_config['phpbb_config']);

				$no_updates = false;
			break;

			case '2.0.RC3':
				$arcade_configs = array(
					array('arcade_flash_highscores', 10),
					array('arcade_flash_logo', ''),
					array('ban_user_ip', 1),
					array('challenge_info_box', 1),
					array('champion_proces_limit', 250),
					array('display_game_control', 1),
					array('score_form_token_lifetime', 15),
					array('send_arcade_challenge_pm', 1),
					array('super_champion', 1),
					array('super_champion_avatar', 1),
					array('super_champion_avatar_width', 50),
					array('super_champion_avatar_height', 50)
				);

				$umil->arcade_config_add($arcade_configs);

				$umil->arcade_permission_remove(array(/*'u_',*/ 'u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_ignoreflood_search', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats'));

				$umil->permission_add(array(
					'a_arcade_plugin', 'a_arcade_plugin_remove', 'a_arcade_viewlogs', 'a_arcade_clearlogs',
					'u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_ignoreflood_search', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats'
				));

				$umil->permission_set(array(
					array('ROLE_ADMIN_FULL'		, array('a_arcade_plugin', 'a_arcade_plugin_remove', 'a_arcade_viewlogs', 'a_arcade_clearlogs')),
					array('ROLE_ADMIN_STANDARD'	, array('a_arcade_plugin', 'a_arcade_plugin_remove', 'a_arcade_viewlogs', 'a_arcade_clearlogs')),
					array('ROLE_ADMIN_FORUM'	, array('a_arcade', 'a_cauth', 'a_arcade_settings', 'a_arcade_game', 'a_arcade_cat', 'a_arcade_plugin', 'a_arcade_user', 'a_arcade_utilities', 'a_arcade_viewlogs')),
					array('ROLE_ADMIN_USERGROUP', array('a_arcade', 'a_cauth', 'a_arcade_settings', 'a_arcade_game', 'a_arcade_cat', 'a_arcade_plugin', 'a_arcade_user', 'a_arcade_utilities', 'a_arcade_viewlogs')),
					array('ROLE_USER_FULL'		, array('u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_ignoreflood_search', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats')),
					array('ROLE_USER_STANDARD'	, array('u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_ignoreflood_search', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats')),
					array('ROLE_USER_LIMITED'	, array('u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats')),
					array('ROLE_USER_NOPM'		, array('u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats')),
					array('ROLE_USER_NOAVATAR'	, array('u_arcade', 'u_arcade_challenge', 'u_arcade_download', 'u_arcade_favorites', 'u_arcade_popup', 'u_arcade_pm', 'u_arcade_resolution', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats')),
					array('ROLE_USER_NEW_MEMBER', array('u_arcade', 'u_arcade_challenge', 'u_arcade_pm', 'u_arcade_search', 'u_arcade_view_whoisplaying', 'u_arcade_viewstats'))
				));

				$umil->arcade_permission_role_remove(array('ROLE_ARCADE_USER_NOACCESS', 'ROLE_ARCADE_USER_FULL', 'ROLE_ARCADE_USER_LIMITED', 'ROLE_ARCADE_USER_STANDARD'));

				$umil->module_remove(array_reverse($arcade_mod_config['modules']));
				$umil->module_remove(array('acp', 'ACP_CAT_USERS', 'ACP_USER_ARCADE'));
				$umil->module_add($arcade_mod_config['modules']);

				// Delete Arcade config
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'share_icons_enable'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'facebook_like_enable'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'google_like_enable'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'tweet_like_enable'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'like_counter_enable'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'like_background_color'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CONFIG_TABLE . " WHERE config_name = 'protect_ibpro'", $errored, $error_ary);

				$no_updates = false;
			break;

			case '2.0.RC4':
				// Start purge anonymous user data
				$tables = array(ACL_ARCADE_USERS_TABLE, ARCADE_ACCESS_TABLE, ARCADE_DOWNLOAD_TABLE, ARCADE_FAVS_TABLE, ARCADE_PLAYS_TABLE, ARCADE_RATING_TABLE, ARCADE_SCORES_TABLE, ARCADE_REPORTS_TABLE, ARCADE_USERS_BANNED_TABLE, ARCADE_SUPER_CHAMP_TABLE);

				foreach ($tables as $table)
				{
					$arcade_install->_sql($db_msg, "DELETE FROM " . $table . " WHERE user_id = " . ANONYMOUS, $errored, $error_ary);
				}

				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CHALLENGE_TABLE . " WHERE challenger_id = " . ANONYMOUS . " OR opponent_id = " . ANONYMOUS, $errored, $error_ary);
				$arcade_install->_sql($db_msg, "DELETE FROM " . ARCADE_CHALLENGE_CHAMP_TABLE . " WHERE champ_challenger_id = " . ANONYMOUS . " OR champ_opponent_id = " . ANONYMOUS, $errored, $error_ary);
				// End purge anonymous user data

				$arcade_install->_sql($db_msg, "UPDATE " . ARCADE_CONFIG_TABLE . " SET config_value = '{$arcade_mod_config['copyright']}' WHERE config_name = 'copyright'", $errored, $error_ary);
				$arcade_install->_sql($db_msg, "UPDATE " . ARCADE_CONFIG_TABLE . " SET config_value = '15' WHERE config_name = 'score_form_token_lifetime' AND config_value < 15", $errored, $error_ary);

				$no_updates = false;
			break;
		}
	}

	function arcade_auth_sync()
	{
		global $db, $user, $template;
		global $auth_arcade;

		$this->page_title = 'STAGE_ARCADE_AUTH_SYNC';

		$sql = 'SELECT user_id
				FROM ' . USERS_TABLE;
		$result = $db->sql_query($sql);
		$user_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$user_ids[] = $row['user_id'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT group_id
				FROM ' . GROUPS_TABLE;
		$result = $db->sql_query($sql);
		$group_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$group_ids[] = $row['group_id'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT cat_id
				FROM ' . ARCADE_CATS_TABLE;
		$result = $db->sql_query($sql);
		$cat_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$cat_ids[] = $row['cat_id'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT auth_option_id
				FROM ' . ACL_ARCADE_OPTIONS_TABLE;
		$result = $db->sql_query($sql);
		$auth_option_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$auth_option_ids[] = $row['auth_option_id'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT role_id
				FROM ' . ACL_ARCADE_ROLES_TABLE;
		$result = $db->sql_query($sql);
		$auth_roles_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$auth_roles_ids[] = $row['role_id'];
		}
		$db->sql_freeresult($result);

		$cat_ids = array_map('intval', $cat_ids);
		$user_ids = array_map('intval', $user_ids);
		$group_ids = array_map('intval', $group_ids);
		$auth_roles_ids = array_map('intval', $auth_roles_ids);
		$auth_option_ids = array_map('intval', $auth_option_ids);

		$sql = 'DELETE FROM ' . ACL_ARCADE_GROUPS_TABLE . '
				WHERE	(' . $db->sql_in_set('group_id', $group_ids, true) . ')
					OR
						(' . $db->sql_in_set('cat_id', $cat_ids, true, true) . ' AND cat_id <> 0)
					OR
						(' . $db->sql_in_set('auth_role_id', $auth_roles_ids, true, true) . ' AND auth_role_id <> 0)
					OR
						(' . $db->sql_in_set('auth_option_id', $auth_option_ids, true, true) . ' AND auth_option_id <> 0)';
		$db->sql_query($sql);

		$sql = 'DELETE FROM ' . ACL_ARCADE_USERS_TABLE . '
				WHERE	(' . $db->sql_in_set('user_id', $user_ids, true) . ')
					OR
						(' . $db->sql_in_set('cat_id', $cat_ids, true, true) . ' AND cat_id <> 0)
					OR
						(' . $db->sql_in_set('auth_role_id', $auth_roles_ids, true, true) . ' AND auth_role_id <> 0)
					OR
						(' . $db->sql_in_set('auth_option_id', $auth_option_ids, true, true) . ' AND auth_option_id <> 0)';
		$db->sql_query($sql);

		$sql = 'DELETE FROM ' . ACL_ARCADE_ROLES_DATA_TABLE . '
				WHERE	' . $db->sql_in_set('auth_option_id', $auth_option_ids, true, true) . ' OR
						' . $db->sql_in_set('role_id', $auth_roles_ids, true, true);
		$db->sql_query($sql);

		$auth_arcade->acl_clear_prefetch();

		$template->assign_vars(array(
			'S_HIDDEN'	=> build_hidden_fields(array('arcade' => true)),
			'U_ACTION'	=> $this->p_master->u_module('mode=update&sub=update'),

			'TITLE'		=> $user->lang['STAGE_ARCADE_AUTH_SYNC'],
			'BODY'		=> '</p><div class="successbox"><p>' . $user->lang['STAGE_ARCADE_AUTH_SYNC_SUCCESS'] . '</p></div><p>',
			'L_SUBMIT'	=> $user->lang['ARCADE_NEXT_DB_UPDATE']
		));
	}
}

?>