<?php
/**
*
* @package phpBB Arcade install
* @version $Id: arcade_ins_verify.php 170 2013-01-02 15:11:35Z killbill $
* @copyright (c) 2008-2011 JRSweets - jrsweets@gmail.com
* @copyright (c) 2011-2013 http://phpbbarcade.jatek-vilag.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_INSTALL') || !defined('IN_PHPBB'))
{
	exit;
}

if (!empty($setmodules))
{
	if (!$this->arcade_version || phpbb_version_compare($this->arcade_version, $arcade_mod_config['version']['current'], '<>'))
	{
		return;
	}

	$module[] = array(
		'module_type'		=> 'install',
		'module_title'		=> 'VERIFY',
		'module_filename'	=> substr(basename(__FILE__), 0, -strlen($phpEx)-1),
		'module_order'		=> 40,
		'module_subs'		=> '',
		'module_stages'		=> array('INTRO', 'VERIFY'),
		'module_reqs'		=> ''
	);
}

class arcade_ins_verify extends module
{
	function arcade_ins_verify(&$p_master)
	{
		$this->p_master = &$p_master;
	}

	function main($mode, $sub)
	{
		global $db, $user, $template, $phpbb_root_path, $phpEx;
		global $umil, $arcade, $arcade_install, $arcade_mod_config, $file_functions;

		$this->tpl_name = 'install_install';

		switch ($sub)
		{
			case 'intro':
				$this->page_title = 'SUB_INTRO';

				$template->assign_vars(array(
					'U_ACTION'	=> $this->p_master->u_module("mode=$mode&amp;sub=verify"),

					'TITLE'		=> $user->lang['VERIFY_INTRO'],
					'BODY'		=> $user->lang['VERIFY_INTRO_BODY'],
					'L_SUBMIT'	=> $user->lang['NEXT_STEP']
				));
			break;

			case 'verify':
				$this->page_title = 'STAGE_VERIFY';

				$passed = $arcade_install->requirements($mode, $mode, $this->p_master->req, $this->p_master->u_module());

				// Check files exist
				$error = array();
				$this->p_master->file_not_exist_check('core', $arcade_mod_config['verify']['files']['core'], $error);

				// We will only check for files from the languages that are installed
				$installed_langs = $arcade_install->installed_languages();

				foreach ($installed_langs as $lang_path)
				{
					$this->p_master->file_not_exist_check('lang', $arcade_mod_config['verify']['files']['langs'], $error, $lang_path);
				}

				// We will only check for files from the styles that are installed
				$installed_styles = $arcade_install->installed_styles();

				foreach ($installed_styles as $style_id => $row)
				{
					$tip = ($row['template_inherits_id'] && $row['template_inherit_path']) ? true : false;
					$template_path = ($tip) ? $row['template_inherit_path'] : $row['template_path'];

					if ($tip)
					{
						$this->p_master->file_not_exist_check('template', $arcade_mod_config['verify']['files']['styles']['template']['other'], $error, $row['template_path']);
					}

					$this->p_master->file_not_exist_check('template', $arcade_mod_config['verify']['files']['styles']['template']['default'], $error, $template_path);
					$this->p_master->file_not_exist_check('theme', $arcade_mod_config['verify']['files']['styles']['theme'], $error, $row['theme_path'], true, $arcade_install->style_bitfield($row['bbcode_bitfield']));
				}

				if (sizeof($error))
				{
					$passed['mod'] = false;
					$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_MISSING_FILES'], $arcade_install->color_text(implode('<br />', $error), 'black')), 'red');
				}
				else
				{
					$result = $arcade_install->color_text($user->lang['VERIFY_ALL_FILES']);
				}

				$data = array('VERIFY_FILES_EXIST' => $result);

				$error = array();
				foreach ($installed_styles as $style_id => $row)
				{
					//$template_path = ($row['template_inherits_id'] && $row['template_inherit_path']) ? $row['template_inherit_path'] : $row['template_path'];
					$file_path = $phpbb_root_path . 'styles/' . $row['template_path'] . '/template/arcade/arcadelist_body.html';

					if (file_exists($file_path))
					{
						if ($content = get_file_contents($file_path))
						{
							if (strpos($content, $arcade_mod_config['verify']['files']['new_html_key']) === false)
							{
								$msg = $arcade->root_key($phpbb_root_path . 'styles/' . $row['template_path'] . '/template/arcade/');

								if (!in_array($msg, $error))
								{
									$error[] = $msg;
								}
							}
						}
					}
				}

				if (sizeof($error))
				{
					$passed['mod'] = false;
					$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_FOLDER_NOT_UPDATE'], $arcade_install->color_text(implode('<br />', $error), 'black')), 'red');
					$data = array_merge($data, array('VERIFY_FOLDER_UPDATE' => $result));
				}

				// Check if old files exist
				$error = array();
				$result = false;
				$this->p_master->file_exist_check('core', $arcade_mod_config['verify']['old_files']['core'], $error);

				foreach ($installed_styles as $style_id => $row)
				{
					$tip = ($row['template_inherits_id'] && $row['template_inherit_path']) ? true : false;
					$template_path = ($tip) ? $row['template_inherit_path'] : $row['template_path'];

					if ($tip)
					{
						$this->p_master->file_exist_check('template', $arcade_mod_config['verify']['old_files']['styles']['template']['other'], $error, $row['template_path']);
					}

					$this->p_master->file_exist_check('template', $arcade_mod_config['verify']['old_files']['styles']['template']['default'], $error, $template_path);
					$this->p_master->file_exist_check('theme', $arcade_mod_config['verify']['old_files']['styles']['theme'], $error, $row['theme_path']);
				}

				foreach ($installed_langs as $lang_path)
				{
					$this->p_master->file_exist_check('lang', $arcade_mod_config['verify']['old_files']['langs'], $error, $lang_path);
				}

				if (sizeof($error))
				{
					$passed['mod'] = false;
					$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_OLD_FILES_PRESENT'], $arcade_install->color_text(implode('<br />', $error), 'black')), 'red');
				}

				if ($result)
				{
					$data = array_merge($data, array('VERIFY_OLD_FILES' => $result));
				}

				// Check files edits exist
				$error = array();
				$this->p_master->file_edit_check('core', $arcade_mod_config['verify']['edits']['core'], $error);

		/*
				foreach ($installed_langs as $lang_path)
				{
					$this->p_master->file_edit_check('lang', $arcade_mod_config['verify']['edits']['langs'], $error, $lang_path);
				}
		*/
				foreach ($installed_styles as $style_id => $row)
				{
					$template_path = ($row['template_inherits_id'] && $row['template_inherit_path']) ? $row['template_inherit_path'] : $row['template_path'];

					if (isset($arcade_mod_config['verify']['edits']['styles'][$template_path]['template']))
					{
						$other = false;
						$path = $arcade_mod_config['verify']['edits']['styles'][$template_path]['template'];
					}
					else
					{
						$other	= $template_path;
						$path	= $arcade_mod_config['verify']['edits']['styles']['other']['template'];
					}

					$this->p_master->file_edit_check('template', $path, $error, $other, true, $arcade_install->style_bitfield($row['bbcode_bitfield']), true, $row['template_path']);

					if (isset($arcade_mod_config['verify']['edits']['styles'][$row['theme_path']]['theme']))
					{
						$other = false;
						$path = $arcade_mod_config['verify']['edits']['styles'][$row['theme_path']]['template'];
					}
					else
					{
						$other = $row['theme_path'];
						$path = $arcade_mod_config['verify']['edits']['styles']['other']['theme'];
					}

					$this->p_master->file_edit_check('theme', $path, $error, $other, true, $arcade_install->style_bitfield($row['bbcode_bitfield']), true);
				}

				if (sizeof($error))
				{
					$passed['mod'] = false;
					$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_MISSING_FILES_EDITED'], '<span style="color:red; font-weight:normal;">' . implode('<br />', $error) . '</span>'), 'black');
				}
				else
				{
					$result = $arcade_install->color_text($user->lang['VERIFY_ALL_FILES_EDITED']);
				}

				$data = array_merge($data, array('VERIFY_FILES_EDITED' => $result));

				// Check if tables exist
				$error = array();
				$result = $arcade_install->verify_tables($arcade_mod_config['schema_data'], $error);

				if (sizeof($error))
				{
					$passed['mod'] = false;
				}

				$data = array_merge($data, array('VERIFY_TABLES_EXIST' => $result));

				// Check other db data
				$error = array();
				$arcade_install->verify_alter_table('old', $arcade_mod_config['verify']['alter_db']['old'], $error);

				if (sizeof($error))
				{
					$data = array_merge($data, array('VERIFY_OLD_OTHER_DB_DATA' => implode('<br />', $error)));
				}

				// Check if modules are present
				$error = array();
				$module_type_ary = array('acp', 'mcp', 'ucp');

				foreach ($module_type_ary as $module_type)
				{
					foreach ($arcade_mod_config['verify']['modules'][$module_type] as $module_langname)
					{
						if (!$umil->module_exists($module_type, false, $module_langname))
						{
							$error[] = $arcade->lang_value($module_langname);
						}
					}
				}

				if (sizeof($error))
				{
					$passed['mod'] = false;
					$result = $arcade_install->color_text(sprintf($user->lang['VERIFY_MISSING_MODULES'], $arcade_install->color_text(implode('<br />', $error), 'black')), 'red');
				}
				else
				{
					$result = $arcade_install->color_text($user->lang['VERIFY_ALL_MODULES']);
				}

				$data = array_merge($data, array('VERIFY_MODULES' => $result));

				$error = array();
				$result = $arcade_install->verify_permission($arcade_mod_config['permission']['phpbb'], $error, 'phpbb');

				if (sizeof($error))
				{
					$passed['mod'] = false;
				}

				$data = array_merge($data, array('VERIFY_PERMISSIONS_PHPBB' => $result));

				$error = array();
				$result = $arcade_install->verify_permission($arcade_mod_config['permission']['arcade'], $error);

				if (sizeof($error))
				{
					$passed['mod'] = false;
				}

				$data = array_merge($data, array('VERIFY_PERMISSIONS_ARCADE' => $result));

				// Check if phpbb configs exist
				$error = array();
				$result = $arcade_install->verify_config($arcade_mod_config['phpbb_config'], $error, 'phpbb');

				if (sizeof($error))
				{
					$passed['mod'] = false;
				}

				$data = array_merge($data, array('VERIFY_PHPBB_CONFIGS' => $result));

				// Check if arcade configs exist
				$error = array();
				$result = $arcade_install->verify_config($arcade_mod_config['config_data'], $error);

				if (sizeof($error))
				{
					$passed['mod'] = false;
				}

				unset($error);

				$template->assign_block_vars('checks', array(
					'S_LEGEND'		=> true,

					'LEGEND'		=> $user->lang['ARCADE_INSTALL_CHECK'],
					'LEGEND_EXPLAIN'=> $user->lang['VERIFY_ARCADE_INSTALLATION_EXPLAIN']
				));

				$data = array_merge($data, array('VERIFY_ARCADE_CONFIGS' => $result));

				foreach ($data as $title => $result)
				{
					$template->assign_block_vars('checks', array(
						'TITLE'		=> $user->lang[$title],
						'RESULT'	=> $result,

						'S_EXPLAIN'	=> false,
						'S_LEGEND'	=> false
					));
				}

				$title			= (!in_array(false, $passed)) ? $user->lang['INSTALL_CONGRATS'] : $user->lang['VERIFY_ERRORS'];
				$body			= (!in_array(false, $passed)) ? sprintf($user->lang['VERIFY_CONGRATS_EXPLAIN'], $arcade_mod_config['version']['current']) : sprintf($user->lang['VERIFY_ERRORS_EXPLAIN'], $arcade_mod_config['version']['current']);
				$url			= (!in_array(false, $passed)) ? append_sid("{$phpbb_root_path}adm/index.$phpEx", 'i=arcade_main', true, $user->session_id) : $this->p_master->u_module("mode=$mode&amp;sub=verify");
				$submit			= (!in_array(false, $passed)) ? $user->lang['INSTALL_LOGIN'] : $user->lang['INSTALL_TEST'];

				if (!in_array(false, $passed))
				{
					$umil->cache_purge(array('template', 'theme', 'imageset', 'auth', ''));
				}

				$template->assign_vars(array(
					'U_ACTION'	=> $url,

					'TITLE'		=> $title,
					'BODY'		=> $body,
					'L_SUBMIT'	=> $submit
				));
			break;
		}
	}
}

?>